Package: xtable
Version: 1.8-8
Date: 2026-02-20
Title: Export Tables to LaTeX or HTML
Authors@R: c(person("David B.", "Dahl", role="aut"),
             person("David", "Scott", role=c("aut","cre"),
               email="d.scott@auckland.ac.nz"),
             person("Charles", "Roosen", role="aut"),
             person("Arni", "Magnusson", role="aut"),
             person("Jonathan", "Swinton", role="aut"),
             person("Ajay", "Shah", role="ctb"),
             person("Arne", "Henningsen", role="ctb"),
             person("Benno", "Puetz", role="ctb"),
             person("Bernhard", "Pfaff", role="ctb"),
             person("Claudio", "Agostinelli", role="ctb"),
             person("Claudius", "Loehnert", role="ctb"),
             person("David", "Mitchell", role="ctb"),
             person("David", "Whiting", role="ctb"),
             person("Fernando da", "Rosa", role="ctb"),
             person("Guido", "Gay", role="ctb"),
             person("Guido", "Schulz", role="ctb"),
             person("Ian", "Fellows", role="ctb"),
             person("Jeff", "Laake", role="ctb"),
             person("John", "Walker", role="ctb"),
             person("Jun", "Yan", role="ctb"),
             person("Liviu", "Andronic", role="ctb"),
             person("Markus", "Loecher", role="ctb"),
             person("Martin", "Gubri", role="ctb"),
             person("Matthieu", "Stigler", role="ctb"),
             person("Robert", "Castelo", role="ctb"),
             person("Seth", "Falcon", role="ctb"),
             person("Stefan", "Edwards", role="ctb"),
             person("Sven", "Garbade", role="ctb"),
             person("Uwe", "Ligges", role="ctb"))
Maintainer: David Scott <d.scott@auckland.ac.nz>
Imports: stats, utils, methods
Suggests: knitr, zoo, survival, glue, tinytex
VignetteBuilder: knitr
Description: Coerce data to LaTeX and HTML tables.
URL: http://xtable.r-forge.r-project.org/
Depends: R (>= 2.10.0)
License: GPL (>= 2)
Repository: CRAN
NeedsCompilation: no
Packaged: 2026-02-20 02:51:53 UTC; dsco0
Author: David B. Dahl [aut],
  David Scott [aut, cre],
  Charles Roosen [aut],
  Arni Magnusson [aut],
  Jonathan Swinton [aut],
  Ajay Shah [ctb],
  Arne Henningsen [ctb],
  Benno Puetz [ctb],
  Bernhard Pfaff [ctb],
  Claudio Agostinelli [ctb],
  Claudius Loehnert [ctb],
  David Mitchell [ctb],
  David Whiting [ctb],
  Fernando da Rosa [ctb],
  Guido Gay [ctb],
  Guido Schulz [ctb],
  Ian Fellows [ctb],
  Jeff Laake [ctb],
  John Walker [ctb],
  Jun Yan [ctb],
  Liviu Andronic [ctb],
  Markus Loecher [ctb],
  Martin Gubri [ctb],
  Matthieu Stigler [ctb],
  Robert Castelo [ctb],
  Seth Falcon [ctb],
  Stefan Edwards [ctb],
  Sven Garbade [ctb],
  Uwe Ligges [ctb]
Date/Publication: 2026-02-22 11:20:02 UTC
Built: R 4.6.0; ; 2026-04-25 15:39:58 UTC; unix

importFrom("methods", "is")
importFrom("utils", "toLatex")
importFrom("stats", "anova", "as.ts", "cycle", "end", "frequency",
           "na.omit", "pchisq", "start", "time")
importFrom("utils", "packageDescription")
export("caption<-", "caption", "label", "label<-",
       "align<-", "align", "digits<-", "digits", "display<-",
       "display", "xtable",
       "xtableMatharray","xtableList", "xtableLSMeans",
       "print.xtable", "print.xtableMatharray", "print.xtableList",
       "xtableFtable", "print.xtableFtable",
       "toLatex.xtable",
       "autoformat", "xalign", "xdigits", "xdisplay",
       "sanitize", "sanitize.numbers", "sanitize.final", "as.is", "as.math")

S3method("print", "xtable")
S3method("print", "xtableMatharray")
S3method("print", "xtableList")
S3method("print", "xtableFtable")
S3method("toLatex", "xtable")

S3method("caption<-", "xtable")
S3method("caption", "xtable")
S3method("label", "xtable")
S3method("label<-", "xtable")
S3method("align<-", "xtable")
S3method("align", "xtable")
S3method("digits", "xtable")
S3method("digits<-", "xtable")
S3method("display<-", "xtable")
S3method("display", "xtable")

S3method("xtable", "data.frame")
S3method("xtable", "matrix")
S3method("xtable", "table")
S3method("xtable", "anova")
S3method("xtable", "aov")
S3method("xtable", "summary.aov")
S3method("xtable", "summary.aovlist")
S3method("xtable", "aovlist")
S3method("xtable", "lm")
S3method("xtable", "summary.lm")
S3method("xtable", "glm")
S3method("xtable", "summary.glm")
S3method("xtable", "prcomp")
S3method("xtable", "summary.prcomp")
S3method("xtable", "coxph")
S3method("xtable", "ts")
S3method("xtable", "zoo")
## S3method("xtable", "sarlm")
## S3method("xtable", "summary.sarlm")
## S3method("xtable", "gmsar")
## S3method("xtable", "summary.gmsar")
## S3method("xtable", "stsls")
## S3method("xtable", "summary.stsls")
## S3method("xtable", "sarlm.pred")
## S3method("xtable", "lagImpact")
## S3method("xtable", "splm")
## S3method("xtable", "summary.splm")
## S3method("xtable", "sphet")
## S3method("xtable", "summary.sphet")
## S3method("xtable", "spautolm")
## S3method("xtable", "summary.spautolm")

## [doc on github.io](https://walkerever.github.io/)

# xtable

print console tables.  xtable serves as both a class and a command line tool.

- [installation](#installation)
- [use the class](#use-the-class)
- [command line](#use-the-command-line) from CSV, preformatted table, JSON list to console table, JSON, markdown and more.
- [convert table data to Markdown](#markdown)

----


## Installation
    `pip install xtable`
   run it through xtable` or `python -mxtable`

----

below is a simple introduction for some basic usage.

## use the class
<pre><font color="#5FD7FF">from</font> xtable <font color="#5FD7FF">import</font> xtable

data = [
        [<font color="#C061CB">1</font>,<font color="#C061CB">2</font>,<font color="#C061CB">3</font>,<font color="#C061CB">4</font>,<font color="#C061CB">5</font>],
        [<font color="#C061CB">11</font>,<font color="#C061CB">52</font>,<font color="#C061CB">3</font>,<font color="#C061CB">4</font>,<font color="#C061CB">5</font>],
        [<font color="#C061CB">11</font>,<font color="#33C7DE"><b>None</b></font>,<font color="#C061CB">3</font>,<font color="#C061CB">4</font>,<font color="#C061CB">5</font>],
        [<font color="#C061CB">11</font>,<font color="#33C7DE"><b>None</b></font>,<font color="#C061CB">3</font>,<font color="#33C7DE"><b>None</b></font>,<font color="#C061CB">5</font>],
        ]
hdr = [<font color="#C061CB">&apos;c1&apos;</font>,<font color="#C061CB">&apos;c2&apos;</font>,<font color="#C061CB">&apos;c3&apos;</font>,<font color="#C061CB">&apos;c4&apos;</font>,<font color="#C061CB">&apos;c5&apos;</font>]

xt = xtable(data=data,header=hdr)
<font color="#33C7DE"><b>print</b></font>(xt)

</pre>

// result : 
<pre>c1 c2 c3 c4 c5
--------------
1  2  3  4  5
11 52 3  4  5
11    3  4  5
11    3     5
</pre>
-----

<pre><font color="#33C7DE"># test/t2.csv</font>
<font color="#C061CB">&apos;&apos;&apos;</font>
<font color="#C061CB">h1,h2,h3</font>
<font color="#C061CB">&quot;asd&quot;,&quot;sdfsadf&quot;,1</font>
<font color="#C061CB">&quot;c&quot;,&quot;cc&quot;,233</font>
<font color="#C061CB">&apos;&apos;&apos;</font>

xt = xtable.init_from_csv(<font color="#C061CB">&quot;test/t2.csv&quot;</font>)
<font color="#33C7DE"><b>print</b></font>(xt)
</pre>

// result :
<pre>h1  h2      h3
---------------
asd sdfsadf 1
c   cc      233
</pre>

-----
<pre>
data = [
        {<font color="#C061CB">&quot;h1&quot;</font>:<font color="#C061CB">&quot;v1&quot;</font>,<font color="#C061CB">&quot;h2&quot;</font>:<font color="#C061CB">&quot;v2&quot;</font>,<font color="#C061CB">&quot;h3&quot;</font>:<font color="#C061CB">&quot;v3&quot;</font>},
        {<font color="#C061CB">&quot;h1&quot;</font>:<font color="#C061CB">&quot;v11&quot;</font>,<font color="#C061CB">&quot;h2&quot;</font>:<font color="#C061CB">&quot;v22&quot;</font>,<font color="#C061CB">&quot;h3&quot;</font>:<font color="#C061CB">&quot;v33&quot;</font>},
        {<font color="#C061CB">&quot;h1&quot;</font>:<font color="#C061CB">&quot;v11111&quot;</font>,<font color="#C061CB">&quot;h2&quot;</font>:<font color="#C061CB">&quot;v22222&quot;</font>,<font color="#C061CB">&quot;h3&quot;</font>:<font color="#C061CB">&quot;v34444&quot;</font>},
     ]
xt = xtable.init_from_list(data)
<font color="#33C7DE"><b>print</b></font>(xt)</pre>

// result :
<pre>
h1     h2     h3
--------------------
v1     v2     v3
v11    v22    v33
v11111 v22222 v34444

</pre>

-----

// all of them support "xheader".

<pre>data = [
        {<font color="#C061CB">&quot;h1&quot;</font>:<font color="#C061CB">&quot;v1&quot;</font>,<font color="#C061CB">&quot;h2&quot;</font>:<font color="#C061CB">&quot;v2&quot;</font>,<font color="#C061CB">&quot;h3&quot;</font>:<font color="#C061CB">&quot;v3&quot;</font>},
        {<font color="#C061CB">&quot;h1&quot;</font>:<font color="#C061CB">&quot;v11&quot;</font>,<font color="#C061CB">&quot;h2&quot;</font>:<font color="#C061CB">&quot;v22&quot;</font>,<font color="#C061CB">&quot;h3&quot;</font>:<font color="#C061CB">&quot;v33&quot;</font>},
        {<font color="#C061CB">&quot;h1&quot;</font>:<font color="#C061CB">&quot;v11111&quot;</font>,<font color="#C061CB">&quot;h2&quot;</font>:<font color="#C061CB">&quot;v22222&quot;</font>,<font color="#C061CB">&quot;h3&quot;</font>:<font color="#C061CB">&quot;v34444&quot;</font>},
     ]
xt = xtable.init_from_list(data, xheader=[<font color="#C061CB">&quot;h1&quot;</font>,<font color="#C061CB">&quot;h3&quot;</font>])
<font color="#33C7DE"><b>print</b></font>(xt)

xt2 = xtable.init_from_list(data, xheader=<font color="#C061CB">&quot;h2,h3&quot;</font>)
<font color="#33C7DE"><b>print</b></font>(xt2)

</pre>

<pre>h1     h3
-------------
v1     v3
v11    v33
v11111 v34444

h2     h3
-------------
v2     v3
v22    v33
v22222 v34444
</pre>

-----

// output json/yaml/csv/html

<pre>

data = [
        {<font color="#C061CB">&quot;h1&quot;</font>:<font color="#C061CB">&quot;v1&quot;</font>,<font color="#C061CB">&quot;h2&quot;</font>:<font color="#C061CB">&quot;v2&quot;</font>,<font color="#C061CB">&quot;h3&quot;</font>:<font color="#C061CB">&quot;v3&quot;</font>},
        {<font color="#C061CB">&quot;h1&quot;</font>:<font color="#C061CB">&quot;v11&quot;</font>,<font color="#C061CB">&quot;h2&quot;</font>:<font color="#C061CB">&quot;v22&quot;</font>,<font color="#C061CB">&quot;h3&quot;</font>:<font color="#C061CB">&quot;v33&quot;</font>},
        {<font color="#C061CB">&quot;h1&quot;</font>:<font color="#C061CB">&quot;v11111&quot;</font>,<font color="#C061CB">&quot;h2&quot;</font>:<font color="#C061CB">&quot;v22222&quot;</font>,<font color="#C061CB">&quot;h3&quot;</font>:<font color="#C061CB">&quot;v34444&quot;</font>},
     ]
xt = xtable.init_from_list(data, xheader=[<font color="#C061CB">&quot;h1&quot;</font>,<font color="#C061CB">&quot;h3&quot;</font>])
<font color="#33C7DE"><b>print</b></font>(xt.csv())
<font color="#33C7DE"><b>print</b></font>(xt.yaml())
<font color="#33C7DE"><b>print</b></font>(xt.json())
<font color="#33C7DE"><b>print</b></font>(xt.html())

</pre>

// result
<pre>- h1: v1
  h3: v3
- h1: v11
  h3: v33
- h1: v11111
  h3: v34444

[
  {
    &quot;h1&quot;: &quot;v1&quot;,
    &quot;h3&quot;: &quot;v3&quot;
  },
  {
    &quot;h1&quot;: &quot;v11&quot;,
    &quot;h3&quot;: &quot;v33&quot;
  },
  {
    &quot;h1&quot;: &quot;v11111&quot;,
    &quot;h3&quot;: &quot;v34444&quot;
  }
]
&lt;table border=1 style=&quot;border-collapse:collapse;&quot;&gt;
&lt;tr&gt;
&lt;td&gt;&lt;b&gt;h1&lt;/b&gt;&lt;/td&gt;
&lt;td&gt;&lt;b&gt;h3&lt;/b&gt;&lt;/td&gt;
&lt;/tr&gt;
&lt;tr&gt;
&lt;td&gt;v1&lt;/td&gt;
&lt;td&gt;v3&lt;/td&gt;
&lt;/tr&gt;
&lt;tr&gt;
&lt;td&gt;v11&lt;/td&gt;
&lt;td&gt;v33&lt;/td&gt;
&lt;/tr&gt;
&lt;tr&gt;
&lt;td&gt;v11111&lt;/td&gt;
&lt;td&gt;v34444&lt;/td&gt;
&lt;/tr&gt;
</pre>

-----

## use the command line

<pre>[yonghang@mtp xtable]$ cat test/t1.txt  
a b c
121 1212 12
12 332  2323
[yonghang@mtp xtable]$ cat test/t1.txt  | xtable 
a   b    c
-------------
121 1212 12
12  332  2323
[yonghang@mtp xtable]$ 
[yonghang@mtp xtable]$ cat test/t2.csv 
h1,h2,h3
&quot;asd&quot;,&quot;sdfsadf&quot;,1
&quot;c&quot;,&quot;cc&quot;,233
[yonghang@mtp xtable]$ 
[yonghang@mtp xtable]$ cat test/t2.csv  | xtable -b&quot;,&quot;
h1    h2        h3
-------------------
&quot;asd&quot; &quot;sdfsadf&quot; 1
&quot;c&quot;   &quot;cc&quot;      233</pre>

-----

<pre>[yonghang@mtp xtable]$ cat test/t3.json  | qic
[
  {
    <font color="#008700"><b>&quot;userId&quot;</b></font>: <font color="#626262">1</font>,
    <font color="#008700"><b>&quot;firstName&quot;</b></font>: <font color="#AF0000">&quot;Krish&quot;</font>,
    <font color="#008700"><b>&quot;lastName&quot;</b></font>: <font color="#AF0000">&quot;Lee&quot;</font>,
    <font color="#008700"><b>&quot;phoneNumber&quot;</b></font>: <font color="#AF0000">&quot;123456&quot;</font>,
    <font color="#008700"><b>&quot;emailAddress&quot;</b></font>: <font color="#AF0000">&quot;krish.lee@learningcontainer.com&quot;</font>
  },
  {
    <font color="#008700"><b>&quot;userId&quot;</b></font>: <font color="#626262">2</font>,
    <font color="#008700"><b>&quot;firstName&quot;</b></font>: <font color="#AF0000">&quot;racks&quot;</font>,
    <font color="#008700"><b>&quot;lastName&quot;</b></font>: <font color="#AF0000">&quot;jacson&quot;</font>,
    <font color="#008700"><b>&quot;phoneNumber&quot;</b></font>: <font color="#AF0000">&quot;123456&quot;</font>,
    <font color="#008700"><b>&quot;emailAddress&quot;</b></font>: <font color="#AF0000">&quot;racks.jacson@learningcontainer.com&quot;</font>
  },
  {
    <font color="#008700"><b>&quot;userId&quot;</b></font>: <font color="#626262">3</font>,
    <font color="#008700"><b>&quot;firstName&quot;</b></font>: <font color="#AF0000">&quot;denial&quot;</font>,
    <font color="#008700"><b>&quot;lastName&quot;</b></font>: <font color="#AF0000">&quot;roast&quot;</font>,
    <font color="#008700"><b>&quot;phoneNumber&quot;</b></font>: <font color="#AF0000">&quot;33333333&quot;</font>,
    <font color="#008700"><b>&quot;emailAddress&quot;</b></font>: <font color="#AF0000">&quot;denial.roast@learningcontainer.com&quot;</font>
  },
  {
    <font color="#008700"><b>&quot;userId&quot;</b></font>: <font color="#626262">4</font>,
    <font color="#008700"><b>&quot;firstName&quot;</b></font>: <font color="#AF0000">&quot;devid&quot;</font>,
    <font color="#008700"><b>&quot;lastName&quot;</b></font>: <font color="#AF0000">&quot;neo&quot;</font>,
    <font color="#008700"><b>&quot;phoneNumber&quot;</b></font>: <font color="#AF0000">&quot;222222222&quot;</font>,
    <font color="#008700"><b>&quot;emailAddress&quot;</b></font>: <font color="#AF0000">&quot;devid.neo@learningcontainer.com&quot;</font>
  },
  {
    <font color="#008700"><b>&quot;userId&quot;</b></font>: <font color="#626262">5</font>,
    <font color="#008700"><b>&quot;firstName&quot;</b></font>: <font color="#AF0000">&quot;jone&quot;</font>,
    <font color="#008700"><b>&quot;lastName&quot;</b></font>: <font color="#AF0000">&quot;mac&quot;</font>,
    <font color="#008700"><b>&quot;phoneNumber&quot;</b></font>: <font color="#AF0000">&quot;111111111&quot;</font>,
    <font color="#008700"><b>&quot;emailAddress&quot;</b></font>: <font color="#AF0000">&quot;jone.mac@learningcontainer.com&quot;</font>
  }
]

[yonghang@mtp xtable]$ cat test/t3.json  | xtable
userId firstName lastName phoneNumber emailAddress
------------------------------------------------------------------------
1      Krish     Lee      123456      krish.lee@learningcontainer.com
2      racks     jacson   123456      racks.jacson@learningcontainer.com
3      denial    roast    33333333    denial.roast@learningcontainer.com
4      devid     neo      222222222   devid.neo@learningcontainer.com
5      jone      mac      111111111   jone.mac@learningcontainer.com
</pre>

-----
### pivot 
<pre>[yonghang@mtp xtable]$ cat test/t3.json  | xtable -v
userId       : 1
firstName    : Krish
lastName     : Lee
phoneNumber  : 123456
emailAddress : krish.lee@learningcontainer.com
--
userId       : 2
firstName    : racks
lastName     : jacson
phoneNumber  : 123456
emailAddress : racks.jacson@learningcontainer.com
--
userId       : 3
firstName    : denial
lastName     : roast
phoneNumber  : 33333333
emailAddress : denial.roast@learningcontainer.com
--
userId       : 4
firstName    : devid
lastName     : neo
phoneNumber  : 222222222
emailAddress : devid.neo@learningcontainer.com
--
userId       : 5
firstName    : jone
lastName     : mac
phoneNumber  : 111111111
emailAddress : jone.mac@learningcontainer.com
</pre>

if we look at the source code, we know class xtable support .pivot() as well,

<pre>        <font color="#E9AD0C">if</font> <font color="#33C7DE"><b>type</b></font>(js) <font color="#E9AD0C">is</font> <font color="#33C7DE"><b>list</b></font> :
            xt = xtable.init_from_json(js,args.header)
            <font color="#E9AD0C">if</font> args.pivot :
                <font color="#33C7DE"><b>print</b></font>(xt.pivot<span style="background-color:#2AA1B3">()</span>)
            <font color="#E9AD0C">else</font> :
                <font color="#33C7DE"><b>print</b></font>(xt)
</pre>


-----
### when header has space...
some command output is already a table, only not that decent, eg. as below. xtable will help a little bit.  `-c` told xtable that `container id` is together while `-t` told xtable the input stream is already in "table" format.

<pre>[yonghang@mtp xtable]$ sudo podman ps --all
CONTAINER ID  IMAGE                           COMMAND               CREATED             STATUS                         PORTS   NAMES
eeb5db3c4f9a  docker.io/library/nginx:latest  nginx -g daemon o...  About a minute ago  Exited (0) About a minute ago          romantic_hamilton
5ef267563b44  docker.io/library/httpd:latest  httpd-foreground      55 seconds ago      Exited (0) 6 seconds ago               sad_lamarr
[yonghang@mtp xtable]$ 
[yonghang@mtp xtable]$ 
[yonghang@mtp xtable]$ sudo podman ps --all | xtable -c &quot;CONTAINER ID&quot; -t
CONTAINER ID IMAGE                          COMMAND              CREATED            STATUS                        PORTS NAMES
----------------------------------------------------------------------------------------------------------------------------------------
eeb5db3c4f9a docker.io/library/nginx:latest nginx -g daemon o... About a minute ago Exited (0) About a minute ago       romantic_hamilto
5ef267563b44 docker.io/library/httpd:latest httpd-foreground     About a minute ago Exited (0) 28 seconds ago           sad_lamar
[yonghang@mtp xtable]$ 
[yonghang@mtp xtable]$ sudo podman ps --all | xtable -c &quot;CONTAINER ID&quot; -tv
CONTAINER ID : eeb5db3c4f9a
IMAGE        : docker.io/library/nginx:latest
COMMAND      : nginx -g daemon o...
CREATED      : About a minute ago
STATUS       : Exited (0) About a minute ago
PORTS        : 
NAMES        : romantic_hamilto
--
CONTAINER ID : 5ef267563b44
IMAGE        : docker.io/library/httpd:latest
COMMAND      : httpd-foreground
CREATED      : About a minute ago
STATUS       : Exited (0) 32 seconds ago
PORTS        : 
NAMES        : sad_lamar
</pre>

### xtable take care of lines

<pre>[yonghang@W5202860 test]$ cat sql.json  | qic
[
  {
    <font color="#008700"><b>&quot;name&quot;</b></font>: <font color="#AF0000">&quot;sql example 1&quot;</font>,
    <font color="#008700"><b>&quot;query&quot;</b></font>: <font color="#AF0000">&quot;SELECT QUARTER, REGION, SUM(SALES)\n FROM SALESTABLE\n GROUP BY CUBE (QUARTER, REGION)&quot;</font>
  },
  {
    <font color="#008700"><b>&quot;name&quot;</b></font>: <font color="#AF0000">&quot;sql example 2&quot;</font>,
    <font color="#008700"><b>&quot;query&quot;</b></font>: <font color="#AF0000">&quot;select name, cast(text as varchar(8000)) \nfrom SYSIBM.SYSVIEWS \n where name=&apos;your table name&apos; &quot;</font>
  },
  {
    <font color="#008700"><b>&quot;name&quot;</b></font>: <font color="#AF0000">&quot;sql example 3&quot;</font>,
    <font color="#008700"><b>&quot;query&quot;</b></font>: <font color="#AF0000">&quot;select Id, max(V1),max(V2),max(V3) from \n (\n select ID,Value V1,&apos;&apos; V2,&apos;&apos; V3 from A where Code=1 \n union all \n select ID,&apos;&apos; V1, Value V2,&apos;&apos; V3 from A where Code=2 \n union all \n select ID,&apos;&apos; V1, &apos;&apos; V2,Value V3 from A where Code=3 \n) AG\n group by ID&quot;</font>
  }
]

[yonghang@W5202860 test]$ 
[yonghang@W5202860 test]$ cat sql.json  | xtable
name          query
-------------------------------------------------------------------
sql example 1 SELECT QUARTER, REGION, SUM(SALES)
               FROM SALESTABLE
               GROUP BY CUBE (QUARTER, REGION)
sql example 2 select name, cast(text as varchar(8000))
              from SYSIBM.SYSVIEWS
               where name=&apos;your table name&apos;
sql example 3 select Id, max(V1),max(V2),max(V3) from
               (
               select ID,Value V1,&apos;&apos; V2,&apos;&apos; V3 from A where Code=1
               union all
               select ID,&apos;&apos; V1, Value V2,&apos;&apos; V3 from A where Code=2
               union all
               select ID,&apos;&apos; V1, &apos;&apos; V2,Value V3 from A where Code=3
              ) AG
               group by ID
</pre>


## markdown

```
yonghang@air xtable % cat test/t2.csv
h1,h2,h3
"asd","sdfsadf",1
"c","cc",233

yonghang@air xtable % cat test/t2.csv | xtable -F md
|  h1,h2,h3           |
|  -----------------  |
|  "asd","sdfsadf",1  |
|  "c","cc",233       |
yonghang@air xtable %
yonghang@air xtable % cat test/t1.txt
a b c
121 1212 12
12 332  2323
yonghang@air xtable % cat test/t1.txt | xtable -F md
|  a    | b    | c     |
|  ---- | ---- | ----  |
|  121  | 1212 | 12    |
|  12   | 332  | 2323  |
yonghang@air xtable %
yonghang@air xtable % cat test/t3.json | qic
[
  {
    "userId": 1,
    "firstName": "Krish",
    "lastName": "Lee",
    "phoneNumber": "123456",
    "emailAddress": "krish.lee@learningcontainer.com"
  },
  {
    "userId": 2,
    "firstName": "racks",
    "lastName": "jacson",
    "phoneNumber": "123456",
    "emailAddress": "racks.jacson@learningcontainer.com"
  },
  {
    "userId": 3,
    "firstName": "denial",
    "lastName": "roast",
    "phoneNumber": "33333333",
    "emailAddress": "denial.roast@learningcontainer.com"
  },
  {
    "userId": 4,
    "firstName": "devid",
    "lastName": "neo",
    "phoneNumber": "222222222",
    "emailAddress": "devid.neo@learningcontainer.com"
  },
  {
    "userId": 5,
    "firstName": "jone",
    "lastName": "mac",
    "phoneNumber": "111111111",
    "emailAddress": "jone.mac@learningcontainer.com"
  }
]

yonghang@air xtable %
yonghang@air xtable % cat test/t3.json | xtable -F md
|  userId | firstName | lastName | phoneNumber | emailAddress                        |
|  ------ | --------- | -------- | ----------- | ----------------------------------  |
|       1 | Krish     | Lee      | 123456      | krish.lee@learningcontainer.com     |
|       2 | racks     | jacson   | 123456      | racks.jacson@learningcontainer.com  |
|       3 | denial    | roast    | 33333333    | denial.roast@learningcontainer.com  |
|       4 | devid     | neo      | 222222222   | devid.neo@learningcontainer.com     |
|       5 | jone      | mac      | 111111111   | jone.mac@learningcontainer.com      |
yonghang@air xtable %
```

## ----set, include=FALSE-------------------------------------------------------
library(knitr)
opts_chunk$set(fig.path = 'Figures/other', debug = TRUE, echo = TRUE)
opts_chunk$set(out.width = '0.9\\textwidth')

## ----package, results='asis'------------------------------
library(xtable)
options(xtable.floating = FALSE)
options(xtable.timestamp = "")
options(width = 60)
set.seed(1234)

## ----zoo, results = 'asis'--------------------------------
library(zoo)
xDate <- as.Date("2003-02-01") + c(1, 3, 7, 9, 14) - 1
as.ts(xDate)
x <- zoo(rnorm(5), xDate)
xtable(x)

## ----zoots, results = 'asis'------------------------------
tempTs <- ts(cumsum(1 + round(rnorm(100), 0)),
              start = c(1954, 7), frequency = 12)
tempTable <- xtable(tempTs, digits = 0)
tempTable
tempZoo <- as.zoo(tempTs)
xtable(tempZoo, digits = 0)

## ----survival, results = 'asis'---------------------------
library(survival)
test1 <- list(time=c(4,3,1,1,2,2,3),
              status=c(1,1,1,0,1,1,0),
              x=c(0,2,1,1,1,0,0),
              sex=c(0,0,0,0,1,1,1))
coxFit <- coxph(Surv(time, status) ~ x + strata(sex), test1)
xtable(coxFit)

### Test code for xtableFtable function
### David Scott, <d.scott@auckland.ac.nz>, 2016-01-14
library(xtable)


tbl <- ftable(mtcars$cyl, mtcars$vs, mtcars$am, mtcars$gear, row.vars = c(2, 4),
              dnn = c("Cylinders", "V/S", "Transmission", "Gears"))

## debug(xtableFtable)
xftbl <- xtableFtable(tbl)
str(xftbl)
unclass(xftbl)
print.xtableFtable(xftbl)
xftbl <- xtableFtable(tbl, method = "row.compact")
print.xtableFtable(xftbl)
xftbl <- xtableFtable(tbl, method = "col.compact")
print.xtableFtable(xftbl)
xftbl <- xtableFtable(tbl, method = "compact")
print.xtableFtable(xftbl)
## debug(print.xtableFtable)
## undebug(print.xtableFtable)
## debug(print.xtable)
## undebug(print.xtable)

## ----set, include=FALSE-----------------------------------
library(knitr)
opts_chunk$set(fig.path='Figures/list', debug=TRUE, echo=TRUE)
opts_chunk$set(out.width='0.9\\textwidth')

## ----package, results='asis'------------------------------
library(xtable)
options(xtable.floating = FALSE)
options(xtable.timestamp = "")
options(width = 60)

## ----data-------------------------------------------------
require(xtable)
data(mtcars)
mtcars <- mtcars[, 1:6]
mtcarsList <- split(mtcars, f = mtcars$cyl)
### Reduce the size of the list elements
mtcarsList[[1]] <- mtcarsList[[1]][1,]
mtcarsList[[2]] <- mtcarsList[[2]][1:2,]
mtcarsList[[3]] <- mtcarsList[[3]][1:3,]
attr(mtcarsList, "subheadings") <- paste0("Number of cylinders = ",
                                          names(mtcarsList))
attr(mtcarsList, "message") <- c("Line 1 of Message",
                                 "Line 2 of Message")
str(mtcarsList)
attributes(mtcarsList)

## ----xtablelist-------------------------------------------
xList <- xtableList(mtcarsList)
str(xList)

## ----xtablelist1------------------------------------------
xList1 <- xtableList(mtcarsList, digits = c(0,2,0,0,0,1,2))
str(xList1)

## ----xtablelist2------------------------------------------
xList2 <- xtableList(mtcarsList, digits = c(0,2,0,0,0,1,2),
                            caption = "Caption to List",
                            label = "tbl:xtableList")
str(xList2)

## ----xtablelist3------------------------------------------
attr(mtcarsList, "subheadings") <- NULL
xList3 <- xtableList(mtcarsList)
str(xList3)

## ----xtablelist4------------------------------------------
attr(mtcarsList, "message") <- NULL
xList4 <- xtableList(mtcarsList)
str(xList4)

## ----singledefault, results='asis'------------------------
print.xtableList(xList)

## ----singlebooktabs, results='asis'-----------------------
print.xtableList(xList, booktabs = TRUE)

## ----singlebooktabs1, results='asis'----------------------
print.xtableList(xList1, booktabs = TRUE)

## ----sanitize---------------------------------------------
large <- function(x){
  paste0('{\\Large{\\bfseries ', x, '}}')
}
italic <- function(x){
  paste0('{\\emph{ ', x, '}}')
}
bold <- function(x){
  paste0('{\\bfseries ', x, '}')
}
red <- function(x){
  paste0('{\\color{red} ', x, '}')
}

## ----sanitizesingle, results='asis'-----------------------
print.xtableList(xList,
                 sanitize.rownames.function = italic,
                 sanitize.colnames.function = large,
                 sanitize.subheadings.function = bold,
                 sanitize.message.function = red,
                 booktabs = TRUE)

## ----singlecaption, results='asis'------------------------
print.xtableList(xList2, floating = TRUE)

## ----singlerotated, results='asis'------------------------
print.xtableList(xList, rotate.colnames = TRUE)

## ----nosubheadings, results='asis'------------------------
print.xtableList(xList3)

## ----nomessage, results='asis'----------------------------
print.xtableList(xList4)

## ----multipledefault, results='asis'----------------------
print.xtableList(xList, colnames.format = "multiple")

## ----multiplebooktabs, results='asis'---------------------
print.xtableList(xList, colnames.format = "multiple",
                 booktabs = TRUE)

## ----sanitizemultiple, results='asis'---------------------
print.xtableList(xList, colnames.format = "multiple",
                 sanitize.rownames.function = italic,
                 sanitize.colnames.function = large,
                 sanitize.subheadings.function = bold,
                 sanitize.message.function = red,
                 booktabs = TRUE)

## ----multiplecaption, results='asis'----------------------
print.xtableList(xList2, colnames.format = "multiple",
                 floating = TRUE)

## ----multiplerotated, results='asis'----------------------
print.xtableList(xList, colnames.format = "multiple",
                 rotate.colnames = TRUE)

## ----multiplenosubheadings, results='asis'----------------
print.xtableList(xList3, colnames.format = "multiple")

## ----multiplenomessage, results='asis'--------------------
print.xtableList(xList4, colnames.format = "multiple")

## ----set, include=FALSE--------------------------------------------------
library(knitr)
opts_chunk$set(fig.path='Figures/list', debug=TRUE, echo=TRUE)
opts_chunk$set(out.width='0.9\\textwidth')

## ----package, results='asis'------------------------------
library(xtable)
options(xtable.floating = FALSE)
options(xtable.timestamp = "")
options(width = 60)

## ----data-------------------------------------------------
require(xtable)
data(mtcars)
mtcars <- mtcars[, 1:6]
mtcarsList <- split(mtcars, f = mtcars$cyl)
### Reduce the size of the list elements
mtcarsList[[1]] <- mtcarsList[[1]][1,]
mtcarsList[[2]] <- mtcarsList[[2]][1:2,]
mtcarsList[[3]] <- mtcarsList[[3]][1:3,]
attr(mtcarsList, "subheadings") <- paste0("Number of cylinders = ",
                                          names(mtcarsList))
attr(mtcarsList, "message") <- c("Line 1 of Message",
                                 "Line 2 of Message")
str(mtcarsList)
attributes(mtcarsList)

## ----xtablelist-------------------------------------------
xList <- xtableList(mtcarsList)
str(xList)

## ----xtablelist1------------------------------------------
xList1 <- xtableList(mtcarsList, digits = c(0,2,0,0,0,1,2))
str(xList1)

## ----xtablelist2------------------------------------------
xList2 <- xtableList(mtcarsList, digits = c(0,2,0,0,0,1,2),
                            caption = "Caption to List",
                            label = "tbl:xtableList")
str(xList2)

## ----xtablelist3------------------------------------------
attr(mtcarsList, "subheadings") <- NULL
xList3 <- xtableList(mtcarsList)
str(xList3)

## ----xtablelist4------------------------------------------
attr(mtcarsList, "message") <- NULL
xList4 <- xtableList(mtcarsList)
str(xList4)

## ----singledefault, results='asis'------------------------
print.xtableList(xList)

## ----singlebooktabs, results='asis'-----------------------
print.xtableList(xList, booktabs = TRUE)

## ----singlebooktabs1, results='asis'----------------------
print.xtableList(xList1, booktabs = TRUE)

## ----sanitize---------------------------------------------
large <- function(x){
  paste0('{\\Large{\\bfseries ', x, '}}')
}
italic <- function(x){
  paste0('{\\emph{ ', x, '}}')
}
bold <- function(x){
  paste0('{\\bfseries ', x, '}')
}
red <- function(x){
  paste0('{\\color{red} ', x, '}')
}

## ----sanitizesingle, results='asis'-----------------------
print.xtableList(xList,
                 sanitize.rownames.function = italic,
                 sanitize.colnames.function = large,
                 sanitize.subheadings.function = bold,
                 sanitize.message.function = red,
                 booktabs = TRUE)

## ----singlecaption, results='asis'------------------------
print.xtableList(xList2, floating = TRUE)

## ----singlerotated, results='asis'------------------------
print.xtableList(xList, rotate.colnames = TRUE)

## ----nosubheadings, results='asis'------------------------
print.xtableList(xList3)

## ----nomessage, results='asis'----------------------------
print.xtableList(xList4)

## ----multipledefault, results='asis'----------------------
print.xtableList(xList, colnames.format = "multiple")

## ----multiplebooktabs, results='asis'---------------------
print.xtableList(xList, colnames.format = "multiple",
                 booktabs = TRUE)

## ----sanitizemultiple, results='asis'---------------------
print.xtableList(xList, colnames.format = "multiple",
                 sanitize.rownames.function = italic,
                 sanitize.colnames.function = large,
                 sanitize.subheadings.function = bold,
                 sanitize.message.function = red,                 
                 booktabs = TRUE)

## ----multiplecaption, results='asis'----------------------
print.xtableList(xList2, colnames.format = "multiple",
                 floating = TRUE)

## ----multiplerotated, results='asis'----------------------
print.xtableList(xList, colnames.format = "multiple",
                 rotate.colnames = TRUE)

## ----multiplenosubheadings, results='asis'----------------
print.xtableList(xList3, colnames.format = "multiple")

## ----multiplenomessage, results='asis'--------------------
print.xtableList(xList4, colnames.format = "multiple")

### Test of implementation of request #2104
### Use of \centering rather that center environment when centering tables
### DJS, 16/8/2012
require(xtable)
ctl <- c(4.17,5.58,5.18,6.11,4.50,4.61,5.17,4.53,5.33,5.14)
trt <- c(4.81,4.17,4.41,3.59,5.87,3.83,6.03,4.89,4.32,4.69)
group <- gl(2,10,20, labels=c("Ctl","Trt"))
weight <- c(ctl, trt)
lm.D9 <- lm(weight ~ group)
class(lm.D9)

xtable(lm.D9, caption="\\tt latex.environment=\"center\"")

## % latex table generated in R 2.15.0 by xtable 1.7-1 package
## % Thu Aug 16 15:44:09 2012
## \begin{table}[ht]
## \centering
## \begin{tabular}{rrrrr}
##   \hline
##  & Estimate & Std. Error & t value & Pr($>$$|$t$|$) \\
##   \hline
## (Intercept) & 5.0320 & 0.2202 & 22.85 & 0.0000 \\
##   groupTrt & -0.3710 & 0.3114 & -1.19 & 0.2490 \\
##    \hline
## \end{tabular}
## \caption{\tt latex.environment="center"}
## \end{table}

require(xtable)
V <- matrix(c(1.140380e-03,  3.010497e-05,  7.334879e-05,
              3.010497e-05,  3.320683e-04, -5.284854e-05,
              7.334879e-05, -5.284854e-05,  3.520928e-04), nrow = 3)
### Simple test of print.xtableMatharray
print.xtableMatharray(xtable(V, display = rep("E", 4)))

class(V) <- c("xtableMatharray")
class(V)

### Test without any additional arguments
mth <- xtableMatharray(V)
str(mth)
print(mth)

### Test with arguments to xtable
mth <- xtableMatharray(V, display = rep("E", 4))
str(mth)
print(mth)

mth <- xtableMatharray(V, digits = 6)
str(mth)
print(mth)

### Test with additional print.xtableMatharray arguments
mth <- xtableMatharray(V, digits = 6)
str(mth)
print(mth, format.args = list(decimal.mark = ","))
print(mth, scalebox = 0.5)
print(mth, comment = TRUE)
print(mth, timestamp = "2000-01-01")
print(mth, comment = TRUE, timestamp = "2000-01-01")

### Test of implementation of request #2104
### Use of \centering rather that center environment when centering tables
### DJS, 16/8/2012
require(xtable)
ctl <- c(4.17,5.58,5.18,6.11,4.50,4.61,5.17,4.53,5.33,5.14)
trt <- c(4.81,4.17,4.41,3.59,5.87,3.83,6.03,4.89,4.32,4.69)
group <- gl(2,10,20, labels=c("Ctl","Trt"))
weight <- c(ctl, trt)
lm.D9 <- lm(weight ~ group)
class(lm.D9)

xtable(lm.D9, caption="\\tt latex.environment=\"center\"")

## % latex table generated in R 2.15.0 by xtable 1.7-1 package
## % Thu Aug 16 15:44:09 2012
## \begin{table}[ht]
## \centering
## \begin{tabular}{rrrrr}
##   \hline
##  & Estimate & Std. Error & t value & Pr($>$$|$t$|$) \\
##   \hline
## (Intercept) & 5.0320 & 0.2202 & 22.85 & 0.0000 \\
##   groupTrt & -0.3710 & 0.3114 & -1.19 & 0.2490 \\
##    \hline
## \end{tabular}
## \caption{\tt latex.environment="center"}
## \end{table}


### General testing removed from xtable.Rd, 15/02/2026
### Was leaving detritus in check directory
sessionDir <- getwd()
print(sessionDir)
  for(i in c("latex","html")) {
    outFileName <- paste("xtable.", ifelse(i=="latex", "tex", i), sep = "")
    data(tli)
    tli.table <- xtable(tli[1:20, ])
    print(tli.table, type = i, file = outFileName, append = FALSE)
    design.matrix <- model.matrix(~ sex*grade, data = tli[1:20, ])
    design.table <- xtable(design.matrix, auto = TRUE)
    print(design.table, type = i, file = outFileName, append = TRUE)
    fm1 <- aov(tlimth ~ sex + ethnicty + grade + disadvg, data = tli)
    fm1.table <- xtable(fm1)
    print(fm1.table, type = i, file = outFileName, append = TRUE)
    fm2 <- lm(tlimth ~ sex*ethnicty, data = tli)
    fm2.table <- xtable(fm2)
    print(fm2.table, type = i, file = outFileName, append = TRUE)
    print(fm2.table, type = i, file = outFileName, append = TRUE,
          math.style.negative = TRUE)
    print(xtable(anova(fm2)), type = i, file = outFileName, append = TRUE)
    fm2b <- lm(tlimth ~ ethnicty, data = tli)
    print(xtable(anova(fm2b, fm2)), type = i, file = outFileName, append = TRUE)
    fm3 <- glm(disadvg ~ ethnicty*grade, data = tli, family = binomial())
    fm3.table <- xtable(fm3)
    print(fm3.table, type = i, file = outFileName, append = TRUE)
    print(xtable(anova(fm3)), type = i, file = outFileName, append = TRUE)
    N <- c(0,1,0,1,1,1,0,0,0,1,1,0,1,1,0,0,1,0,1,0,1,1,0,0)
    P <- c(1,1,0,0,0,1,0,1,1,1,0,0,0,1,0,1,1,0,0,1,0,1,1,0)
    K <- c(1,0,0,1,0,1,1,0,0,1,0,1,0,1,1,0,0,0,1,1,1,0,1,0)
    yield <- c(49.5,62.8,46.8,57.0,59.8,58.5,55.5,56.0,62.8,55.8,69.5,55.0,
               62.0,48.8,45.5,44.2,52.0,51.5,49.8,48.8,57.2,59.0,53.2,56.0)
    npk <- data.frame(block = gl(6,4), N = factor(N), P = factor(P),
                      K = factor(K), yield = yield)
    npk.aov <- aov(yield ~ block + N*P*K, npk)
    print(xtable(npk.aov), type = i, file = outFileName, append = TRUE)
    print(xtable(anova(npk.aov)), type = i, file = outFileName, append = TRUE)
    print(xtable(summary(npk.aov)), type = i, file = outFileName, append = TRUE)
    op <- options(contrasts = c("contr.helmert", "contr.treatment"))
    npk.aovE <- aov(yield ~  N*P*K + Error(block), npk)
    print(xtable(npk.aovE), type = i, file = outFileName, append = TRUE)
    options(op)
    print(xtable(summary(npk.aovE)),
          type = i, file = outFileName, append = TRUE)
    if(i=="latex") cat("\\\\clearpage\n", file = outFileName, append = TRUE)
    ctl <- c(4.17,5.58,5.18,6.11,4.50,4.61,5.17,4.53,5.33,5.14)
    trt <- c(4.81,4.17,4.41,3.59,5.87,3.83,6.03,4.89,4.32,4.69)
    group <- gl(2,10,20, labels = c("Ctl","Trt"))
    weight <- c(ctl, trt)
    lm.D9 <- lm(weight ~ group)
    print(xtable(lm.D9),
          type = i, file = outFileName, append = TRUE, latex.environment = NULL)
    print(xtable(lm.D9),
          type = i, file = outFileName, append = TRUE, latex.environment = "")
    print(xtable(lm.D9),
          type = i, file = outFileName, append = TRUE,
          latex.environment = "center")
    print(xtable(anova(lm.D9)), type = i, file = outFileName, append = TRUE)
    counts <- c(18,17,15,20,10,20,25,13,12)
    outcome <- gl(3,1,9)
    treatment <- gl(3,3)
    d.AD <- data.frame(treatment, outcome, counts)
    glm.D93 <- glm(counts ~ outcome + treatment, family = poisson())
    print(xtable(glm.D93), type = i, file = outFileName, append = TRUE)
    print(xtable(anova(glm.D93, test = "Chisq")),
          type = i, file = outFileName, append = TRUE)
    print(xtable(glm.D93, align = "r|llrc"),
          include.rownames = FALSE, include.colnames = TRUE,
          type = i, file = outFileName, append = TRUE)
    print(xtable(glm.D93, align = "r||llrc"),
          include.rownames = TRUE, include.colnames = FALSE,
          type = i, file = outFileName, append = TRUE)
    print(xtable(glm.D93, align = "|r||llrc"),
          include.rownames = FALSE, include.colnames = FALSE,
          type = i, file = outFileName, append = TRUE)
    print(xtable(glm.D93, align = "|r||llrc|"),
          type = i, file = outFileName, append = TRUE)
    print(xtable(anova(glm.D93)),
          hline.after = c(1), size = "small",
          type = i, file = outFileName, append = TRUE)
    if(require(stats, quietly = TRUE)) {
      data(USArrests)
      pr1 <- prcomp(USArrests)
      print(xtable(pr1), type = i, file = outFileName, append = TRUE)
      print(xtable(summary(pr1)), type = i, file = outFileName, append = TRUE)
      # print(xtable(pr2), type = i, file = outFileName, append = TRUE)
    }
    temp.table <- xtable(ts(cumsum(1+round(rnorm(100), 2)),
                            start = c(1954, 7), frequency = 12))
    caption(temp.table) <- "Time series example"
    print(temp.table, type = i, file = outFileName,
          append = TRUE, caption.placement = "top", table.placement = "h")
    print(temp.table, type = i, file = outFileName,
          append = TRUE, caption.placement = "bottom", table.placement = "htb")
  }

### xtableMatharray object
### To deal with numeric arrays such as a variance-covariance matrix
### From a request by James Curran, 16 October 2015
xtableMatharray <- function(x, caption = NULL, label = NULL,
                            align = NULL, digits = NULL,
                            display = NULL, auto = FALSE,
                            ...) {
  class(x) <- c("xtableMatharray","matrix")
  xtbl <- xtable.matrix(x,
                        caption = caption, label = label, align = align,
                        digits = digits, display = display, auto = auto,
                        ...)
  class(xtbl) <- c("xtableMatharray","xtable","data.frame")
  return(xtbl)
}

print.xtableMatharray <- function(x,
           print.results = TRUE,
           format.args = getOption("xtable.format.args", NULL),
           scalebox = getOption("xtable.scalebox", NULL),
           comment = FALSE,
           timestamp = NULL,
           ...)
{
  class(x) <- c("xtableMatharray","data.frame")
  print.xtable(x, floating = FALSE,
               tabular.environment = 'array',
               include.rownames = FALSE, include.colnames = FALSE,
               hline.after = NULL,
               print.results = print.results,
               format.args = format.args,
               scalebox = scalebox,
               comment = comment,
               timestamp = timestamp,
               ...)
}

### xtable package
###
### Produce LaTeX and HTML tables from R objects.
###
### Copyright 2000-2013 David B. Dahl <dahl@stat.byu.edu>
###
### Maintained by David Scott <d.scott@auckland.ac.nz>
###
### This file is part of the `xtable' library for R and related languages.
### It is made available under the terms of the GNU General Public
### License, version 2, or at your option, any later version,
### incorporated herein by reference.
###
### This program is distributed in the hope that it will be
### useful, but WITHOUT ANY WARRANTY; without even the implied
### warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
### PURPOSE.  See the GNU General Public License for more
### details.
###
### You should have received a copy of the GNU General Public
### License along with this program; if not, write to the Free
### Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
### MA 02111-1307, USA

### The generic for toLatex() is declared in the base package "utils"

toLatex.xtable <- function(object, ...){
  ## Initially just capturing the output of print.xtable().  At some
  ## point this could be refactored to have print.xtable() call
  ## toLatex() instead. - CR, 30/01/2012
  dotArgs <- list(...)
  dotArgs$x <- object
  dotArgs$type <- "latex"
  dotArgs$print.results <- FALSE
  z <- do.call("print.xtable", dotArgs)

  z <- strsplit(z, split="\n")[[1]]
  class(z) <- "Latex"
  z
}

autoformat <- function(xtab, zap = getOption("digits")) {
  align(xtab) <- xalign(xtab)
  digits(xtab) <- xdigits(xtab, zap = zap)
  display(xtab) <- xdisplay(xtab)
  return(xtab)
}

xalign <- function(x, pad = TRUE) {
  lr <- function(v) if(is.numeric(v)) "r" else "l"

  is.2d <- length(dim(x)) == 2
  alignment <- if(is.2d) sapply(as.data.frame(x), lr) else lr(x)
  output <- if(is.2d && pad) c("l", alignment) else alignment

  return(output)
}

xdigits <- function(x, pad = TRUE, zap = getOption("digits")) {
  dig <- function(v) {
    if(is.numeric(v)) {
      v <- na.omit(v)
      v <- zapsmall(abs(v - floor(v)), zap)
      dec <- if(any(v > 0)) max(nchar(v) - 2L) else 0L
    } else {
      dec <- 0L
    }
    return(dec)
  }

  is.2d <- length(dim(x)) == 2
  decimals <- if(is.2d) sapply(as.data.frame(x), dig) else dig(x)
  output <- if(is.2d && pad) c(0L, decimals) else decimals

  return(output)
}

xdisplay <- function(x, pad = TRUE) {
  type <- function(v) {
    if(is.numeric(v)) {
      tp <- if(xdigits(v) == 0) "d" else "f"
    } else {
      tp <- "s"
    }
    return(tp)
  }

  is.2d <- length(dim(x)) == 2
  disp <- if(is.2d) sapply(as.data.frame(x), type) else type(x)
  output <- if(is.2d && pad) c("s", disp) else disp

  return(output)
}

### xtable package
###
### Produce LaTeX and HTML tables from R objects.
###
### Copyright 2000-2013 David B. Dahl <dahl@stat.byu.edu>
###
### Maintained by David Scott <d.scott@auckland.ac.nz>
###
### This file is part of the `xtable' library for R and related languages.
### It is made available under the terms of the GNU General Public
### License, version 2, or at your option, any later version,
### incorporated herein by reference.
###
### This program is distributed in the hope that it will be
### useful, but WITHOUT ANY WARRANTY; without even the implied
### warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
### PURPOSE.  See the GNU General Public License for more
### details.
###
### You should have received a copy of the GNU General Public
### License along with this program; if not, write to the Free
### Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
### MA 02111-1307, USA
print.xtable <- function(x,
  type = getOption("xtable.type", "latex"),
  file = getOption("xtable.file", ""),
  append = getOption("xtable.append", FALSE),
  floating = getOption("xtable.floating", TRUE),
  floating.environment = getOption("xtable.floating.environment", "table"),
  table.placement = getOption("xtable.table.placement", "ht"),
  caption.placement = getOption("xtable.caption.placement", "bottom"),
  caption.width = getOption("xtable.caption.width", NULL),
  latex.environments = getOption("xtable.latex.environments", c("center")),
  tabular.environment = getOption("xtable.tabular.environment", "tabular"),
  size = getOption("xtable.size", NULL),
  hline.after = getOption("xtable.hline.after", c(-1,0,nrow(x))),
  NA.string = getOption("xtable.NA.string", ""),
  include.rownames = getOption("xtable.include.rownames", TRUE),
  include.colnames = getOption("xtable.include.colnames", TRUE),
  only.contents = getOption("xtable.only.contents", FALSE),
  add.to.row = getOption("xtable.add.to.row", NULL),
  sanitize.text.function = getOption("xtable.sanitize.text.function", NULL),
  sanitize.rownames.function = getOption("xtable.sanitize.rownames.function",
                                         sanitize.text.function),
  sanitize.colnames.function = getOption("xtable.sanitize.colnames.function",
                                         sanitize.text.function),
  math.style.negative = getOption("xtable.math.style.negative", FALSE),
  math.style.exponents = getOption("xtable.math.style.exponents", FALSE),
  html.table.attributes = getOption("xtable.html.table.attributes", "border=1"),
  print.results = getOption("xtable.print.results", TRUE),
  format.args = getOption("xtable.format.args", NULL),
  rotate.rownames = getOption("xtable.rotate.rownames", FALSE),
  rotate.colnames = getOption("xtable.rotate.colnames", FALSE),
  booktabs = getOption("xtable.booktabs", FALSE),
  scalebox = getOption("xtable.scalebox", NULL),
  width = getOption("xtable.width", NULL),
  comment = getOption("xtable.comment", TRUE),
  timestamp = getOption("xtable.timestamp", date()),
  ...)
{
  ## If caption is length 2, treat the second value as the "short caption"
  caption <- attr(x,"caption",exact = TRUE)
  short.caption <- NULL
  if (!is.null(caption) && length(caption) > 1){
    short.caption <- caption[2]
    caption <- caption[1]
  }

  ## Claudio Agostinelli <claudio@unive.it> dated 2006-07-28 hline.after
  ## By default it print an \hline before and after the columns names
  ## independently they are printed or not and at the end of the table
  ## Old code that set hline.after should include c(-1, 0, nrow(x)) in the
  ## hline.after vector
  ## If you do not want any \hline inside the data, set hline.after to NULL
  ## PHEADER instead the string '\\hline\n' is used in the code
  ## Now hline.after counts how many time a position appear
  ## I left an automatic PHEADER in the longtable is this correct?

  ## Claudio Agostinelli <claudio@unive.it> dated 2006-07-28 include.rownames,
  ## include.colnames
  pos <- 0
  if (include.rownames) pos <- 1

  ## Claudio Agostinelli <claudio@unive.it> dated 2006-07-28
  ## hline.after checks
  if (any(hline.after < -1) | any(hline.after > nrow(x))) {
    stop("'hline.after' must be inside [-1, ", nrow(x), "]")
  }

  ## Claudio Agostinelli <claudio@unive.it> dated 2006-07-28
  ## add.to.row checks
  if (!is.null(add.to.row)) {
    if (is.list(add.to.row) && length(add.to.row) == 2) {
      if (is.null(names(add.to.row))) {
        names(add.to.row) <- c('pos', 'command')
      } else if (any(sort(names(add.to.row))!= c('command', 'pos'))) {
        stop("the names of the elements of 'add.to.row' must be 'pos' and 'command'")
      }
      if (is.list(add.to.row$pos) && is.vector(add.to.row$command,
                                               mode = 'character')) {
        if ((npos <- length(add.to.row$pos)) !=
            length(add.to.row$command)) {
          stop("the length of 'add.to.row$pos' must be equal to the length of 'add.to.row$command'")
        }
        if (any(unlist(add.to.row$pos) < -1) |
            any(unlist(add.to.row$pos) > nrow(x))) {
          stop("the values in add.to.row$pos must be inside the interval [-1, ", nrow(x), "]")
        }
      } else {
        stop("the first argument ('pos') of 'add.to.row' must be a list, the second argument ('command') must be a vector of mode character")
      }
    } else {
      stop("'add.to.row' argument must be a list of length 2")
    }
  } else {
    add.to.row <- list(pos = list(),
                       command = vector(length = 0, mode = "character"))
    npos <- 0
  }

  ## Claudio Agostinelli <claudio@unive.it> dated 2006-07-28 add.to.row
  ## Add further commands at the end of rows
  if (type == "latex") {
    ## Original code before changes in version 1.6-1
    ## PHEADER <- "\\hline\n"

    ## booktabs code from Matthieu Stigler <matthieu.stigler@gmail.com>,
    ## 1 Feb 2012
    if(!booktabs){
      PHEADER <- "\\hline\n"
    } else {
      ## This code replaced to fix bug #2309, David Scott, 8 Jan 2014
      ## PHEADER <- ifelse(-1%in%hline.after, "\\toprule\n", "")
      ## if(0%in%hline.after) {
      ##     PHEADER <- c(PHEADER, "\\midrule\n")
      ## }
      ## if(nrow(x)%in%hline.after) {
      ##     PHEADER <- c(PHEADER, "\\bottomrule\n")
      ## }
      if (is.null(hline.after)){
        PHEADER <- ""
      } else {
        hline.after <- sort(hline.after)
        PHEADER <- rep("\\midrule\n", length(hline.after))
        if (hline.after[1] == -1) {
          PHEADER[1] <- "\\toprule\n"
        }
        if (hline.after[length(hline.after)] == nrow(x)) {
          PHEADER[length(hline.after)] <- "\\bottomrule\n"
        }
      }
    }
  } else {
    PHEADER <- ""
  }

  lastcol <- rep(" ", nrow(x)+2)
  if (!is.null(hline.after)) {
    ## booktabs change - Matthieu Stigler: fill the hline arguments
    ## separately, 1 Feb 2012
    ##
    ## Code before booktabs change was:
    ##    add.to.row$pos[[npos+1]] <- hline.after

    if (!booktabs){
      add.to.row$pos[[npos+1]] <- hline.after
    } else {
      for(i in 1:length(hline.after)) {
        add.to.row$pos[[npos+i]] <- hline.after[i]
      }
    }
    add.to.row$command <- c(add.to.row$command, PHEADER)
  }

  if ( length(add.to.row$command) > 0 ) {
    for (i in 1:length(add.to.row$command)) {
      addpos <- add.to.row$pos[[i]]
      freq <- table(addpos)
      addpos <- unique(addpos)
      for (j in 1:length(addpos)) {
        lastcol[addpos[j]+2] <- paste(lastcol[addpos[j]+2],
                                      paste(rep(add.to.row$command[i],
                                                freq[j]),
                                            sep = "", collapse = ""),
                                      sep = " ")
      }
    }
  }

  if (length(type)>1) stop("\"type\" must have length 1")
  type <- tolower(type)
  if (!all(!is.na(match(type, c("latex","html"))))) {
    stop("\"type\" must be in {\"latex\", \"html\"}")
  }
  ## Disabling the check on known floating environments as many users
  ## want to use additional environments.
  ##    if (!all(!is.na(match(floating.environment,
  ##                          c("table","table*","sidewaystable",
  ##                            "margintable"))))) {
  ##        stop("\"type\" must be in {\"table\", \"table*\", \"sidewaystable\", \"margintable\"}")
  ##    }
  if (("margintable" %in% floating.environment)
      & (!is.null(table.placement))) {
    warning("margintable does not allow for table placement; setting table.placement to NULL")
    table.placement <- NULL
  }
  if (!is.null(table.placement) &&
      !all(!is.na(match(unlist(strsplit(table.placement,  split = "")),
                        c("H","h","t","b","p","!"))))) {
    stop("\"table.placement\" must contain only elements of {\"h\",\"t\",\"b\",\"p\",\"!\"}")
  }
  if (!all(!is.na(match(caption.placement, c("bottom","top"))))) {
    stop("\"caption.placement\" must be either {\"bottom\",\"top\"}")
  }

  if (type == "latex") {
    BCOMMENT <- "% "
    ECOMMENT <- "\n"
    ## See e-mail from "John S. Walker <jsw9c@uic.edu>" dated 5-19-2003
    ## regarding "texfloat"
    ## See e-mail form "Fernando Henrique Ferraz P. da Rosa"
    ## <academic@feferraz.net>" dated 10-28-2005 regarding "longtable"
    if ( tabular.environment == "longtable" & floating == TRUE ) {
      warning("Attempt to use \"longtable\" with floating = TRUE. Changing to FALSE.")
      floating <- FALSE
    }
    if ( floating == TRUE ) {
      ## See e-mail from "Pfaff, Bernhard <Bernhard.Pfaff@drkw.com>"
      ## dated 7-09-2003 regarding "suggestion for an amendment of
      ## the source"
      ## See e-mail from "Mitchell, David"
      ## <David.Mitchell@dotars.gov.au>" dated 2003-07-09 regarding
      ## "Additions to R xtable package"
      ## See e-mail from "Garbade, Sven"
      ## <Sven.Garbade@med.uni-heidelberg.de> dated 2006-05-22
      ## regarding the floating environment.
      BTABLE <- paste("\\begin{", floating.environment, "}",
                      ifelse(!is.null(table.placement),
                             paste("[", table.placement, "]", sep = ""),
                             ""), "\n", sep = "")
      if ( is.null(latex.environments) ||
           (length(latex.environments) == 0) ) {
        BENVIRONMENT <- ""
        EENVIRONMENT <- ""
      } else {
        BENVIRONMENT <- ""
        EENVIRONMENT <- ""
        if ("center" %in% latex.environments){
          BENVIRONMENT <- paste(BENVIRONMENT, "\\centering\n",
                                sep = "")
        }
        for (i in 1:length(latex.environments)) {
          if (latex.environments[i] == "") next
          if (latex.environments[i] != "center"){
            BENVIRONMENT <- paste(BENVIRONMENT,
                                  "\\begin{", latex.environments[i],
                                  "}\n", sep = "")
            EENVIRONMENT <- paste("\\end{", latex.environments[i],
                                  "}\n", EENVIRONMENT, sep = "")
          }
        }
      }
      ETABLE <- paste("\\end{", floating.environment, "}\n", sep = "")
    } else {
      BTABLE <- ""
      ETABLE <- ""
      BENVIRONMENT <- ""
      EENVIRONMENT <- ""
    }

    tmp.index.start <- 1
    if ( ! include.rownames ) {
      while ( attr(x, "align", exact = TRUE)[tmp.index.start] == '|' )
        tmp.index.start <- tmp.index.start + 1
      tmp.index.start <- tmp.index.start + 1
    }
    ## Added "width" argument for use with "tabular*" or
    ## "tabularx" environments - CR, 7/2/12
    if (is.null(width)){
      WIDTH <-""
    } else if (is.element(tabular.environment,
                          c("tabular", "longtable"))){
      warning("Ignoring 'width' argument.  The 'tabular' and 'longtable' environments do not support a width specification.  Use another environment such as 'tabular*' or 'tabularx' to specify the width.")
      WIDTH <- ""
    } else {
      WIDTH <- paste("{", width, "}", sep = "")
    }

    BTABULAR <-
      paste("\\begin{", tabular.environment, "}",
            WIDTH, "{",
            paste(c(attr(x, "align",
                         exact = TRUE)[
              tmp.index.start:length(attr(x, "align",
                                          exact = TRUE))],
              "}\n"),
              sep = "", collapse = ""),
            sep = "")

    ## fix 10-26-09 (robert.castelo@upf.edu) the following
    ## 'if' condition is added here to support
    ## a caption on the top of a longtable
    if (tabular.environment == "longtable" && caption.placement == "top") {
      if (is.null(short.caption)){
        BCAPTION <- "\\caption{"
      } else {
        BCAPTION <- paste("\\caption[", short.caption, "]{", sep = "")
      }
      ECAPTION <- "} \\\\ \n"
      if ((!is.null(caption)) && (type == "latex")) {
        BTABULAR <- paste(BTABULAR,  BCAPTION, caption, ECAPTION,
                          sep = "")
      }
    }
    ## Claudio Agostinelli <claudio@unive.it> dated 2006-07-28
    ## add.to.row position -1
    BTABULAR <- paste(BTABULAR, lastcol[1], sep = "")
    ## the \hline at the end, if present, is set in full matrix
    ETABULAR <- paste("\\end{", tabular.environment, "}\n", sep = "")

    ## Add scalebox - CR, 7/2/12
    if (!is.null(scalebox)){
      BTABULAR <- paste("\\scalebox{", scalebox, "}{\n", BTABULAR,
                        sep = "")
      ETABULAR <- paste(ETABULAR, "}\n", sep = "")
    }

    ## BSIZE contributed by Benno <puetz@mpipsykl.mpg.de> in e-mail
    ## dated Wednesday, December 01, 2004
    if (is.null(size) || !is.character(size)) {
      BSIZE <- ""
      ESIZE <- ""
    } else {
      if(length(grep("^\\\\", size)) == 0){
        size <- paste("\\", size, sep = "")
      }
      ## Change suggested by Claudius Loehnert reported in Bug #6260
      ## BSIZE <- paste("{", size, "\n", sep = "")
      ## ESIZE <- "{\n"
      BSIZE <- paste("\\begingroup", size, "\n", sep = "")
      ESIZE <- "\\endgroup\n"
    }
    BLABEL <- "\\label{"
    ELABEL <- "}\n"
    ## Added caption width (jeff.laake@nooa.gov)
    if(!is.null(caption.width)){
      BCAPTION <- paste("\\parbox{",caption.width,"}{",sep="")
      ECAPTION <- "}"
    } else {
      BCAPTION <- NULL
      ECAPTION <- NULL
    }
    if (is.null(short.caption)){
      BCAPTION <- paste(BCAPTION,"\\caption{",sep="")
    } else {
      BCAPTION <- paste(BCAPTION,"\\caption[", short.caption, "]{", sep="")
    }
    ECAPTION <- paste(ECAPTION,"} \n",sep="")
    BROW <- ""
    EROW <- " \\\\ \n"
    BTH <- ""
    ETH <- ""
    STH <- " & "
    BTD1 <- " & "
    BTD2 <- ""
    BTD3 <- ""
    ETD  <- ""
    } else {
      BCOMMENT <- "<!-- "
      ECOMMENT <- " -->\n"
      BTABLE <- paste("<table ", html.table.attributes, ">\n", sep = "")
      ETABLE <- "</table>\n"
      BENVIRONMENT <- ""
      EENVIRONMENT <- ""
      BTABULAR <- ""
      ETABULAR <- ""
      BSIZE <- ""
      ESIZE <- ""
      BLABEL <- "<a name="
      ELABEL <- "></a>\n"
      BCAPTION <- paste("<caption align=\"", caption.placement, "\"> ",
                        sep = "")
      ECAPTION <- " </caption>\n"
      BROW <- "<tr>"
      EROW <- " </tr>\n"
      BTH <- " <th> "
      ETH <- " </th> "
      STH <- " </th> <th> "
      BTD1 <- " <td align=\""
      align.tmp <- attr(x, "align", exact = TRUE)
      align.tmp <- align.tmp[align.tmp!="|"]
      if (nrow(x) == 0) {
        BTD2 <- matrix(nrow = 0, ncol = ncol(x)+pos)
      } else {
        BTD2 <- matrix(align.tmp[(2-pos):(ncol(x)+1)],
                       nrow = nrow(x), ncol = ncol(x)+pos, byrow = TRUE)
      }
      ## Based on contribution from Jonathan Swinton <jonathan@swintons.net>
      ## in e-mail dated Wednesday, January 17, 2007
      BTD2[regexpr("^p", BTD2)>0] <- "left"
      BTD2[BTD2 == "r"] <- "right"
      BTD2[BTD2 == "l"] <- "left"
      BTD2[BTD2 == "c"] <- "center"
      BTD3 <- "\"> "
      ETD  <- " </td>"
    }

  result <- string("", file = file, append = append)
  info <- R.Version()
  ## modified Claudio Agostinelli <claudio@unive.it> dated 2006-07-28
  ## to set automatically the package version
  if (comment){
    result <- result + BCOMMENT + type + " table generated in " +
      info$language + " " + info$major + "." + info$minor +
      " by xtable " +  packageDescription('xtable')$Version +
                                                  " package" + ECOMMENT
    if (!is.null(timestamp)){
      result <- result + BCOMMENT + timestamp + ECOMMENT
    }
  }
  ## Claudio Agostinelli <claudio@unive.it> dated 2006-07-28 only.contents
  if (!only.contents) {
    result <- result + BTABLE
    result <- result + BENVIRONMENT
    if ( floating == TRUE ) {
      if ((!is.null(caption)) &&
          (type == "html" ||caption.placement == "top")) {
        result <- result + BCAPTION + caption + ECAPTION
      }
      if (!is.null(attr(x, "label", exact = TRUE)) &&
          (type == "latex" && caption.placement == "top")) {
        result <- result + BLABEL +
          attr(x, "label", exact = TRUE) + ELABEL
      }
    }
    result <- result + BSIZE
    result <- result + BTABULAR
  }
  ## Claudio Agostinelli <claudio@unive.it> dated 2006-07-28
  ## include.colnames, include.rownames
  if (include.colnames) {
    result <- result + BROW + BTH
    if (include.rownames) {
      result <- result + STH
    }
    ## David G. Whiting in e-mail 2007-10-09
    if (is.null(sanitize.colnames.function)) {
      CNAMES <- sanitize(names(x), type = type)
    } else {
      CNAMES <- sanitize.colnames.function(names(x))
    }
    if (rotate.colnames) {
      ##added by Markus Loecher, 2009-11-16
      CNAMES <- paste("\\begin{sideways}", CNAMES, "\\end{sideways}")
    }
    result <- result + paste(CNAMES, collapse = STH)

    result <- result + ETH + EROW
  }

  cols <- matrix("", nrow = nrow(x), ncol = ncol(x)+pos)
  if (include.rownames) {
    ## David G. Whiting in e-mail 2007-10-09
    if (is.null(sanitize.rownames.function)) {
      RNAMES <- sanitize(row.names(x), type = type)
    } else {
      RNAMES <- sanitize.rownames.function(row.names(x))
    }
    if (rotate.rownames) {
      ##added by Markus Loecher, 2009-11-16
      RNAMES <- paste("\\begin{sideways}", RNAMES, "\\end{sideways}")
    }
    cols[, 1] <- RNAMES
  }

  ## Begin vectorizing the formatting code by Ian Fellows [ian@fellstat.com]
  ## 06 Dec 2011
  ##
  ##  disp <- function(y) {
  ##    if (is.factor(y)) {
  ##      y <- levels(y)[y]
  ##    }
  ##    if (is.list(y)) {
  ##      y <- unlist(y)
  ##    }
  ##    return(y)
  ##  }
  varying.digits <- is.matrix( attr( x, "digits", exact = TRUE ) )
  ## Code for letting "digits" be a matrix was provided by
  ## Arne Henningsen <ahenningsen@agric-econ.uni-kiel.de>
  ## in e-mail dated 2005-06-04.
  ##if( !varying.digits ) {
  ## modified Claudio Agostinelli <claudio@unive.it> dated 2006-07-28
  ##  attr(x,"digits") <- matrix( attr( x, "digits",exact=TRUE ),
  ## nrow = nrow(x), ncol = ncol(x)+1, byrow = TRUE )
  ##}
  for(i in 1:ncol(x)) {
    xcol <- x[, i]
    if(is.factor(xcol))
      xcol <- as.character(xcol)
    if(is.list(xcol))
      xcol <- sapply(xcol, unlist)
    ina <- is.na(xcol)
    is.numeric.column <- is.numeric(xcol)

    if(is.character(xcol)) {
      cols[, i+pos] <- xcol
    } else {
      if (is.null(format.args)){
        format.args <- list()
      }
      if (is.null(format.args$decimal.mark)){
        format.args$decimal.mark <- options()$OutDec
      }
      if(!varying.digits){
        curFormatArgs <-
          c(list(
            x = xcol,
            format =
              ifelse(attr(x, "digits", exact = TRUE )[i+1] < 0, "E",
                     attr(x, "display", exact = TRUE )[i+1]),
            digits = abs(attr(x, "digits", exact = TRUE )[i+1])),
            format.args)
        cols[, i+pos] <- do.call("formatC", curFormatArgs)
      }else{
        for( j in 1:nrow( cols ) ) {
          curFormatArgs <-
            c(list(
              x = xcol[j],
              format =
                ifelse(attr(x, "digits", exact = TRUE )[j, i+1] < 0,
                       "E", attr(x, "display", exact = TRUE )[i+1]),
              digits =
                abs(attr(x, "digits", exact = TRUE )[j, i+1])),
              format.args)
          cols[j, i+pos] <- do.call("formatC", curFormatArgs)
        }
      }
    }
    ## End Ian Fellows changes

    if ( any(ina) ) cols[ina, i+pos] <- NA.string
    ## Based on contribution from Jonathan Swinton <jonathan@swintons.net>
    ## in e-mail dated Wednesday, January 17, 2007
    if ( is.numeric.column ) {
      cols[, i+pos] <-
        sanitize.numbers(cols[, i+pos], type = type,
                         math.style.negative = math.style.negative,
                         math.style.exponents = math.style.exponents)
    } else {
      if (is.null(sanitize.text.function)) {
        cols[, i+pos] <- sanitize(cols[, i+pos], type = type)
      } else {
        cols[, i+pos] <- sanitize.text.function(cols[, i+pos])
      }
    }
  }

  multiplier <- 5
  full <- matrix("", nrow = nrow(x), ncol = multiplier*(ncol(x)+pos)+2)
  full[, 1] <- BROW
  full[, multiplier*(0:(ncol(x)+pos-1))+2] <- BTD1
  full[, multiplier*(0:(ncol(x)+pos-1))+3] <- BTD2
  full[, multiplier*(0:(ncol(x)+pos-1))+4] <- BTD3
  full[, multiplier*(0:(ncol(x)+pos-1))+5] <- cols
  full[, multiplier*(0:(ncol(x)+pos-1))+6] <- ETD

  full[, multiplier*(ncol(x)+pos)+2] <- paste(EROW, lastcol[-(1:2)],
                                              sep = " ")

  if (type == "latex") full[, 2] <- ""
  result <- result + lastcol[2] + paste(t(full), collapse = "")
  if (!only.contents) {
    if (tabular.environment == "longtable") {
      ## booktabs change added the if() - 1 Feb 2012
      if(!booktabs) {
        result <- result + PHEADER
      }

      ## fix 10-27-09 Liviu Andronic (landronimirc@gmail.com) the
      ## following 'if' condition is inserted in order to avoid
      ## that bottom caption interferes with a top caption of a longtable
      if(caption.placement == "bottom"){
        if ((!is.null(caption)) && (type == "latex")) {
          result <- result + BCAPTION + caption + ECAPTION
        }
      }
      if (!is.null(attr(x, "label", exact = TRUE))) {
        result <- result + BLABEL + attr(x, "label", exact = TRUE) +
          ELABEL
      }
      ETABULAR <- "\\end{longtable}\n"
    }
    result <- result + ETABULAR
    result <- result + ESIZE
    if ( floating == TRUE ) {
      if ((!is.null(caption)) &&
          (type == "latex" && caption.placement == "bottom")) {
        result <- result + BCAPTION + caption + ECAPTION
      }
      if (!is.null(attr(x, "label", exact = TRUE)) &&
          caption.placement == "bottom") {
        result <- result + BLABEL + attr(x, "label", exact = TRUE) +
          ELABEL
      }
    }
    result <- result + EENVIRONMENT
    result <- result + ETABLE
  }
  result <- sanitize.final(result, type = type)

  if (print.results){
    print(result)
  }

  return(invisible(result$text))
}

"+.string" <- function(x, y) {
  x$text <- paste(x$text, as.string(y)$text, sep = "")
  return(x)
}

print.string <- function(x, ...) {
  cat(x$text, file = x$file, append = x$append)
  return(invisible())
}

string <- function(text, file = "", append = FALSE) {
  x <- list(text = text, file = file, append = append)
  class(x) <- "string"
  return(x)
}

as.string <- function(x, file = "", append = FALSE) {
  if (is.null(attr(x, "class", exact = TRUE)))
    switch(data.class(x),
           character = return(string(x, file, append)),
           numeric = return(string(as.character(x), file, append)),
           stop("Cannot coerce argument to a string"))
  if (class(x) == "string")
    return(x)
  stop("Cannot coerce argument to a string")
}

is.string <- function(x) {
  return(class(x) == "string")
}

### xtable package
###
### Produce LaTeX and HTML tables from R objects.
###
### Copyright 2000-2013 David B. Dahl <dahl@stat.byu.edu>
###
### This file is part of the `xtable' library for R and related languages.
### It is made available under the terms of the GNU General Public
### License, version 2, or at your option, any later version,
### incorporated herein by reference.
###
### This program is distributed in the hope that it will be
### useful, but WITHOUT ANY WARRANTY; without even the implied
### warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
### PURPOSE.  See the GNU General Public License for more
### details.
###
### You should have received a copy of the GNU General Public
### License along with this program; if not, write to the Free
### Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
### MA 02111-1307, USA

"caption<-" <- function(x, value) UseMethod("caption<-")
"caption<-.xtable" <- function(x, value) {
  if (length(value) > 2)
    stop("\"caption\" must have length 1 or 2")
  attr(x, "caption") <- value
  return(x)
}

caption <- function(x, ...) UseMethod("caption")
caption.xtable <- function(x, ...) {
  return(attr(x, "caption", exact = TRUE))
}

"label<-" <- function(x, value) UseMethod("label<-")
"label<-.xtable" <- function(x, value) {
  if (length(value) > 1)
    stop("\"label\" must have length 1")
  attr(x, "label") <- value
  return(x)
}

label <- function(x, ...) UseMethod("label")
label.xtable <- function(x, ...) {
  return(attr(x, "label", exact = TRUE))
}

"align<-" <- function(x, value) UseMethod("align<-")

### Based on contribution from Jonathan Swinton <jonathan@swintons.net>
### in e-mail dated Wednesday, January 17, 2007
.alignStringToVector <- function(aString) {
  ## poor mans parsing - separating string of form "l{2in}llr|p{1in}c|{1in}"
  ## into "l{2in}" "l"  "l"  "r" "|" "p{1in}" "c" "|{1in}"
  aString.Align <- character(0);
  aString.Width <- character(0);
  wString <- aString
  while( nchar(wString) > 0) {
    aString.Align <- c(aString.Align, substr(wString, 1, 1))
    ## is it followed by a brace?
    thisWidth <- ""
    if ( nchar(wString) > 1 & substr(wString, 2, 2) == "{") {
      beforeNextBrace <- regexpr("[^\\]\\}", wString)
      if (beforeNextBrace <0 ) {
        stop("No closing } in align string")
      }
      thisWidth <- substr(wString, 2, beforeNextBrace + 1)
      wString <- substr(wString, beforeNextBrace + 2, nchar(wString))
    } else {
      wString <- substr(wString, 2, nchar(wString))
    }
    aString.Width <- c(aString.Width, thisWidth)
  }

  alignAllowed <- c("l","r","p","c","|","X")

  if (any( !(aString.Align %in% alignAllowed))) {
    warning("Nonstandard alignments in align string")
  }
  res <- paste(aString.Align, aString.Width, sep = "")
  res
}
###.alignStringToVector ("l{2in}llr|p{1in}c|{1in}")
###.alignStringToVector ("l{2in}llr|p{1in}c|")
### latex syntax error, but gives wrong alignment
###.alignStringToVector ("{2in}llr|p{1in}c|")
###.alignStringToVector("llllp{3cm}")

"align<-.xtable" <- function(x, value) {
### Based on contribution from Benno <puetz@mpipsykl.mpg.de>
### in e-mail dated Wednesday, December 01, 2004
### Based on contribution from Jonathan Swinton <jonathan@swintons.net>
### in e-mail dated Wednesday, January 17, 2007
  ## cat("%", value, "\n")
  if ( (!is.null(value)) && ( is.character(value) ) &&
       ( length(value) == 1 ) && ( nchar(value) > 1 ) ) {
        value <- .alignStringToVector(value)
  }
  ## That should have checked we had only lrcp|
  ## but what if the "if statement" is false?
  ## For simplicity, deleting check present in version 1.4-2 and earlier.
  c.value <- if (any(!is.na(match(value, "|")))) {
               value[-which(value == '|')]
             } else {
               value
             }
  if (length(c.value) != ncol(x) + 1)
    stop(paste("\"align\" must have length equal to",
               ncol(x) + 1, "( ncol(x) + 1 )"))
  attr(x, "align") <- value
  return(x)
}

align <- function(x, ...) UseMethod("align")
align.xtable <- function(x, ...) {
  return(attr(x, "align", exact = TRUE))
}

"digits<-" <- function(x, value) UseMethod("digits<-")
"digits<-.xtable" <- function(x, value) {
  if( is.matrix( value ) ) {
    if( ncol( value ) != ncol(x) + 1 || nrow( value ) != nrow(x) ) {
      stop( "if argument 'digits' is a matrix, it must have columns equal",
           " to ", ncol(x) + 1, " ( ncol(x) + 1 ) and rows equal to ", nrow(x),
           " ( nrow( x )" )
    }
  } else {
    if( length(value) == 1 ) { value <- rep(value, ncol(x) + 1) }
    if( length( value ) > 1 & length( value ) != ncol(x) + 1 ) {
      stop( "if argument 'digits' is a vector of length more than one, it must have length equal",
           " to ", ncol(x) + 1, " ( ncol(x) + 1 )" )
    }
  }
  if (!is.numeric(value))
    stop("\"digits\" must be numeric")
  attr(x, "digits") <- value
  return(x)
}

digits <- function(x, ...) UseMethod("digits")
digits.xtable <- function(x, ...) {
  return(attr(x, "digits", exact = TRUE))
}

"display<-" <- function(x, value) UseMethod("display<-")
"display<-.xtable" <- function(x, value) {
  if (length(value) != ncol(x) + 1)
    stop(paste("\"display\" must have length equal to",
               ncol(x) + 1, "( ncol(x) + 1 )"))
  if (!all(!is.na(match(value, c("d","f","e","E","g","G","fg","s")))))
    stop("\"display\" must be in {\"d\",\"f\",\"e\",\"E\",\"g\",\"G\", \"fg\", \"s\"}")
  attr(x, "display") <- value
  return(x)
}

display <- function(x, ...) UseMethod("display")
display.xtable <- function(x, ...) {
  return(attr(x, "display", exact = TRUE))
}

### xtable package
###
### Produce LaTeX and HTML tables from R objects.
###
### Copyright 2000-2013 David B. Dahl <dahl@stat.byu.edu>
###
### This file is part of the `xtable' library for R and related languages.
### It is made available under the terms of the GNU General Public
### License, version 2, or at your option, any later version,
### incorporated herein by reference.
###
### This program is distributed in the hope that it will be
### useful, but WITHOUT ANY WARRANTY; without even the implied
### warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
### PURPOSE.  See the GNU General Public License for more
### details.
###
### You should have received a copy of the GNU General Public
### License along with this program; if not, write to the Free
### Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
### MA 02111-1307, USA

xtable <- function(x, caption = NULL, label = NULL, align = NULL,
                   digits = NULL, display = NULL, auto = FALSE, ...) {
  UseMethod("xtable")
}


### data.frame and matrix objects

xtable.data.frame <- function(x, caption = NULL, label = NULL, align = NULL,
                              digits = NULL, display = NULL, auto = FALSE,
                              ...) {
  logicals <- unlist(lapply(x, is.logical))
  ##x[, logicals] <- lapply(x[, logicals], as.character)
  ## Patch for logicals bug, no 1911
  ## David Scott, <d.scott@auckland.ac.nz>, 2012-08-10
  x[, logicals] <- lapply(x[, logicals, drop = FALSE], as.character)
  characters <- unlist(lapply(x, is.character))
  factors <- unlist(lapply(x, is.factor))
  ints <- sapply(x, is.integer)
  class(x) <- c("xtable","data.frame")
  caption(x) <- caption
  label(x) <- label
  if(auto && is.null(align))   align   <- xalign(x)
  if(auto && is.null(digits))  digits  <- xdigits(x)
  if(auto && is.null(display)) display <- xdisplay(x)
  align(x) <- switch(1+is.null(align), align,
                     c("r",c("r","l")[(characters|factors)+1]))
  digits(x) <- switch(1+is.null(digits), digits, c(0,rep(2,ncol(x))))
  ## Patch from Seth Falcon <sfalcon@fhcrc.org>, 18-May-2007
  if (is.null(display)) {
      display <- rep("f", ncol(x))
      display[ints] <- "d"
      display[characters | factors] <- "s"
      display <- c("s", display)
  }
  display(x) <- display
  return(x)
}

xtable.matrix <- function(x, caption = NULL, label = NULL, align = NULL,
                          digits = NULL, display = NULL, auto = FALSE, ...) {
  return(xtable.data.frame(data.frame(x, check.names = FALSE),
                           caption = caption, label = label, align = align,
                           digits = digits, display = display, auto = auto,
                           ...))
}



### table objects (of 1 or 2 dimensions) by Guido Gay, 9 Feb 2007
### Fixed to pass R checks by DBD, 9 May 2007
xtable.table <- function(x, caption = NULL, label = NULL, align = NULL,
                       digits = NULL, display = NULL, auto = FALSE, ...) {
  if (length(dim(x)) == 1) {
    return(xtable.matrix(matrix(x,
                                dimnames = list(rownames(x),
                                                names(dimnames(x)))),
                         caption = caption, label = label, align = align,
                         digits = digits, display = display, auto = auto, ...))
  } else if (length(dim(x))==2) {
    return(xtable.matrix(matrix(x, ncol = dim(x)[2], nrow = dim(x)[1],
                                dimnames = list(rownames(x), colnames(x))),
                         caption = caption, label = label, align = align,
                         digits = digits, display = display, auto = auto, ...))
  } else {
    stop("xtable.table is not implemented for tables of > 2 dimensions")
  }
}


### anova objects
xtable.anova <- function(x, caption = NULL, label = NULL, align = NULL,
                         digits = NULL, display = NULL, auto = FALSE, ...) {
  suggested.digits <- c(0,rep(2, ncol(x)))
  suggested.digits[grep("Pr\\(>", names(x))+1] <- 4
  suggested.digits[grep("P\\(>", names(x))+1] <- 4
  suggested.digits[grep("Df", names(x))+1] <- 0

  class(x) <- c("xtable","data.frame")
  caption(x) <- caption
  label(x) <- label
  if(auto && is.null(align))   align   <- xalign(x)
  if(auto && is.null(digits))  digits  <- xdigits(x)
  if(auto && is.null(display)) display <- xdisplay(x)
  align(x) <- switch(1+is.null(align), align, c("l",rep("r", ncol(x))))
  digits(x) <- switch(1+is.null(digits), digits, suggested.digits)
  display(x) <- switch(1+is.null(display), display, c("s",rep("f", ncol(x))))
  return(x)
}


### aov objects
xtable.aov <- function(x, caption = NULL, label = NULL, align = NULL,
                       digits = NULL, display = NULL, auto = FALSE, ...) {
  return(xtable.anova(anova(x, ...), caption = caption, label = label,
                      align = align, digits = digits, display = display,
                      auto = auto, ...))
}

xtable.summary.aov <- function(x, caption = NULL, label = NULL, align = NULL,
                               digits = NULL, display = NULL, auto = FALSE,
                               ...) {
  return(xtable.anova(x[[1]], caption = caption, label = label, align = align,
                      digits = digits, display = display, auto = auto, ...))
}

xtable.summary.aovlist <- function(x, caption = NULL, label = NULL,
                                   align = NULL, digits = NULL, display = NULL,
                                   auto = FALSE, ...) {
    for (i in 1:length(x)) {
        if (i == 1) {
            result <- xtable.summary.aov(x[[i]], caption = caption,
                                         label = label,
                                         align = align, digits = digits,
                                         display = display, auto = auto, ...)
        } else {
            result <- rbind(result,
                            xtable.anova(x[[i]][[1]], caption = caption,
                                         label = label, align = align,
                                         digits = digits, display = display,
                                         auto = auto, ...))
        }
    }
    return(result)
}

xtable.aovlist <- function(x, caption = NULL, label = NULL, align = NULL,
                           digits = NULL, display = NULL, auto = FALSE, ...) {
  return(xtable.summary.aovlist(summary(x), caption = caption, label = label,
                                align = align, digits = digits,
                                display = display, auto = auto, ...))
}



### lm objects
xtable.lm <- function(x, caption = NULL, label = NULL, align = NULL,
                      digits = NULL, display = NULL, auto = FALSE, ...) {
  return(xtable.summary.lm(summary(x), caption = caption, label = label,
                           align = align, digits = digits, display = display,
                           auto = auto, ...))
}

xtable.summary.lm <- function(x, caption = NULL, label = NULL, align = NULL,
                              digits = NULL, display = NULL, auto = FALSE,
                              ...) {
  x <- data.frame(x$coef, check.names = FALSE)

  class(x) <- c("xtable","data.frame")
  caption(x) <- caption
  label(x) <- label
  if(auto && is.null(align))   align   <- xalign(x)
  if(auto && is.null(digits))  digits  <- xdigits(x)
  if(auto && is.null(display)) display <- xdisplay(x)
  align(x) <- switch(1+is.null(align), align, c("r","r","r","r","r"))
  digits(x) <- switch(1+is.null(digits), digits, c(0,4,4,2,4))
  display(x) <- switch(1+is.null(display), display, c("s","f","f","f","f"))
  return(x)
}


### glm objects
xtable.glm <- function(x, caption = NULL, label = NULL, align = NULL,
                       digits = NULL, display = NULL, auto = FALSE, ...) {
  return(xtable.summary.glm(summary(x), caption = caption,
                            label = label, align = align,
                            digits = digits, display = display,
                            auto = auto, ...))
}

xtable.summary.glm <- function(x, caption = NULL, label = NULL, align = NULL,
                               digits = NULL, display = NULL, auto = FALSE,
                               ...) {
  return(xtable.summary.lm(x, caption = caption, label = label, align = align,
                           digits = digits, display = display,
                           auto = auto, ...))
}


### prcomp objects
xtable.prcomp <- function(x, caption = NULL, label = NULL, align = NULL,
                          digits = NULL, display = NULL, auto = FALSE, ...) {
  x <- data.frame(x$rotation, check.names = FALSE)

  class(x) <- c("xtable","data.frame")
  caption(x) <- caption
  label(x) <- label
  if(auto && is.null(align))   align   <- xalign(x)
  if(auto && is.null(digits))  digits  <- xdigits(x)
  if(auto && is.null(display)) display <- xdisplay(x)
  align(x) <- switch(1+is.null(align), align, c("r",rep("r", ncol(x))))
  digits(x) <- switch(1+is.null(digits), digits, c(0,rep(4, ncol(x))))
  display(x) <- switch(1+is.null(display), display, c("s",rep("f", ncol(x))))
  return(x)
}

xtable.summary.prcomp <- function(x, caption = NULL, label = NULL, align = NULL,
                                  digits = NULL, display = NULL, auto = FALSE,
                                  ...) {
  x <- data.frame(x$importance, check.names = FALSE)

  class(x) <- c("xtable","data.frame")
  caption(x) <- caption
  label(x) <- label
  if(auto && is.null(align))   align   <- xalign(x)
  if(auto && is.null(digits))  digits  <- xdigits(x)
  if(auto && is.null(display)) display <- xdisplay(x)
  align(x) <- switch(1+is.null(align), align, c("r",rep("r", ncol(x))))
  digits(x) <- switch(1+is.null(digits), digits, c(0,rep(4, ncol(x))))
  display(x) <- switch(1+is.null(display), display, c("s",rep("f", ncol(x))))
  return(x)
}


### Slightly modified version of xtable.coxph contributed on r-help by
###   Date: Wed, 2 Oct 2002 17:47:56 -0500 (CDT)
###   From: Jun Yan <jyan@stat.wisc.edu>
###   Subject: Re: [R] xtable for Cox model output
xtable.coxph <- function (x, caption = NULL, label = NULL, align = NULL,
                          digits = NULL, display = NULL, auto = FALSE, ...)
{
  cox <- x
  beta <- cox$coef
  se <- sqrt(diag(cox$var))
  if (is.null(cox$naive.var)) {
    tmp <- cbind(beta, exp(beta), se, beta/se, 1 - pchisq((beta/se)^2, 1))
    dimnames(tmp) <- list(names(beta),
      c("coef", "exp(coef)", "se(coef)", "z", "p"))
  } else {
    tmp <- cbind( beta, exp(beta), se, beta/se,
      signif(1 - pchisq((beta/se)^2, 1), digits - 1))
    dimnames(tmp) <- list(names(beta),
      c("coef", "exp(coef)", "robust se", "z", "p"))
  }
  return(xtable(tmp, caption = caption, label = label, align = align,
                digits = digits, display = display, auto = auto, ...))
}

### Additional method: xtable.ts
### Contributed by David Mitchell (davidm@netspeed.com.au)
### Date: July 2003
xtable.ts <- function(x, caption = NULL, label = NULL, align = NULL,
                      digits = NULL, display = NULL, auto = FALSE, ...) {
  if (inherits(x, "ts") && !is.null(ncol(x))) {
    ## COLNAMES <- paste(colnames(x));
    tp.1 <- trunc(time(x))
    tp.2 <- trunc(cycle(x))
    day.abb <- c("Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat")
    ROWNAMES <- switch(frequency(x),
                       tp.1,
                       "Arg2", "Arg3",              # Dummy arguments
                       paste(tp.1, c("Q1", "Q2", "Q3", "Q4")[tp.2], sep = " "),
                       "Arg5", "Arg6",
                       paste("Wk.", tp.1, " ", day.abb[tp.2], sep = ""),
                       "Arg8", "Arg9", "Arg10", "Arg11",
                       paste(tp.1, month.abb[tp.2], sep = " "))
    tmp <- data.frame(x, row.names = ROWNAMES);
  } else if (inherits(x, "ts") && is.null(ncol(x))) {
    COLNAMES <- switch(frequency(x),
                       "Value",
                       "Arg2", "Arg3",              # Dummy arguments
                       c("Q1", "Q2", "Q3", "Q4"),
                       "Arg5", "Arg6",
                       day.abb,
                       "Arg8", "Arg9", "Arg10", "Arg11",
                       month.abb)
    ROWNAMES <- seq(from = start(x)[1], to = end(x)[1])
    tmp <- data.frame(matrix(c(rep(NA, start(x)[2] - 1), x,
                               rep(NA, frequency(x) - end(x)[2])),
                             ncol = frequency(x), byrow = TRUE),
                      row.names = ROWNAMES)
    names(tmp) <- COLNAMES
  }
  return(xtable(tmp, caption = caption, label = label, align = align,
                digits = digits, display = display, auto = auto, ...))
}

### Suggested by Ajay Narottam Shah <ajayshah@mayin.org> in e-mail 2006/07/22
xtable.zoo <- function(x, caption = NULL, label = NULL, align = NULL,
                       digits = NULL, display = NULL, auto = FALSE, ...) {
  return(xtable(as.ts(x), caption = caption, label = label,
                align = align, digits = digits,
                display = display, auto = auto, ...))
}

### Date: Fri, 29 May 2015 11:41:04 +0200
### From: Martin G. <martin.gubri@framasoft.org>
### Subject: [xtable] Code for spdep, splm and sphet objects outputs
### package spdep
### sarlm objects
## xtable.sarlm <- function(x, caption = NULL, label = NULL, align = NULL,
##                          digits = NULL, display = NULL, auto = FALSE, ...) {
##   return(xtable.summary.sarlm(summary(x), caption = caption, label = label,
##                               align = align, digits = digits,
##                               display = display, auto = auto, ...))
## }

## xtable.summary.sarlm <- function(x, caption = NULL, label = NULL, align = NULL,
##                                 digits = NULL, display = NULL, auto = FALSE,
##                                 ...) {
##   x <- data.frame(x$Coef, check.names = FALSE)

##   class(x) <- c("xtable","data.frame")
##   caption(x) <- caption
##   label(x) <- label
##   if(auto && is.null(align))   align   <- xalign(x)
##   if(auto && is.null(digits))  digits  <- xdigits(x)
##   if(auto && is.null(display)) display <- xdisplay(x)
##   align(x) <- switch(1+is.null(align), align, c("r","r","r","r","r"))
##   digits(x) <- switch(1+is.null(digits), digits, c(0,4,4,2,4))
##   display(x) <- switch(1+is.null(display), display, c("s","f","f","f","f"))
##   return(x)
## }

## ### spautolm objects: added by David Scott, 6/1/2016, after suggestion by
## ### Guido Schulz
## ### Date: Wed, 29 Apr 2015 10:45:16 +0200
## ### Guido Schulz <schulzgu@student.hu-berlin.de>
## xtable.spautolm <- function(x, caption = NULL, label = NULL, align = NULL,
##                             digits = NULL, display = NULL, auto = FALSE, ...) {
##     return(xtable.summary.sarlm(summary(x), caption = caption, label = label,
##                               align = align, digits = digits,
##                               display = display, auto = auto, ...))
## }

## xtable.summary.spautolm <- function(x, caption = NULL, label = NULL,
##                                     align = NULL, digits = NULL,
##                                     display = NULL, auto = FALSE, ...) {
##     return(xtable.summary.sarlm(summary(x), caption = caption, label = label,
##                               align = align, digits = digits,
##                               display = display, auto = auto, ...))
## }


## ### gmsar objects
## xtable.gmsar <- function(x, caption = NULL, label = NULL, align = NULL,
##                          digits = NULL, display = NULL, auto = FALSE, ...) {
##     return(xtable.summary.sarlm(summary(x), caption = caption, label = label,
##                               align = align, digits = digits,
##                               display = display, auto = auto, ...))
## }

## xtable.summary.gmsar <- function(x, caption = NULL, label = NULL, align = NULL,
##                                  digits = NULL, display = NULL,
##                                  auto = FALSE, ...) {
##   return(xtable.summary.sarlm(x, caption = caption, label = label,
##                               align = align, digits = digits,
##                               display = display, auto = auto, ...))
## }

## ### stsls objects
## xtable.stsls <- function(x, caption = NULL, label = NULL, align = NULL,
##                          digits = NULL, display = NULL, auto = FALSE, ...) {
##   return(xtable.summary.sarlm(summary(x), caption = caption, label = label,
##                               align = align, digits = digits,
##                               display = display, auto = auto, ...))
## }

## xtable.summary.stsls <- function(x, caption = NULL, label = NULL, align = NULL,
##                                  digits = NULL, display = NULL,
##                                  auto = FALSE, ...) {
##   return(xtable.summary.sarlm(x, caption = caption, label = label,
##                               align = align, digits = digits,
##                               display = display, auto = auto, ...))
## }

## ### pred.sarlm objects
## xtable.sarlm.pred <- function(x, caption = NULL, label = NULL, align = NULL,
##                               digits = NULL, display = NULL,
##                               auto = FALSE, ...) {
##   return(xtable(as.data.frame(x), caption = caption, label = label,
##                 align = align, digits = digits,
##                 display = display, auto = auto, ...))
## }

## ### lagImpact objects
## xtable.lagImpact <- function(x, caption = NULL, label = NULL, align = NULL,
##                              digits = NULL, display = NULL,
##                              auto = FALSE, ...) {
##   requireNamespace('spdep')
##   lagImpactMat <- get('lagImpactMat', environment(spdep::spdep))
##   xtable(lagImpactMat(x), caption = caption, label = label,
##          align = align, digits = digits,
##          display = display, auto = auto, ...)
## }

### package splm
### splm objects
## xtable.splm <- function(x, caption = NULL, label = NULL, align = NULL,
##                         digits = NULL, display = NULL, auto = FALSE, ...) {
##   return(xtable.summary.splm(summary(x), caption = caption, label = label,
##                              align = align, digits = digits,
##                              display = display, auto = auto, ...))
## }

## xtable.summary.splm <- function(x, caption = NULL, label = NULL, align = NULL,
##                                 digits = NULL, display = NULL, auto = FALSE,
##                                 ...) {
##   x <- data.frame(x$CoefTable, check.names = FALSE)

##   class(x) <- c("xtable","data.frame")
##   caption(x) <- caption
##   label(x) <- label
##   if(auto && is.null(align))   align   <- xalign(x)
##   if(auto && is.null(digits))  digits  <- xdigits(x)
##   if(auto && is.null(display)) display <- xdisplay(x)
##   align(x) <- switch(1+is.null(align), align, c("r","r","r","r","r"))
##   digits(x) <- switch(1+is.null(digits), digits, c(0,4,4,2,4))
##   display(x) <- switch(1+is.null(display), display, c("s","f","f","f","f"))
##   return(x)
## }

### package sphet
### sphet objects
## xtable.sphet <- function(x, caption = NULL, label = NULL, align = NULL,
##                          digits = NULL, display = NULL, auto = FALSE, ...) {
##   return(xtable.summary.splm(summary(x), caption = caption, label = label,
##                              align = align, digits = digits,
##                              display = display, auto = auto, ...))
## }

## xtable.summary.sphet <- function(x, caption = NULL, label = NULL, align = NULL,
##                                  digits = NULL, display = NULL,
##                                  auto = FALSE, ...) {
##   return(xtable.summary.splm(x, caption = caption, label = label,
##                              align = align, digits = digits,
##                              display = display, auto = auto, ...))
## }

### ftable objects, requested by Charles Roosen
### Feature request #2248, 2/9/2012
xtableFtable <- function(x, caption = NULL, label = NULL, align = NULL,
                         digits = 0, display = NULL,
                         quote = FALSE,
                         method = c("non.compact", "row.compact",
                                    "col.compact", "compact"),
                         lsep = " $\\vert$ ", ...) {
  method <- match.arg(method)
  saveMethod <- method
  xDim <- dim(x)
  nRowVars <- length(attr(x, "row.vars"))
  nColVars <- length(attr(x, "col.vars"))
  if (nRowVars == 0){
    if (method =="col.compact"){
      method <- "non.compact"
    } else if (method == "compact"){
      method <- "row.compact"
    }
  }
  if (nColVars == 0){
    if (method =="row.compact"){
      method <- "non.compact"
    } else if (method == "compact"){
      method <- "col.compact"
    }
  }
  if (method == "non.compact"){
    nCharCols <- nRowVars + 2
    nCharRows <- nColVars + 1
  }
  if (method == "row.compact"){
    nCharCols <- nRowVars + 2
    nCharRows <- nColVars
  }
  if (method == "col.compact"){
    nCharCols <- nRowVars + 1
    nCharRows <- nColVars + 1
  }
  if (method == "compact"){
    nCharCols <- nRowVars + 1
    nCharRows <- nColVars
  }

  if(is.null(align)) {
    align <- c(rep("l", nCharCols - 1), "l |", rep("r", xDim[2]))
  }
  if(is.null(display)) {
    display <- c(rep("s", nCharCols), rep("d", xDim[2]))
  }

  attr(x, "ftableCaption") <- caption
  attr(x, "ftableLabel") <- label
  attr(x, "ftableAlign") <- align
  attr(x, "ftableDigits") <- digits
  attr(x, "quote") <- quote
  attr(x, "ftableDisplay") <- display
  attr(x, "method") <- method
  attr(x, "lsep") <- lsep
  attr(x, "nChars") <- c(nCharRows, nCharCols)
  class(x) <- c("xtableFtable", "ftable")
  return(x)
}

print.xtableFtable <- function(x,
  type = getOption("xtable.type", "latex"),
  file = getOption("xtable.file", ""),
  append = getOption("xtable.append", FALSE),
  floating = getOption("xtable.floating", TRUE),
  floating.environment = getOption("xtable.floating.environment", "table"),
  table.placement = getOption("xtable.table.placement", "ht"),
  caption.placement = getOption("xtable.caption.placement", "bottom"),
  caption.width = getOption("xtable.caption.width", NULL),
  latex.environments = getOption("xtable.latex.environments", c("center")),
  tabular.environment = getOption("xtable.tabular.environment", "tabular"),
  size = getOption("xtable.size", NULL),
  hline.after = getOption("xtable.hline.after", NULL),
  NA.string = getOption("xtable.NA.string", ""),
  only.contents = getOption("xtable.only.contents", FALSE),
  add.to.row = getOption("xtable.add.to.row", NULL),
  sanitize.text.function = getOption("xtable.sanitize.text.function", as.is),
  sanitize.rownames.function = getOption("xtable.sanitize.rownames.function",
                                         sanitize.text.function),
  sanitize.colnames.function = getOption("xtable.sanitize.colnames.function",
                                         sanitize.text.function),
  math.style.negative = getOption("xtable.math.style.negative", FALSE),
  math.style.exponents = getOption("xtable.math.style.exponents", FALSE),
  html.table.attributes = getOption("xtable.html.table.attributes", "border=1"),
  print.results = getOption("xtable.print.results", TRUE),
  format.args = getOption("xtable.format.args", NULL),
  rotate.rownames = getOption("xtable.rotate.rownames", FALSE),
  rotate.colnames = getOption("xtable.rotate.colnames", FALSE),
  booktabs = getOption("xtable.booktabs", FALSE),
  scalebox = getOption("xtable.scalebox", NULL),
  width = getOption("xtable.width", NULL),
  comment = getOption("xtable.comment", TRUE),
  timestamp = getOption("xtable.timestamp", date()),
  ...) {
  if (type == "latex"){
    ## extract the information in the attributes
    caption <- attr(x, "ftableCaption")
    label <- attr(x, "ftableLabel")
    align <- attr(x, "ftableAlign")
    digits <- attr(x, "ftableDigits")
    quote <- attr(x, "quote")
    digits <- attr(x, "ftabelDigits")
    method <- attr(x, "method")
    lsep <- attr(x, "lsep")
    nCharRows <- attr(x, "nChars")[1]
    nCharCols <- attr(x, "nChars")[2]
    nRowVars <- length(attr(x, "row.vars"))
    nColVars <- length(attr(x, "col.vars"))

    ## change class so format method will find format.ftable
    ## even though format.ftable is not exported from 'stats'
    class(x) <- "ftable"
    fmtFtbl <- format(x, quote = quote, digits = digits,
                      method = method, lsep = lsep)
    attr(fmtFtbl, "caption") <- caption
    attr(fmtFtbl, "label") <- label

    ## sanitization is possible for row names and/or column names
    ## row names
    if (is.null(sanitize.rownames.function)) {
      fmtFtbl[nCharRows, 1:nRowVars] <-
        sanitize(fmtFtbl[nCharRows, 1:nRowVars], type = type)
    } else {
      fmtFtbl[nCharRows, 1:nRowVars] <-
        sanitize.rownames.function(fmtFtbl[nCharRows, 1:nRowVars])
    }
    ## column names
    if (is.null(sanitize.colnames.function)) {
      fmtFtbl[1:nColVars, nCharCols - 1] <-
        sanitize(fmtFtbl[1:nColVars, nCharCols - 1],
                 type = type)
    } else {
      fmtFtbl[1:nColVars, nCharCols - 1] <-
        sanitize.colnames.function(fmtFtbl[1:nColVars, nCharCols - 1])
    }
    ## rotations are possible
    if (rotate.rownames){
      fmtFtbl[1:dim(fmtFtbl)[1], 1:(nCharCols - 1)] <-
        paste0("\\begin{sideways} ",
               fmtFtbl[1:dim(fmtFtbl)[1], 1:(nCharCols - 1)],
               "\\end{sideways}")
    }
    if (rotate.colnames){
      if (rotate.rownames){
        fmtFtbl[1:(nCharRows), (nCharCols):dim(fmtFtbl)[2]] <-
          paste0("\\begin{sideways} ",
                 fmtFtbl[1:(nCharRows), (nCharCols):dim(fmtFtbl)[2]],
                 "\\end{sideways}")
      } else {
        fmtFtbl[1:(nCharRows), 1:dim(fmtFtbl)[2]] <-
          paste0("\\begin{sideways} ",
                 fmtFtbl[1:(nCharRows), 1:dim(fmtFtbl)[2]],
                 "\\end{sideways}")
      }
    }


    ## booktabs is incompatible with vertical lines in tables
    if (booktabs) align <- gsub("|","", align, fixed = TRUE)
    attr(fmtFtbl, "align") <- align
    attr(fmtFtbl, "digits") <- digits
    attr(fmtFtbl, "quote") <- quote
    attr(fmtFtbl, "display") <- display

    ## labels should be left aligned
    for (i in 1:nCharRows){
      fmtFtbl[i, nCharCols:dim(fmtFtbl)[2]] <-
        paste0("\\multicolumn{1}{l}{ ",
               fmtFtbl[i, nCharCols:dim(fmtFtbl)[2]], "}")
    }


    if(is.null(hline.after)) {
      hline.after <- c(-1, nCharRows, dim(fmtFtbl)[1])
    }
    print.xtable(fmtFtbl, hline.after = hline.after,
                 include.rownames = FALSE, include.colnames = FALSE,
                 booktabs = booktabs,
                 sanitize.text.function = as.is,
                 file = file,
                 append = append,
                 floating = floating,
                 floating.environment = floating.environment,
                 table.placement = table.placement,
                 caption.placement = caption.placement,
                 caption.width = caption.width,
                 latex.environments = latex.environments,
                 tabular.environment = tabular.environment,
                 size = size,
                 NA.string = NA.string,
                 only.contents = only.contents,
                 add.to.row = add.to.row,,
                 math.style.negative = math.style.negative,
                 math.style.exponents = math.style.exponents,
                 print.results = print.results,
                 format.args = format.args,
                 scalebox = scalebox,
                 width = width,
                 comment = comment,
                 timestamp = timestamp,
                 ...)
  } else {
    stop("print.xtableFtable not yet implemented for this type")
  }
}

### Function to create lists of tables
xtableList <- function(x, caption = NULL, label = NULL, align = NULL,
                       digits = NULL, display = NULL, ...) {
  if (is.null(digits)){
    digitsList <- vector("list", length(x))
  } else {
    if (!is.list(digits)){
      digitsList <- vector("list", length(x))
      for (i in 1:length(x)) digitsList[[i]] <- digits
    }
  }
  if (is.null(display)){
    displayList <- vector("list", length(x))
  } else {
    if (!is.list(display)){
      displayList <- vector("list", length(x))
      for (i in 1:length(x)) displayList[[i]] <- display
    }
  }
  xList <- vector("list", length(x))
  for (i in 1:length(x)){
    xList[[i]] <- xtable(x[[i]], caption = caption, label = label,
                         align = align, digits = digitsList[[i]],
                         display = displayList[[i]], ...)
    attr(xList[[i]], 'subheading') <- attr(x, 'subheadings')[[i]]
  }
  attr(xList, "message") <- attr(x, "message")
  attr(xList, "caption") <- caption
  attr(xList, "label") <- label
  class(xList) <- c("xtableList")
  return(xList)
}

print.xtableList <- function(x,
  type = getOption("xtable.type", "latex"),
  file = getOption("xtable.file", ""),
  append = getOption("xtable.append", FALSE),
  floating = getOption("xtable.floating", TRUE),
  floating.environment = getOption("xtable.floating.environment", "table"),
  table.placement = getOption("xtable.table.placement", "ht"),
  caption.placement = getOption("xtable.caption.placement", "bottom"),
  caption.width = getOption("xtable.caption.width", NULL),
  latex.environments = getOption("xtable.latex.environments", c("center")),
  tabular.environment = getOption("xtable.tabular.environment", "tabular"),
  size = getOption("xtable.size", NULL),
  hline.after = NULL,
  NA.string = getOption("xtable.NA.string", ""),
  include.rownames = getOption("xtable.include.rownames", TRUE),
  colnames.format = "single",
  only.contents = getOption("xtable.only.contents", FALSE),
  add.to.row = NULL,
  sanitize.text.function = getOption("xtable.sanitize.text.function", NULL),
  sanitize.rownames.function = getOption("xtable.sanitize.rownames.function",
                                         sanitize.text.function),
  sanitize.colnames.function = getOption("xtable.sanitize.colnames.function",
                                         sanitize.text.function),
  sanitize.subheadings.function =
    getOption("xtable.sanitize.subheadings.function",
              sanitize.text.function),
  sanitize.message.function =
    getOption("xtable.sanitize.message.function",
              sanitize.text.function),
  math.style.negative = getOption("xtable.math.style.negative", FALSE),
  math.style.exponents = getOption("xtable.math.style.exponents", FALSE),
  html.table.attributes = getOption("xtable.html.table.attributes", "border=1"),
  print.results = getOption("xtable.print.results", TRUE),
  format.args = getOption("xtable.format.args", NULL),
  rotate.rownames = getOption("xtable.rotate.rownames", FALSE),
  rotate.colnames = getOption("xtable.rotate.colnames", FALSE),
  booktabs = getOption("xtable.booktabs", FALSE),
  scalebox = getOption("xtable.scalebox", NULL),
  width = getOption("xtable.width", NULL),
  comment = getOption("xtable.comment", TRUE),
  timestamp = getOption("xtable.timestamp", date()),
  ...)
{
  ## Get number of rows for each table in list of tables
  nCols <- dim(x[[1]])[2]
  rowNums <- sapply(x, dim)[1,]
  combinedRowNums <- cumsum(rowNums)
  combined <- do.call(rbind, x)
  if (type == "latex"){
    ## Special treatment if using booktabs
    if (booktabs){
      tRule <- "\\toprule"
      mRule <- "\\midrule"
      bRule <- "\\bottomrule"
    } else {
      tRule <- "\\hline"
      mRule <- "\\hline"
      bRule <- "\\hline"
    }
    ## Sanitize subheadings if required
    if (!is.null(sanitize.subheadings.function)) {
      for (i in 1:length(x)){
        attr(x[[i]], 'subheading') <-
          sanitize.subheadings.function(attr(x[[i]], 'subheading'))
      }
    }
    ## Sanitize message if required
    if (!is.null(sanitize.message.function)) {
      xMessage <- attr(x, 'message')
      xMessage <- sapply(xMessage, sanitize.message.function)
      attr(x, 'message') <- xMessage
    }
    if (colnames.format == "single"){

      add.to.row <- list(pos = NULL, command = NULL)
      add.to.row$pos <- as.list(c(0, combinedRowNums[-length(x)],
                                  dim(combined)[1]))
      command <- sapply(x, attr, "subheading")

      for (i in 1:length(x)){
        if( !is.null(command[[i]]) ){
          add.to.row$command[i] <-
            paste0(mRule,"\n\\multicolumn{",
                   nCols + include.rownames,
                   "}{l}{",
                   command[[i]],
                   "}\\\\\n")
        } else {
          add.to.row$command[i] <- paste0(mRule, "\n")
        }
      }
      ## Changed at request of Russ Lenth
      ## add.to.row$command[1:length(x)] <-
      ##   paste0(mRule,"\n\\multicolumn{", nCols, "}{l}{", command, "}\\\\\n")

      if ( (booktabs) & length(attr(x, "message") > 0) ){
        attr(x, "message")[1] <-
          paste0("\\rule{0em}{2.5ex}", attr(x, "message")[1])
      }
      add.to.row$command[length(x) + 1] <-
        paste0("\n\\multicolumn{", nCols + include.rownames, "}{l}{",
               attr(x, "message"), "}\\\\\n",
               collapse = "")
      add.to.row$command[length(x) + 1] <-
        paste0(bRule, add.to.row$command[length(x) + 1])

      class(combined) <- c("xtableList", "data.frame")
      hline.after <- c(-1)
      include.colnames <- TRUE
    }

    ## Create headings for columns if multiple headings are needed
    if (colnames.format == "multiple"){
      if (is.null(sanitize.colnames.function)) {
        colHead <- names(x[[1]])
      } else {
        colHead <- sanitize.colnames.function(names(x[[1]]))
      }
      if (rotate.colnames) {
        colHead <- paste("\\begin{sideways}", colHead, "\\end{sideways}")
      }
      colHead <- paste0(colHead, collapse = " & ")
      if (include.rownames) {
        colHead <- paste0(" & ", colHead)
      }
      colHead <- paste0(tRule, "\n", colHead, " \\\\", mRule, "\n")
      add.to.row <- list(pos = NULL, command = NULL)
      add.to.row$pos <- as.list(c(0, c(combinedRowNums[1:length(x)])))
      command <- sapply(x, attr, "subheading")


      add.to.row$command[1] <-
        if( !is.null(command[[1]]) ){
          add.to.row$command[1] <-
            paste0("\n\\multicolumn{", nCols + include.rownames, "}{l}{",
                   command[[1]],
                   "}\\\\ \n", colHead, "\n")
        } else {
          add.to.row$command[1] <- colHead
        }

      for (i in 2:length(x)) {
        add.to.row$command[i] <-
          if( !is.null(command[[i]]) ) {
            paste0(bRule,
                   "\\\\ \n\\multicolumn{",
                   nCols + include.rownames, "}{l}{",
                   command[[i]], "}",
                   "\\\\ \n",
                   colHead)
          } else {
            add.to.row$command[i] <- paste0("\n", colHead)
          }
      }

      ## Changed at request of Russ Lenth
      ## add.to.row$command[1] <-
      ##   paste0("\n\\multicolumn{", nCols, "}{l}{", command[1],
      ##          "}", " \\\\ \n",
      ##          colHead)
      ## add.to.row$command[2:length(x)] <-
      ##   paste0(bRule,
      ##          "\\\\ \n\\multicolumn{", nCols, "}{l}{",
      ##          command[2:length(x)], "}",
      ##          "\\\\ \n",
      ##          colHead)
      if ( (booktabs) & length(attr(x, "message") > 0) ){
        attr(x, "message")[1] <-
          paste0("\\rule{0em}{2.5ex}", attr(x, "message")[1])
      }
      add.to.row$command[length(x) + 1] <-
        paste0("\n\\multicolumn{", nCols + include.rownames, "}{l}{",
               attr(x, "message"), "}\\\\\n",
               collapse = "")
      add.to.row$command[length(x) + 1] <-
        paste0(bRule, add.to.row$command[length(x) + 1])

      class(combined) <- c("xtableList", "data.frame")
      hline.after <- NULL

      include.colnames <- FALSE
    }

    print.xtable(combined,
                 type = type,
                 floating = floating,
                 floating.environment = floating.environment,
                 table.placement = table.placement,
                 caption.placement = caption.placement,
                 caption.width = caption.width,
                 latex.environments = latex.environments,
                 tabular.environment = tabular.environment,
                 size = size,
                 hline.after = hline.after,
                 NA.string = NA.string,
                 include.rownames = include.rownames,
                 include.colnames = include.colnames,
                 only.contents = only.contents,
                 add.to.row = add.to.row,
                 sanitize.text.function = sanitize.text.function,
                 sanitize.rownames.function = sanitize.rownames.function,
                 sanitize.colnames.function = sanitize.colnames.function,
                 math.style.negative = math.style.negative,
                 math.style.exponents = math.style.exponents,
                 html.table.attributes = html.table.attributes,
                 print.results = print.results,
                 format.args = format.args,
                 rotate.rownames = rotate.rownames,
                 rotate.colnames = rotate.colnames,
                 booktabs = booktabs,
                 scalebox = scalebox,
                 width = width,
                 comment = comment,
                 timestamp = timestamp,
                 ...)
  } else {
    stop("print.xtableList not yet implemented for this type")
  }
}


### Uses xtableList
xtableLSMeans <- function(x, caption = NULL, label = NULL,
                          align = NULL, digits = NULL,
                          display = NULL, auto = FALSE,
                          ...){
  if (attr(x, "estName") == "lsmean"){
    xList <- split(x, f = x[, 2])
    for (i in 1:length(xList)){
      xList[[i]] <- as.data.frame(xList[[i]][, -2])
    }
    attr(xList, "subheadings") <-
      paste0(dimnames(x)[[2]][2], " = ", levels(x[[2]]))
    attr(xList, "message") <- c("", attr(x, "mesg"))
    xList <- xtableList(xList, caption = caption, label = label,
                        align = align, digits = digits,
                        display = display, auto = auto, ...)
  } else {
    xList <- x
    xList <- xtable.data.frame(xList, caption = caption, label = label,
                               align = align, digits = digits,
                               display = display, auto = auto, ...)
  }
  return(xList)
}

sanitize <- function(str, type = "latex") {
  if(type == "latex"){
    result <- str
    result <- gsub("\\\\", "SANITIZE.BACKSLASH", result)
    result <- gsub("$", "\\$", result, fixed = TRUE)
    result <- gsub(">", "$>$", result, fixed = TRUE)
    result <- gsub("<", "$<$", result, fixed = TRUE)
    result <- gsub("|", "$|$", result, fixed = TRUE)
    result <- gsub("{", "\\{", result, fixed = TRUE)
    result <- gsub("}", "\\}", result, fixed = TRUE)
    result <- gsub("%", "\\%", result, fixed = TRUE)
    result <- gsub("&", "\\&", result, fixed = TRUE)
    result <- gsub("_", "\\_", result, fixed = TRUE)
    result <- gsub("#", "\\#", result, fixed = TRUE)
    result <- gsub("^", "\\verb|^|", result, fixed = TRUE)
    result <- gsub("~", "\\~{}", result, fixed = TRUE)
    result <- gsub("SANITIZE.BACKSLASH", "$\\backslash$", result, fixed = TRUE)
    return(result)
  } else {
    result <- str
    result <- gsub("&", "&amp;", result, fixed = TRUE)
    result <- gsub(">", "&gt;", result, fixed = TRUE)
    result <- gsub("<", "&lt;", result, fixed = TRUE)
    return(result)
  }
}


sanitize.numbers <- function(str, type,
                             math.style.negative = FALSE,
                             math.style.exponents = FALSE){
  if (type == "latex"){
    result <- str
    if ( math.style.negative ) {
      for(i in 1:length(str)) {
        result[i] <- gsub("-", "$-$", result[i], fixed = TRUE)
      }
    }
    if ( math.style.exponents ) {
      if (is.logical(math.style.exponents) && ! math.style.exponents ) {
      } else if (is.logical(math.style.exponents) && math.style.exponents ||
                 math.style.exponents == "$$"
                 ) {
        for(i in 1:length(str)) {
          result[i] <-
            gsub("^\\$?(-?)\\$?([0-9.]+)[eE]\\$?(-?)\\+?\\$?0*(\\d+)$",
                 "$\\1\\2 \\\\times 10^{\\3\\4}$", result[i])
        }
      } else if (math.style.exponents == "ensuremath") {
        for(i in 1:length(str)) {
          result[i] <-
            gsub("^\\$?(-?)\\$?([0-9.]+)[eE]\\$?(-?)\\+?\\$?0*(\\d+)$",
                 "\\\\ensuremath{\\1\\2 \\\\times 10^{\\3\\4}}",
                 result[i])
        }
      } else if (math.style.exponents == "UTF8" ||
                 math.style.exponents == "UTF-8") {
        for(i in 1:length(str)) {
          ## this code turns 1e5 into a UTF-8 representation of 1\times10^5
          if (all(grepl("^\\$?(-?)\\$?([0-9.]+)[eE]\\$?(-?)\\+?\\$?0*(\\d+)$",
                        result[i]))) {
            temp <- strsplit(result[i],"eE",result[i])
            result[i] <-
              paste0(temp[1],
                     "\u00d710",
                     chartr("-1234567890",
                            "\u207b\u00b9\u00b2\u00b3\u2074\u2075\u20746\u20747\u20748\u20749\u2070",
                            temp[2]))
          }
        }
      }
    }
    return(result)
  } else {
    return(str)
  }
}


sanitize.final <- function(str, type){
  if (type == "latex"){
    return(str)
  } else {
    str$text <- gsub("  *", " ",  str$text, fixed = TRUE)
    str$text <- gsub(' align="left"',  "", str$text,
                     fixed = TRUE)
    return(str)
  }
}

### Some trivial helper functions
### Suggested by Stefan Edwards, sme@iysik.com
### Helper function for disabling sanitizing
as.is <- function(str) {str}

### Helper function for embedding names in a math environment
as.math <- function(str, ...) { paste0('$',str,'$', ...) }

%\VignetteIndexEntry{xtable Gallery}
%\VignetteDepends{xtable}
%\VignetteKeywords{LaTeX, HTML, table}
%\VignettePackage{xtable}
% !Rnw weave = knitr
% \VignetteEngine{knitr::knitr}
%**************************************************************************
\documentclass{article}
\usepackage[a4paper,height=24cm]{geometry} % geometry first
\usepackage{array}
\usepackage{booktabs}
\usepackage{longtable}
\usepackage{parskip}
\usepackage{rotating}
\usepackage{tabularx}
\usepackage{titlesec}
\usepackage{hyperref} % hyperref last
\titleformat\subsubsection{\bfseries\itshape}{}{0pt}{}
\newcommand\p{\vspace{2ex}}
\newcommand\code[1]{\texttt{#1}}
\newcommand\pkg[1]{\textbf{#1}}
\setcounter{tocdepth}{2}
\begin{document}

\title{The \pkg{xtable} Gallery}
\author{Jonathan Swinton and others}
\maketitle

\tableofcontents

\newpage

\section{Introduction}
This document gives a gallery of tables which can be made using the
\pkg{xtable} package to create \LaTeX\ output. It doubles as a
regression check for the package.

<<include=FALSE>>=
library(knitr)
opts_chunk$set(fig.path='figdir/fig', debug=TRUE, echo=TRUE)
set.seed(1234)
@

The first step is to load the package and set an option for this document.
<<results='asis'>>=
library(xtable)
options(xtable.floating = FALSE)
options(xtable.timestamp = "")
@

\section{Gallery}
\subsection{Data frame}
<<results='asis'>>=
data(tli)
xtable(tli[1:10, ])
@

\subsection{Matrix}
<<results='asis'>>=
design.matrix <- model.matrix(~ sex*grade, data = tli[1:10, ])
xtable(design.matrix, digits = 0)
@

\newpage
\subsection{aov}
<<results='asis'>>=
fm1 <- aov(tlimth ~ sex + ethnicty + grade + disadvg, data = tli)
xtable(fm1)
@

\subsection{lm}
<<results='asis'>>=
fm2 <- lm(tlimth ~ sex*ethnicty, data = tli)
xtable(fm2)
@

\subsubsection{Anova table (one model)}
<<results='asis'>>=
xtable(anova(fm2))
@

\subsubsection{Anova table (two models)}
<<results='asis'>>=
fm2b <- lm(tlimth ~ ethnicty, data = tli)
xtable(anova(fm2b, fm2))
@

\subsubsection{Anova list}

<<aovlist>>=
Block <- gl(8, 4)
A <- factor(c(0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,
              0,1,0,1,0,1,0,1,0,1,0,1))
B <- factor(c(0,0,1,1,0,0,1,1,0,1,0,1,1,0,1,0,0,0,1,1,
              0,0,1,1,0,0,1,1,0,0,1,1))
C <- factor(c(0,1,1,0,1,0,0,1,0,0,1,1,0,0,1,1,0,1,0,1,
              1,0,1,0,0,0,1,1,1,1,0,0))
Yield <- c(101, 373, 398, 291, 312, 106, 265, 450, 106, 306, 324, 449,
           272, 89, 407, 338, 87, 324, 279, 471, 323, 128, 423, 334,
           131, 103, 445, 437, 324, 361, 302, 272)
aovdat <- data.frame(Block, A, B, C, Yield)

old <- getOption("contrasts")
options(contrasts = c("contr.helmert", "contr.poly"))
(fit <- aov(Yield ~ A*B*C + Error(Block), data = aovdat))
class(fit)
summary(fit)
options(contrasts = old)
@

\p

<<xtableaovlist, results='asis'>>=
xtable(fit)
@


\newpage
\subsection{glm}
<<results='asis'>>=
fm3 <- glm(disadvg ~ ethnicty*grade, data = tli, family = binomial)
xtable(fm3)
@

\subsubsection{Analysis of deviance}
<<results='asis'>>=
xtable(anova(fm3))
@

\subsection{prcomp}
<<results='asis'>>=
pr1 <- prcomp(USArrests)
xtable(pr1)
@

\p
<<results='asis'>>=
xtable(summary(pr1))
@

<<include=FALSE>>=
# pr2 <- princomp(USArrests)
# xtable(pr2)
@

\newpage

\subsection{Time series}
<<results='asis'>>=
temp.ts <- ts(cumsum(1 + round(rnorm(100), 0)),
              start = c(1954, 7), frequency = 12)
temp.table <- xtable(temp.ts, digits = 0)
temp.table
@

\subsection{Flat tables}
\label{sec:flat-tables}

See the \textbf{Details} section of the help for \code{ftable} for a
description of these tables, which are flat versions of
multi-dimensional contingency tables. They require special methods to
enable them to be printed using \pkg{xtable}.


<<ftable>>=
data(mtcars)
mtcars$cyl <- factor(mtcars$cyl, levels = c("4","6","8"),
                     labels = c("four","six","eight"))
tbl <- ftable(mtcars$cyl, mtcars$vs, mtcars$am, mtcars$gear,
              row.vars = c(2, 4),
              dnn = c("Cylinders", "V/S", "Transmission", "Gears"))
tbl
@ %def

Here is the \LaTeX{} produced:

<<ftablecheck>>=
xftbl <- xtableFtable(tbl, method = "compact")
print.xtableFtable(xftbl, booktabs = TRUE)
@ %def

And here is a basic flat table:

<<ftable1, results = 'asis'>>=
xftbl <- xtableFtable(tbl)
print.xtableFtable(xftbl)
@ %def

This illustrates the \code{method} argument:

<<ftable2, results = 'asis'>>=
xftbl <- xtableFtable(tbl, method = "col.compact")
print.xtableFtable(xftbl, rotate.rownames = TRUE)
@ %def

Booktabs is incompatible with vertical lines in tables, so the
vertical dividing line is removed.

<<ftable3, results = 'asis'>>=
xftbl <- xtableFtable(tbl, method = "compact")
print.xtableFtable(xftbl, booktabs = TRUE)
@ %def
\p

Row and column variable names can be formatted specially using
sanitization, and row and column variable names and labels can be
rotated.

If special formatting is required for row and column labels, that can
be done as a workaround by redefining the data and associated labels.

<<ftable4, results = 'asis'>>=
italic <- function(x){
  paste0('{\\emph{', x, '}}')
}
mtcars$cyl <- factor(mtcars$cyl, levels = c("four","six","eight"),
                     labels = c("four",italic("six"),"eight"))
large <- function(x){
  paste0('{\\Large ', x, '}')
}
bold <- function(x){
  paste0('{\\bfseries ', x, '}')
}
tbl <- ftable(mtcars$cyl, mtcars$vs, mtcars$am, mtcars$gear,
              row.vars = c(2, 4),
              dnn = c("Cylinders", "V/S", "Transmission", "Gears"))
xftbl <- xtableFtable(tbl, method = "row.compact")
print.xtableFtable(xftbl,
                   sanitize.rownames.function = large,
                   sanitize.colnames.function = bold,
                   rotate.colnames = TRUE,
                   rotate.rownames = TRUE)
@ %def



\newpage

<<include=FALSE>>=
# ## Demonstrate saving to file
# for(i in c("latex", "html")) {
#   outFileName <- paste("xtable.", ifelse(i=="latex", "tex", i), sep = "")
#   print(xtable(lm.D9), type = i, file = outFileName, append = TRUE,
#         latex.environments = NULL)
#   print(xtable(lm.D9), type = i, file = outFileName, append = TRUE,
#         latex.environments = "")
#   print(xtable(lm.D9), type = i, file = outFileName, append = TRUE,
#         latex.environments = "center")
#   print(xtable(anova(glm.D93, test = "Chisq")),
#         type = i, file = outFileName,
#         append = TRUE)
#   print(xtable(anova(glm.D93)), hline.after = c(1),
#         size = "small", type = i,
#         file = outFileName, append = TRUE)
#   # print(xtable(pr2), type = i, file = outFileName, append = TRUE)
# }
@

\section{Automatic formatting}
\subsection{Suggest alignment, digits, and display}
The functions \code{xalign}, \code{xdigits}, and \code{xdisplay} are
useful for formatting tables in a sensible way. Consider the output
produced by the default formatting.

<<results='asis'>>=
data(mtcars)
dat <- mtcars[1:3, 1:6]
x <- xtable(dat)
x
@

\p
Now change the default alignment, digits and display using helper functions
\code{xalign}, \code{xdigits}, and \code{xdisplay}. This produces a better
format as shown below.

<<results='asis'>>=
align(x) <- xalign(x)
digits(x) <- xdigits(x)
display(x) <- xdisplay(x)
x
@

\subsection{Shorthand notation}
For convenience, the three `autoformat' functions (\code{xalign},
\code{xdigits}, and \code{xdisplay}) can be applied together when an
\code{xtable} is created, using the \code{auto} argument:

<<results='asis'>>=
xtable(dat, auto = TRUE)
@

\p
Similarly, the \code{autoformat} function can be used to postprocess an
existing \code{xtable}:

<<results='asis'>>=
x <- xtable(dat)
autoformat(x)
@

\newpage

\subsection{Math-Style Exponents}
If you prefer $5 \times 10^5$ in your tables to 5e5, the
\code{math.style.exponents} option to \code{print.xtable} is useful:

<<results='asis'>>=
print(xtable(data.frame(text = c("foo","bar"),
                        googols = c(10e10,50e10),
                        small = c(8e-24,7e-5),
                        row.names = c("A","B")),
             display = c("s","s","g","g")),
      math.style.exponents = TRUE)
@

This option also supports the values \code{ensuremath} which uses
\code{\char`\\ensuremath} instead of \code{\$\$} and \code{UTF-8}
which uses UTF-8 to approximate the \LaTeX\ typesetting.


\section{Sanitization}
<<results='asis'>>=
insane <- data.frame(Name = c("Ampersand","Greater than","Less than",
                            "Underscore","Per cent","Dollar",
                            "Backslash","Hash","Caret","Tilde",
                            "Left brace","Right brace"),
                     Character = I(c("&",">","<","_","%","$",
                                     "\\","#","^","~","{","}")))
colnames(insane)[2] <- paste(insane[, 2], collapse = "")
xtable(insane)
@

\p
Sometimes you might want to have your own sanitization function.


<<results='asis'>>=
wanttex <- xtable(data.frame(Column =
                             paste("Value_is $10^{-",1:3,"}$", sep = "")))
print(wanttex, sanitize.text.function =
      function(str) gsub("_", "\\_", str, fixed = TRUE))
@

\p
Sanitization can be useful in formatting column headings and row names:

<<sanitize3>>=
dat <- mtcars[1:3, 1:6]
large <- function(x){
  paste0('{\\Large{\\bfseries ', x, '}}')
}
italic <- function(x){
  paste0('{\\emph{ ', x, '}}')
}
@ %def

<<sanitize4, results = 'asis'>>=
print(xtable(dat),
      sanitize.rownames.function = italic,
      sanitize.colnames.function = large,
      booktabs = TRUE)
@ %def



\newpage

\subsection{Markup in tables}
Markup can be included in tables, including in column and row names,
by using a custom \code{sanitize.text.function}.

<<results='asis'>>=
mat <- round(matrix(c(0.9, 0.89, 200, 0.045, 2.0), c(1, 5)), 4)
rownames(mat) <- "$y_{t-1}$"
colnames(mat) <- c("$R^2$", "$\\bar{x}$", "F-stat", "S.E.E", "DW")
mat <- xtable(mat)
print(mat, sanitize.text.function = function(x) {x})
@

% By David Dahl to demonstrate contribution from David Whitting, 2007-10-09.
\p
You can also have sanitize functions that are specific to column or
row names.  In the table below, the row name is not sanitized but
column names and table elements are.

<<results='asis'>>=
money <- matrix(c("$1,000","$900","$100"), ncol = 3,
                dimnames = list("$\\alpha$",
                                c("Income (US$)","Expenses (US$)",
                                  "Profit (US$)")))
print(xtable(money), sanitize.rownames.function = function(x) {x})
@

\section{Format examples}
\subsection{Adding a centering environment}
<<results='asis'>>=
print(xtable(anova(fm3), caption = "\\tt latex.environments = \"\""),
      floating = TRUE, latex.environments = "")
print(xtable(anova(fm3), caption = "\\tt latex.environments = \"center\""),
      floating = TRUE, latex.environments = "center")
@

\newpage

\subsection{Column alignment}
<<results='asis'>>=
tli.table <- xtable(tli[1:10, ])
align(tli.table) <- rep("r", 6)
tli.table
@

\subsubsection{Left aligned strings with column lines}
<<results='asis'>>=
align(tli.table) <- "|rrl|l|lr|"
tli.table
@

\subsubsection{Fixed width columns}
<<results='asis'>>=
align(tli.table) <- "|rr|lp{3cm}l|r|"
tli.table
@

\newpage

\subsection{Number of digits}
One number for all columns,
<<results='asis'>>=
display(tli.table)[c(2,6)] <- "f"
digits(tli.table) <- 3
tli.table
@

\p
or one for each column, including the row names,
<<results='asis'>>=
digits(tli.table) <- 1:(ncol(tli)+1)
tli.table
@

\p
or as a full matrix.
<<results='asis'>>=
digits(tli.table) <- matrix(0:4, nrow = 10, ncol = ncol(tli)+1)
tli.table
@

\newpage

\subsection{Suppress row/column names}
\subsubsection{Suppress row names}
<<results='asis'>>=
tli.table <- xtable(tli[1:10, ])
print(tli.table, include.rownames = FALSE)
@

\p
If you want a vertical line on the left, you need to change the \code{align}
attribute.
<<results='asis'>>=
align(tli.table) <- "|r|r|lp{3cm}l|r|"
print(tli.table, include.rownames = FALSE)
@

\p
Revert the alignment to what is was before.
<<>>=
align(tli.table) <- "|rr|lp{3cm}l|r|"
@

\newpage

\subsubsection{Suppress column names}
<<results='asis'>>=
print(tli.table, include.colnames = FALSE)
@

\p
Note the doubled header lines which can be suppressed.
<<results='asis'>>=
print(tli.table, include.colnames = FALSE,
      hline.after = c(0,nrow(tli.table)))
@

\subsubsection{Suppress row and column names}
<<results='asis'>>=
print(tli.table, include.colnames = FALSE, include.rownames = FALSE)
@

\newpage

\subsection{Rotate row/column names}
The \code{rotate.rownames} and \code{rotate.colnames} arguments can be
used to rotate the row and/or column names. This requires
\verb|\usepackage{rotating}| in the \LaTeX\ preamble.

<<results='asis'>>=
print(tli.table, rotate.rownames = TRUE, rotate.colnames = TRUE)
@

\newpage

\subsection{Horizontal lines}
\subsubsection{Line locations}
Use the \code{hline.after} argument to specify the position of the
horizontal lines.

<<results='asis'>>=
print(xtable(anova(fm3)), hline.after = c(1))
@

\subsubsection{Line styles}
Specifying \code{booktabs = TRUE} will generate three line types. By
default, when no value is given for \code{hline.after}, a
\verb|\toprule| will be drawn above the table, a \verb|\midrule| after
the table headings and a \verb|\bottomrule| below the table. This
requires \verb|\usepackage{booktabs}| in the \LaTeX\ preamble.

\p

The top and bottom rules are slightly thicker than the mid rule. The
thickness of the lines can be set via the \LaTeX\ lengths
\verb|\heavyrulewidth| and \verb|\lightrulewidth|.

<<results='asis'>>=
tli.table <- xtable(tli[1:10, ])
print(tli.table, include.rownames = FALSE, booktabs = TRUE)
@

\p

If \code{hline.after} includes \code{-1}, a \verb|\toprule| will be
drawn above the table. If \code{hline.after} includes the number of
rows in the table, a \verb|\bottomrule| will be drawn below the
table. For any other values specified in \code{hline.after}, a
\verb|\midrule| will be drawn after that line of the table.

\p
The following table has more than one \verb|\midrule|.

<<results='asis'>>=
bktbs <- xtable(matrix(1:10, ncol = 2))
hlines <- c(-1, 0, 1, nrow(bktbs))
print(bktbs, booktabs = TRUE, hline.after = hlines)
@

\subsection{Table level commands}
<<results='asis'>>=
print(xtable(anova(fm3)), size = "large")
@

\p
<<results='asis'>>=
print(xtable(anova(fm3)), size = "\\setlength{\\tabcolsep}{12pt}")
@

\subsection{Long tables}
Requires \verb|\usepackage{longtable}| in the \LaTeX\ preamble.

<<results='asis'>>=
x <- matrix(rnorm(1000), ncol = 10)
x.big <- xtable(x, caption = "A \\code{longtable} spanning several pages")
print(x.big, hline.after=c(-1, 0), tabular.environment = "longtable")
@

Extra features of the \pkg{longtable} \LaTeX{} package can typically
be activated using \code{add.to.row}, as shown below.

<<results='asis'>>=
add.to.row <- list(pos = list(0), command = NULL)
command <- paste0("\\hline\n\\endhead\n",
                  "\\hline\n",
                  "\\multicolumn{", dim(x)[2] + 1, "}{l}",
                  "{\\footnotesize Continued on next page}\n",
                  "\\endfoot\n",
                  "\\endlastfoot\n")
add.to.row$command <- command
print(x.big, hline.after=c(-1), add.to.row = add.to.row,
      tabular.environment = "longtable")
@


\newpage

\subsection{Use of \code{add.to.row} argument}
The following frequency table has outer dimnames: \code{Grade3} and
\code{Grade6}.

<<>>=
Grade3 <- c("A","B","B","A","B","C","C","D","A","B",
            "C","C","C","D","B","B","D","C","C","D")
Grade6 <- c("A","A","A","B","B","B","B","B","C","C",
            "A","C","C","C","D","D","D","D","D","D")
Cohort <- table(Grade3, Grade6)
Cohort
@

\p
The default behavior of \code{print.xtable} is to strip outer dimnames.
<<results='asis'>>=
xtable(Cohort)
@

\p
The desired column labels can be created using \code{add.to.row}, in this case
applying two commands to ``row number zero'' while suppressing the basic column
names.

<<results='asis'>>=
addtorow <- list()
addtorow$pos <- list(0, 0)
addtorow$command <- c("& \\multicolumn{4}{c}{Grade 6} \\\\\n",
                      "Grade 3 & A & B & C & D \\\\\n")
print(xtable(Cohort), add.to.row = addtorow, include.colnames = FALSE)
@

\subsection{Sideways tables}
Requires \verb|\usepackage{rotating}| in the \LaTeX\
preamble.  Sideways tables can't be forced in place with the \code{[H]}
specifier, but you can use the \verb|\clearpage| command to get them
fairly nearby.

<<results='asis'>>=
x <- x[1:30, ]
x.side <- xtable(x, caption = "A sideways table")
print(x.side, floating = TRUE, floating.environment = "sidewaystable")
@
\clearpage

\subsection{Rescaled tables}
Specify a \code{scalebox} value to rescale the table.
<<results='asis'>>=
x <- x[1:20, ]
x.rescale <- xtable(x)
print(x.rescale, scalebox = 0.7)
@

\subsection{Aligning fixed width columns}
Note that using specifications such as \verb|p{2cm}| always
produces a \textbf{left aligned} column. What if some other alignment
is desired?

This is not really a problem with \pkg{xtable} but with the formatting
of tables with fixed width columns and different alignments using
standard \LaTeX.

One solution is to use the \verb|array| package, defining new
column formats.

\begin{verbatim}
\newcolumntype{L}[1]{>{\raggedright\let\newline\\
    \arraybackslash\hspace{0pt}}m{#1}}
\newcolumntype{C}[1]{>{\centering\let\newline\\
    \arraybackslash\hspace{0pt}}m{#1}}
\newcolumntype{R}[1]{>{\raggedleft\let\newline\\
    \arraybackslash\hspace{0pt}}m{#1}}
\newcolumntype{P}[1]{>{\raggedright\tabularxbackslash}p{#1}}
\end{verbatim}

These allow for very sophisticated cell formatting, namely
left-aligned, centred, or right-aligned text, with recognition of line
breaks for the first three new column types. If these lines are
included along with \verb|\usepackage{array}|, then the following is
possible.

\newcolumntype{L}[1]{>{\raggedright\let\newline\\
    \arraybackslash\hspace{0pt}}m{#1}}
\newcolumntype{C}[1]{>{\centering\let\newline\\
    \arraybackslash\hspace{0pt}}m{#1}}
\newcolumntype{R}[1]{>{\raggedleft\let\newline\\
    \arraybackslash\hspace{0pt}}m{#1}}
\newcolumntype{P}[1]{>{\raggedright\tabularxbackslash}p{#1}}

<<results='asis'>>=
df <- data.frame(name = c("A","B"), right = c(1.4, 34.6),
                 left = c(1.4, 34.6), text = c("txt1","txt2"))
print(xtable(df, align = c("l", "|c", "|R{3cm}", "|L{3cm}", "| p{3cm}|")),
      floating = FALSE, include.rownames = FALSE)
@

\newpage

\subsection{Table width}
The \code{tabularx} environment is for typesetting tables whose
overall width is fixed. The column alignment code \code{X} denotes
columns that will be stretched to achieve the desired table
width. Requires \verb|\usepackage{tabularx}| in the \LaTeX\ preamble.

<<results='asis'>>=
df.width <- data.frame(One = c("item 1", "A"), Two = c("item 2", "B"),
                       Three = c("item 3", "C"), Four = c("item 4", "D"))
x.width <- xtable(df.width)
align(x.width) <- "|l|X|l|l|l|"
print(x.width, tabular.environment = "tabularx", width = "\\textwidth")
@

\subsection{Using \texttt{glue} to create captions}
First make the package \textsf{glue} available and create the desired caption

<<loadglue, message=FALSE, warning=FALSE>>=
library(glue)
now <- format(as.Date(Sys.Date()), "%d %B, %Y" )
caption <- glue("Printed on {now}")
@ %def

Then this caption will be printed.

<<printglue, results = 'asis'>>=
print(xtable(Cohort, caption = caption),
      floating = TRUE, latex.environments = "center")
@ %def


\newpage
\section{Suppressing printing}
By default the \code{print} method will print the \LaTeX\ or HTML to
standard output and also return the character strings invisibly.  The
printing to standard output can be suppressed by specifying
\code{print.results = FALSE}.

<<>>=
x.out <- print(tli.table, print.results = FALSE)
@

Formatted output can also be captured without printing with the
\code{toLatex} method.  This function returns an object of class
\code{"Latex"}.

<<>>=
x.ltx <- toLatex(tli.table)
class(x.ltx)
x.ltx
@


\newpage

\section{Acknowledgements}
Most of the examples in this gallery are taken from the \pkg{xtable}
documentation. Two examples (\code{add.to.row} and `Aligning fixed width
columns') are from Stack Exchange.

\section{Session information}
<<results='asis'>>=
toLatex(sessionInfo())
@

\end{document}

%\VignetteIndexEntry{xtable Other Packages Gallery}
%\VignetteDepends{xtable}
%\VignetteKeywords{LaTeX, HTML, table}
%\VignettePackage{xtable}
% !Rnw weave = knitr
% \VignetteEngine{knitr::knitr}
%**************************************************************************
\documentclass{article}
\usepackage[a4paper, height=24cm]{geometry} % geometry first
\usepackage{array}
\usepackage{booktabs}
\usepackage{longtable}
\usepackage{parskip}
\usepackage{rotating}
\usepackage{tabularx}
\usepackage{titlesec}
\usepackage{hyperref} % hyperref last
\titleformat\subsubsection{\bfseries\itshape}{}{0pt}{}
\newcommand\p{\vspace{2ex}}
\newcommand\code[1]{\texttt{#1}}
\newcommand\pkg[1]{\textbf{#1}}
\setcounter{tocdepth}{2}
\begin{document}

\title{\bfseries\Large The Other Packages Gallery}
\author{\bfseries David J. Scott}
\maketitle

\tableofcontents

\newpage

\section{Introduction}
This document represents a test of the functions in \pkg{xtable} which
deal with other packages.

<<set, include=FALSE>>=
library(knitr)
opts_chunk$set(fig.path = 'Figures/other', debug = TRUE, echo = TRUE)
opts_chunk$set(out.width = '0.9\\textwidth')
@

The first step is to load the package and set some options for this document.
<<package, results='asis'>>=
library(xtable)
options(xtable.floating = FALSE)
options(xtable.timestamp = "")
options(width = 60)
set.seed(1234)
@

%% \section{The packages \pkg{spdep}, \pkg{splm}, and \pkg{sphet}}

%% Code for supporting these packages and most of the examples used in
%% this section was originally provided by Martin Gubri
%% (\url{martin.gubri@framasoft.org}).

%% \subsection{The package \pkg{spdep}}
%% \label{sec:package-pkgspdep}

%% First load the package and create some objects.
%% <<dataspdep>>=
%% library(spdep)
%% data("oldcol", package = "spdep")

%% data.in.sample <- COL.OLD[1:44,]
%% data.out.of.sample <- COL.OLD[45:49,]

%% listw.in.sample <- nb2listw(subset(COL.nb, !(1:49 %in% 45:49)))
%% listw.all.sample <- nb2listw(COL.nb)

%% COL.lag.eig <- lagsarlm(CRIME ~ INC + HOVAL, data = data.in.sample,
%%                         listw.in.sample)
%% class(COL.lag.eig)
%% COL.errW.GM <- GMerrorsar(CRIME ~ INC + HOVAL, data = data.in.sample,
%%                           listw.in.sample, returnHcov = TRUE)
%% class(COL.errW.GM)
%% COL.lag.stsls <- stsls(CRIME ~ INC + HOVAL, data = data.in.sample,
%%                        listw.in.sample)
%% class(COL.lag.stsls)

%% p1 <- predict(COL.lag.eig, newdata = data.out.of.sample,
%%               listw = listw.all.sample)
%% class(p1)
%% p2 <- predict(COL.lag.eig, newdata = data.out.of.sample,
%%               pred.type = "trend", type = "trend")
%% #type option for retrocompatibility with spdep 0.5-92
%% class(p2)

%% imp.exact <- impacts(COL.lag.eig, listw = listw.in.sample)
%% class(imp.exact)
%% imp.sim <- impacts(COL.lag.eig, listw = listw.in.sample, R = 200)
%% class(imp.sim)
%% @ %def


%% \subsubsection{\code{sarlm} objects}
%% \label{sec:codesarlm-objects}

%% There is an \code{xtable} method for objects of this type.
%% <<xtablesarlm, results = 'asis'>>=
%% xtable(COL.lag.eig)
%% @ %def

%% The method for \code{xtable} actually uses the summary of the object,
%% and an identical result is obtained when using the summary of the
%% object, even if the summary contains more additional information.

%% <<xtablesarlmsumm, results = 'asis'>>=
%% xtable(summary(COL.lag.eig, correlation = TRUE))
%% @ %def

%% This same pattern applies to the other objects from this group of packages.

%% Note that additional prettying of the resulting table is possible, as
%% for any table produced using \code{xtable}. For example using the
%% \pkg{booktabs} package we get:

%% <<xtablesarlmbooktabs, results = 'asis'>>=
%% print(xtable(COL.lag.eig), booktabs = TRUE)
%% @ %def

%% \subsubsection{\code{gmsar} objects}
%% \label{sec:codegmsar-objects}


%% <<xtablegmsar, results = 'asis'>>=
%% xtable(COL.errW.GM)
%% @ %def

%% \subsubsection{\code{stsls} objects}
%% \label{sec:codestsls-objects}


%% <<xtablestsls, results = 'asis'>>=
%% xtable(COL.lag.stsls)
%% @ %def

%% \subsubsection{\code{sarlm.pred} objects}
%% \label{sec:codesarlmpred-objects}

%% \code{xtable} has a method for predictions of \code{sarlm} models.

%% <<xtablesarlmpred, results = 'asis'>>=
%% xtable(p1)
%% @ %def

%% This method transforms the \code{sarlm.pred} objects into data frames,
%% allowing any number of attributes vectors which may vary according to
%% predictor types.

%% <<xtablesarlmpred2, results = 'asis'>>=
%% xtable(p2)
%% @ %def

%% \subsubsection{\code{lagImpact} objects}
%% \label{sec:codelagimpact-objects}

%% The \code{xtable} method returns the values of direct, indirect and
%% total impacts for all the variables in the model. The class
%% \code{lagImpact} has two different sets of attributes according to if
%% simulations are used. But the \code{xtable} method always returns the
%% three components of the non-simulation case.

%% <<xtablelagimpactexact, results = 'asis'>>=
%% xtable(imp.exact)
%% @ %def

%% \p
%% <<xtablelagimpactmcmc, results = 'asis'>>=
%% xtable(imp.sim)
%% @ %def


%% \subsubsection{\code{spautolm} objects}
%% \label{sec:codespautolm-objects}

%% The need for an \code{xtable} method for \code{spautolm} was expressed
%% by Guido Schulz (\url{schulzgu@student.hu-berlin.de}), who also
%% provided an example of an object of this type. The required code was
%% implemented by David Scott (\url{d.scott@auckland.ac.nz}).

%% First create an object of the required type.

%% <<minimalexample, results = 'hide'>>=
%% library(spdep)
%% example(NY_data)
%% spautolmOBJECT <- spautolm(Z ~ PEXPOSURE + PCTAGE65P,data = nydata,
%%                            listw = listw_NY, family = "SAR",
%%                            method = "eigen", verbose = TRUE)
%% summary(spautolmOBJECT, Nagelkerke = TRUE)
%% @ %def

%% \p
%% <<spautolmclass>>=
%% class(spautolmOBJECT)
%% @ %def


%% <<xtablespautolm, results = 'asis'>>=
%% xtable(spautolmOBJECT,
%%        display = c("s",rep("f", 3), "e"), digits = 4)
%% @ %def



%% \subsection{The package \pkg{splm}}
%% \label{sec:package-pkgsplm}

%% First load the package and create some objects.
%% <<datasplm>>=
%% library(splm)
%% data("Produc", package = "plm")
%% data("usaww",  package = "splm")
%% fm <- log(gsp) ~ log(pcap) + log(pc) + log(emp) + unemp
%% respatlag <- spml(fm, data = Produc, listw = mat2listw(usaww),
%%                    model="random", spatial.error="none", lag=TRUE)
%% class(respatlag)
%% GM <- spgm(log(gsp) ~ log(pcap) + log(pc) + log(emp) + unemp, data = Produc,
%%            listw = usaww, moments = "fullweights", spatial.error = TRUE)
%% class(GM)

%% imp.spml <- impacts(respatlag, listw = mat2listw(usaww, style = "W"), time = 17)
%% class(imp.spml)
%% @ %def


%% \subsubsection{\code{splm} objects}
%% \label{sec:codesplm-objects}

%% <<xtablesplm, results = 'asis'>>=
%% xtable(respatlag)
%% @ %def


%% \p
%% <<xtablesplm1, results = 'asis'>>=
%% xtable(GM)
%% @ %def



%% The \code{xtable} method works the same on impacts of \code{splm} models.

%% <<xtablesplmimpacts, results = 'asis'>>=
%% xtable(imp.spml)
%% @ %def

%% \subsection{The package \pkg{sphet}}
%% \label{sec:package-pkgsphet}

%% First load the package and create some objects.
%% <<datasphet>>=
%% library(sphet)
%% data("columbus", package = "spdep")
%% listw <- nb2listw(col.gal.nb)
%% data("coldis", package = "sphet")
%% res.stsls <- stslshac(CRIME ~ HOVAL + INC, data = columbus, listw = listw,
%%                       distance = coldis, type = 'Triangular')
%% class(res.stsls)
%% @ %def


%% \subsubsection{\code{sphet} objects}
%% \label{sec:codesphet-objects}

%% <<xtablesphet, results = 'asis'>>=
%% xtable(res.stsls)
%% @ %def



\section{The \pkg{zoo} package}
\label{sec:pkgzoo-package}


<<zoo, results = 'asis'>>=
library(zoo)
xDate <- as.Date("2003-02-01") + c(1, 3, 7, 9, 14) - 1
as.ts(xDate)
x <- zoo(rnorm(5), xDate)
xtable(x)
@ %def


\p

<<zoots, results = 'asis'>>=
tempTs <- ts(cumsum(1 + round(rnorm(100), 0)),
              start = c(1954, 7), frequency = 12)
tempTable <- xtable(tempTs, digits = 0)
tempTable
tempZoo <- as.zoo(tempTs)
xtable(tempZoo, digits = 0)
@ %def


\section{The \pkg{survival} package}
\label{sec:pkgsurvival-package}


<<survival, results = 'asis'>>=
library(survival)
test1 <- list(time=c(4,3,1,1,2,2,3),
              status=c(1,1,1,0,1,1,0),
              x=c(0,2,1,1,1,0,0),
              sex=c(0,0,0,0,1,1,1))
coxFit <- coxph(Surv(time, status) ~ x + strata(sex), test1)
xtable(coxFit)
@ %def

\end{document}

%\VignetteIndexEntry{xtable Other Packages Gallery}
%\VignetteDepends{xtable}
%\VignetteKeywords{LaTeX, HTML, table}
%\VignettePackage{xtable}
% !Rnw weave = knitr
% \VignetteEngine{knitr::knitr}
%**************************************************************************
\documentclass{article}
\usepackage[a4paper, height=24cm]{geometry} % geometry first
\usepackage{array}
\usepackage{booktabs}
\usepackage{longtable}
\usepackage{parskip}
\usepackage{rotating}
\usepackage{tabularx}
\usepackage{titlesec}
\usepackage{hyperref} % hyperref last
\titleformat\subsubsection{\bfseries\itshape}{}{0pt}{}
\newcommand\p{\vspace{2ex}}
\newcommand\code[1]{\texttt{#1}}
\newcommand\pkg[1]{\textbf{#1}}
\setcounter{tocdepth}{2}
\begin{document}

\title{\bfseries\Large The Other Packages Gallery}
\author{\bfseries David J. Scott}
\maketitle

\tableofcontents

\newpage

\section{Introduction}
This document represents a test of the functions in \pkg{xtable} which
deal with other packages.

<<set, include=FALSE>>=
library(knitr)
opts_chunk$set(fig.path = 'Figures/other', debug = TRUE, echo = TRUE)
opts_chunk$set(out.width = '0.9\\textwidth')
@

The first step is to load the package and set some options for this document.
<<package, results='asis'>>=
library(xtable)
options(xtable.floating = FALSE)
options(xtable.timestamp = "")
options(width = 60)
set.seed(1234)
@

% \section{The packages \pkg{spatialreg}, \pkg{spdep}, \pkg{splm}, and \pkg{sphet}}

% Code for supporting these packages and most of the examples used in
% this section was originally provided by Martin Gubri
% (\url{martin.gubri@framasoft.org}).

% \subsection{The package \pkg{spdep}}
% \label{sec:package-pkgspdep}

% First load the package and create some objects.
% <<dataspdep>>=
% library(spdep)
% library(spatialreg, warn.conflicts = FALSE)
% data("oldcol", package = "spdep")

% data.in.sample <- COL.OLD[1:44,]
% data.out.of.sample <- COL.OLD[45:49,]

% listw.in.sample <- nb2listw(subset(COL.nb, !(1:49 %in% 45:49)))
% listw.all.sample <- nb2listw(COL.nb)

% COL.lag.eig <- lagsarlm(CRIME ~ INC + HOVAL, data = data.in.sample,
%                         listw.in.sample)
% class(COL.lag.eig)
% COL.errW.GM <- GMerrorsar(CRIME ~ INC + HOVAL, data = data.in.sample,
%                           listw.in.sample, returnHcov = TRUE)
% class(COL.errW.GM)
% COL.lag.stsls <- stsls(CRIME ~ INC + HOVAL, data = data.in.sample,
%                        listw.in.sample)
% class(COL.lag.stsls)

% p1 <- predict(COL.lag.eig, newdata = data.out.of.sample,
%               listw = listw.all.sample)
% class(p1)
% p2 <- predict(COL.lag.eig, newdata = data.out.of.sample,
%               pred.type = "trend", type = "trend")
% #type option for retrocompatibility with spdep 0.5-92
% class(p2)

% imp.exact <- impacts(COL.lag.eig, listw = listw.in.sample)
% class(imp.exact)
% imp.sim <- impacts(COL.lag.eig, listw = listw.in.sample, R = 200)
% class(imp.sim)
% @ %def


% \subsubsection{\code{sarlm} objects}
% \label{sec:codesarlm-objects}

% There is an \code{xtable} method for objects of this type.
% <<xtablesarlm, results = 'asis'>>=
% xtable(COL.lag.eig)
% @ %def

% The method for \code{xtable} actually uses the summary of the object,
% and an identical result is obtained when using the summary of the
% object, even if the summary contains more additional information.

% <<xtablesarlmsumm, results = 'asis'>>=
% xtable(summary(COL.lag.eig, correlation = TRUE))
% @ %def

% This same pattern applies to the other objects from this group of packages.

% Note that additional prettying of the resulting table is possible, as
% for any table produced using \code{xtable}. For example using the
% \pkg{booktabs} package we get:

% <<xtablesarlmbooktabs, results = 'asis'>>=
% print(xtable(COL.lag.eig), booktabs = TRUE)
% @ %def

% \subsubsection{\code{gmsar} objects}
% \label{sec:codegmsar-objects}


% <<xtablegmsar, results = 'asis'>>=
% xtable(COL.errW.GM)
% @ %def

% \subsubsection{\code{stsls} objects}
% \label{sec:codestsls-objects}


% <<xtablestsls, results = 'asis'>>=
% xtable(COL.lag.stsls)
% @ %def

% \subsubsection{\code{sarlm.pred} objects}
% \label{sec:codesarlmpred-objects}

% \code{xtable} has a method for predictions of \code{sarlm} models.

% <<xtablesarlmpred, results = 'asis'>>=
% xtable(p1)
% @ %def

% This method transforms the \code{sarlm.pred} objects into data frames,
% allowing any number of attributes vectors which may vary according to
% predictor types.

% <<xtablesarlmpred2, results = 'asis'>>=
% xtable(p2)
% @ %def

% \subsubsection{\code{lagImpact} objects}
% \label{sec:codelagimpact-objects}

% The \code{xtable} method returns the values of direct, indirect and
% total impacts for all the variables in the model. The class
% \code{lagImpact} has two different sets of attributes according to if
% simulations are used. But the \code{xtable} method always returns the
% three components of the non-simulation case.

% <<xtablelagimpactexact, results = 'asis'>>=
% xtable(imp.exact)
% @ %def

% \p
% <<xtablelagimpactmcmc, results = 'asis'>>=
% xtable(imp.sim)
% @ %def


% \subsubsection{\code{spautolm} objects}
% \label{sec:codespautolm-objects}

% The need for an \code{xtable} method for \code{spautolm} was expressed
% by Guido Schulz (\url{schulzgu@student.hu-berlin.de}), who also
% provided an example of an object of this type. The required code was
% implemented by David Scott (\url{d.scott@auckland.ac.nz}).

% First create an object of the required type.

% <<minimalexample, results = 'hide'>>=
% library(spdep)
% example(NY_data)
% spautolmOBJECT <- spautolm(Z ~ PEXPOSURE + PCTAGE65P,data = nydata,
%                            listw = listw_NY, family = "SAR",
%                            method = "eigen", verbose = TRUE)
% summary(spautolmOBJECT, Nagelkerke = TRUE)
% @ %def

% \p
% <<spautolmclass>>=
% class(spautolmOBJECT)
% @ %def


% <<xtablespautolm, results = 'asis'>>=
% xtable(spautolmOBJECT,
%        display = c("s",rep("f", 3), "e"), digits = 4)
% @ %def



% \subsection{The package \pkg{splm}}
% \label{sec:package-pkgsplm}

% First load the package and create some objects.
% <<datasplm>>=
% library(splm)
% data("Produc", package = "plm")
% data("usaww",  package = "splm")
% fm <- log(gsp) ~ log(pcap) + log(pc) + log(emp) + unemp
% ## the two standard specifications (SEM and SAR) one with FE
% ## and the other with RE:
% ## fixed effects panel with spatial errors
% fespaterr <- spml(fm, data = Produc, listw = mat2listw(usaww),
%                    model="within", spatial.error="b", Hess = FALSE)
% summary(fespaterr)
% ## random effects panel with spatial lag
% respatlag <- spml(fm, data = Produc, listw = mat2listw(usaww),
%                    model="random", spatial.error="none", lag=TRUE)
% summary(respatlag)
% ## calculate impact measures
% impac1 <- splm:::impacts.splm(respatlag, 
%                               listw = mat2listw(usaww, style = "W"), 
%                               time = 17)
% summary(impac1, zstats = TRUE, short = TRUE)

% GM <- spgm(log(gsp) ~ log(pcap) + log(pc) + log(emp) + unemp, data = Produc,
%            listw = usaww, moments = "fullweights", spatial.error = TRUE)
% class(GM)
% @ %def


% \subsubsection{\code{splm} objects}
% \label{sec:codesplm-objects}

% <<xtablesplm, results = 'asis'>>=
% xtable(respatlag)
% @ %def


% \p
% <<xtablegm, results = 'asis'>>=
% xtable(GM)
% @ %def



% The \code{xtable} method works the same on impacts of \code{splm} models.

% <<xtablesplmimpacts, results = 'asis'>>=
% xtable(impac1)
% @ %def

% \subsection{The package \pkg{sphet}}
% \label{sec:package-pkgsphet}

% First load the package and create some objects.
% <<datasphet>>=
% library(sphet)
% data("columbus", package = "spdep")
% listw <- nb2listw(col.gal.nb)
% data("coldis", package = "sphet")
% res.stsls <- stslshac(CRIME ~ HOVAL + INC, data = columbus, listw = listw,
%                       distance = coldis, type = 'Triangular')
% class(res.stsls)
% @ %def


% \subsubsection{\code{sphet} objects}
% \label{sec:codesphet-objects}

% <<xtablesphet, results = 'asis'>>=
% xtable(res.stsls)
% @ %def



\section{The \pkg{zoo} package}
\label{sec:pkgzoo-package}


<<zoo, results = 'asis'>>=
library(zoo)
xDate <- as.Date("2003-02-01") + c(1, 3, 7, 9, 14) - 1
as.ts(xDate)
x <- zoo(rnorm(5), xDate)
xtable(x)
@ %def


\p

<<zoots, results = 'asis'>>=
tempTs <- ts(cumsum(1 + round(rnorm(100), 0)),
              start = c(1954, 7), frequency = 12)
tempTable <- xtable(tempTs, digits = 0)
tempTable
tempZoo <- as.zoo(tempTs)
xtable(tempZoo, digits = 0)
@ %def


\section{The \pkg{survival} package}
\label{sec:pkgsurvival-package}


<<survival, results = 'asis'>>=
library(survival)
test1 <- list(time=c(4,3,1,1,2,2,3),
              status=c(1,1,1,0,1,1,0),
              x=c(0,2,1,1,1,0,0),
              sex=c(0,0,0,0,1,1,1))
coxFit <- coxph(Surv(time, status) ~ x + strata(sex), test1)
xtable(coxFit)
@ %def

\end{document}

%\VignetteIndexEntry{xtable margintable}
%\VignetteKeywords{LaTeX, HTML, table, margintable}
% !Rnw weave = knitr
% \VignetteEngine{knitr::knitr}

\documentclass{tufte-handout}


\title{
A Margin Table Example
}
\author{David J. Scott}



<<include=FALSE>>=
library(knitr)
@

\usepackage{rotating}
\usepackage{longtable}
\usepackage{booktabs}
\usepackage{tabularx}
\usepackage{hyperref}
\usepackage{fancyvrb}
\fvset{fontsize=\normalsize}

\begin{document}

\section{The Example}
\label{sec:example}

This document uses the Tufte handout \LaTeX\ document style. This is
specified by the use of the document class as
\Verb|\documentclass{tufte-handout}|.

The Tufte-\LaTeX\ document classes define a style similar to the
style Edward Tufte uses in his books and handouts.  Tufte's style is known
for its extensive use of sidenotes, tight integration of graphics with
text, and well-set typography.

One of the most prominent and distinctive features of this style is the
extensive use of sidenotes.  There is a wide margin to provide ample room
for sidenotes and small figures.  Any \Verb|\footnote|s will automatically
be converted to sidenotes.

Here is an example of a margin table, suggested by Felix Sch\"onbrodt
\Verb|<nicebread@gmx.net>|. I am not sure about its usefullness in
practice however. Note that \Verb|latex.environments = ""| avoids the
default option of tables being centered, which in this example would
lead to the caption not being directly under the table. Most
importantly, \Verb|table.placement = NULL| is required to ensure that
the default table placement options \Verb|[ht]| being inserted after
\Verb|\begin{margintable}|. This is necessary because the margintable
  environment does not allow placement options like \Verb|[ht]|.

<<>>=
library(xtable)
x <- matrix(rnorm(6), ncol = 2)
x.small <- xtable(x, label = 'tabsmall', caption = 'A margin table')
@

<<results='asis'>>=
print(x.small,floating.environment='margintable',
      latex.environments = "",
      table.placement = NULL)
@

\end{document}

%\VignetteIndexEntry{xtable List of Tables Gallery}
%\VignetteDepends{xtable}
%\VignetteKeywords{LaTeX, HTML, table}
%\VignettePackage{xtable}
% !Rnw weave = knitr
% \VignetteEngine{knitr::knitr}
%**************************************************************************
\documentclass{article}
\usepackage[a4paper,height=24cm]{geometry} % geometry first
\usepackage{array}
\usepackage{booktabs}
\usepackage{longtable}
\usepackage{parskip}
\usepackage{rotating}
\usepackage{tabularx}
\usepackage{titlesec}
\usepackage{hyperref} % hyperref last
\titleformat\subsubsection{\bfseries\itshape}{}{0pt}{}
\newcommand\p{\vspace{2ex}}
\newcommand\code[1]{\texttt{#1}}
\newcommand\pkg[1]{\textbf{#1}}
\setcounter{tocdepth}{2}
\begin{document}

\title{\bfseries\Large The \code{xtableList} Gallery}
\author{\bfseries David J. Scott}
\maketitle

\tableofcontents

\newpage

\section{Introduction}
This document represents a test of the functions in \pkg{xtable} which
deal with lists of dataframes.

<<set, include=FALSE>>=
library(knitr)
opts_chunk$set(fig.path='Figures/list', debug=TRUE, echo=TRUE)
opts_chunk$set(out.width='0.9\\textwidth')
@

The first step is to load the package and set some options for this document.
<<package, results='asis'>>=
library(xtable)
options(xtable.floating = FALSE)
options(xtable.timestamp = "")
options(width = 60)
@


Next we create a list of dataframes with attributes.

<<data>>=
require(xtable)
data(mtcars)
mtcars <- mtcars[, 1:6]
mtcarsList <- split(mtcars, f = mtcars$cyl)
### Reduce the size of the list elements
mtcarsList[[1]] <- mtcarsList[[1]][1,]
mtcarsList[[2]] <- mtcarsList[[2]][1:2,]
mtcarsList[[3]] <- mtcarsList[[3]][1:3,]
attr(mtcarsList, "subheadings") <- paste0("Number of cylinders = ",
                                          names(mtcarsList))
attr(mtcarsList, "message") <- c("Line 1 of Message",
                                 "Line 2 of Message")
str(mtcarsList)
attributes(mtcarsList)
@ %def

Now create a list of \code{xtable} objects.


<<xtablelist>>=
xList <- xtableList(mtcarsList)
str(xList)
@ %def

Create an alternative version where the lists have different values
for \code{digits}.


<<xtablelist1>>=
xList1 <- xtableList(mtcarsList, digits = c(0,2,0,0,0,1,2))
str(xList1)
@ %def

<<xtablelist2>>=
xList2 <- xtableList(mtcarsList, digits = c(0,2,0,0,0,1,2),
                            caption = "Caption to List",
                            label = "tbl:xtableList")
str(xList2)
@ %def

Further versions with no subheadings, and no message

<<xtablelist3>>=
attr(mtcarsList, "subheadings") <- NULL
xList3 <- xtableList(mtcarsList)
str(xList3)
@ %def

<<xtablelist4>>=
attr(mtcarsList, "message") <- NULL
xList4 <- xtableList(mtcarsList)
str(xList4)
@ %def

\newpage

\section{Single Column Names}
\label{sec:single-column-names}

Print the list of \code{xtable} objects with a single header of the
column names.

First the default.


<<singledefault, results='asis'>>=
print.xtableList(xList)
@ %def

Booktabs should work.
<<singlebooktabs, results='asis'>>=
print.xtableList(xList, booktabs = TRUE)
@ %def

With digits being specified.
<<singlebooktabs1, results='asis'>>=
print.xtableList(xList1, booktabs = TRUE)
@ %def

Row and column names, subheadings, and the message can be sanitized.

<<sanitize>>=
large <- function(x){
  paste0('{\\Large{\\bfseries ', x, '}}')
}
italic <- function(x){
  paste0('{\\emph{ ', x, '}}')
}
bold <- function(x){
  paste0('{\\bfseries ', x, '}')
}
red <- function(x){
  paste0('{\\color{red} ', x, '}')
}
@ %def


<<sanitizesingle, results='asis'>>=
print.xtableList(xList,
                 sanitize.rownames.function = italic,
                 sanitize.colnames.function = large,
                 sanitize.subheadings.function = bold,
                 sanitize.message.function = red,
                 booktabs = TRUE)
@ %def

A label and caption can be added.
<<singlecaption, results='asis'>>=
print.xtableList(xList2, floating = TRUE)
@ %def

Rotated column names?
<<singlerotated, results='asis'>>=
print.xtableList(xList, rotate.colnames = TRUE)
@ %def

No subheadings?
<<nosubheadings, results='asis'>>=
print.xtableList(xList3)
@ %def

No message?
<<nomessage, results='asis'>>=
print.xtableList(xList4)
@ %def


\section{Multiple Column Names}
\label{sec:multiple-column-names}

Print the list of \code{xtable} objects with multiple headers of the
column names.

First the default with multiple column name headers.

<<multipledefault, results='asis'>>=
print.xtableList(xList, colnames.format = "multiple")
@ %def

Using booktabs:

<<multiplebooktabs, results='asis'>>=
print.xtableList(xList, colnames.format = "multiple",
                 booktabs = TRUE)
@ %def

With sanitization.
<<sanitizemultiple, results='asis'>>=
print.xtableList(xList, colnames.format = "multiple",
                 sanitize.rownames.function = italic,
                 sanitize.colnames.function = large,
                 sanitize.subheadings.function = bold,
                 sanitize.message.function = red,
                 booktabs = TRUE)
@ %def

A label and caption can be added.
<<multiplecaption, results='asis'>>=
print.xtableList(xList2, colnames.format = "multiple",
                 floating = TRUE)
@ %def

Rotated column names?
<<multiplerotated, results='asis'>>=
print.xtableList(xList, colnames.format = "multiple",
                 rotate.colnames = TRUE)
@ %def

No subheadings?
<<multiplenosubheadings, results='asis'>>=
print.xtableList(xList3, colnames.format = "multiple")
@ %def

No message?
<<multiplenomessage, results='asis'>>=
print.xtableList(xList4, colnames.format = "multiple")
@ %def


\end{document}

\name{autoformat}
\alias{autoformat}
\alias{xalign}
\alias{xdigits}
\alias{xdisplay}
\title{Automatically Format Export Tables}
\description{
  Suggest an appropriate alignment, number of digits, and display type
  for \code{xtable}.
}
\usage{
autoformat(xtab, zap = getOption("digits"))

xalign(x, pad = TRUE)
xdigits(x, pad = TRUE, zap = getOption("digits"))
xdisplay(x, pad = TRUE)
}
\arguments{
  \item{xtab}{an object of class \code{xtable}.}
  \item{x}{a vector, matrix, or data frame.}
  \item{pad}{whether to format row names, when \code{x} is
    two-dimensional.}
  \item{zap}{the number of digits passed to \code{zapsmall}.}
}
\value{
  \code{autoformat} returns a copy of \code{xtab}, after applying
  \code{xalign}, \code{xdigits}, and \code{xdisplay}.

  \code{xalign} returns a character vector consisting of \code{"l"} and
  \code{"r"} elements, for left/right alignment.

  \code{xdigits} returns an integer vector.

  \code{xdisplay} returns a character vector of \code{"d"}, \code{"f"},
  and \code{"s"} elements, for integer/double/string display.
}
\author{Arni Magnusson.}
\seealso{
  \code{\link{xtable}}, \code{\link{align}}, \code{\link{digits}},
  \code{\link{display}}
}
\examples{
## 1  Vector
xalign(precip)
xdigits(precip)
xdisplay(precip)


## 2  Data frame
head(mtcars)
xdigits(mtcars, pad = FALSE)
xdigits(mtcars, pad = TRUE)
xalign(mtcars)
xdisplay(mtcars)


## 3  Autoformat when xtable is created
xtable(mtcars, align = xalign(mtcars), digits = xdigits(mtcars),
       display = xdisplay(mtcars))

## equivalent shortcut
xtable(mtcars, auto = TRUE)


## 4  Autoformat existing xtable
mt <- xtable(mtcars)
align(mt) <- xalign(mt)
digits(mt) <- xdigits(mt)
display(mt) <- xdisplay(mt)

## equivalent shortcut
mt <- autoformat(mt)
}
\keyword{array}
\keyword{print}

\name{sanitize}
\alias{sanitize}
\alias{sanitize.numbers}
\alias{sanitize.final}
\alias{as.is}
\alias{as.math}

\title{
  Sanitization Functions
}
\description{
  Functions for sanitizing elements of a table produced by
  \pkg{xtable}. Used for dealing with characters which have special
  meaning in the output format.
}
\usage{
sanitize(str, type = "latex")
sanitize.numbers(str, type, math.style.negative = FALSE,
                 math.style.exponents = FALSE)
sanitize.final(str, type)
as.is(str)
as.math(str, ...)
}

\arguments{
  \item{str}{A character object to be sanitized.}
  \item{type}{Type of table to produce. Possible values for \code{type}
    are \code{"latex"} or \code{"html"}.
    Default value is \code{"latex"}.}
  \item{math.style.negative}{In a LaTeX table, if \code{TRUE}, then use
    $-$ for the negative sign (as was the behavior prior to version 1.5-3).
    Default value is \code{FALSE}.}
  \item{math.style.exponents}{In a LaTeX table, if \code{TRUE} or
    \code{"$$"}, then use \verb{$5 \times 10^{5}$} for 5e5. If
    \code{"ensuremath"}, then use \verb{\ensuremath{5 \times 10^{5}}}
    for 5e5. If \code{"UTF-8"} or \code{"UTF-8"}, then use UTF-8 to
    approximate the LaTeX typsetting for 5e5.
    Default value is \code{FALSE}.}
  \item{\dots}{Additional arguments. Character strings or character
    vectors.}
}
\details{

  If \code{type} is \code{"latex"}, \code{sanitize()} will replace
  special characters such as \verb{&} and the like by strings which will
  reproduce the actual character, e.g. \verb{&} is replaced by
  \verb{\\&}.

  If \code{type} is \code{"html"}, \code{sanitize()} will replace
  special characters such as \verb{<} and the like by strings which will
  reproduce the actual character, e.g. \verb{<} is replaced by
  \verb{&lt;}.

  When \code{math.style.negative} is \code{TRUE}, and \code{type} is
  \code{"latex"}, $-$ is used for the negative sign rather than a
  simple hyphen (-). No effect when \code{type} is \code{"html"}.

  When \code{type} is \code{"latex"}, and \code{math.style.exponents}
  is \code{TRUE} or \verb{"$$"}, then use \verb{$5 \times 10^{5}$} for
  5e5. If \code{"ensuremath"}, then use \verb{\ensuremath{5 \times
  10^{5}}} for 5e5. If \code{"UTF-8"} or \code{"UTF-8"}, then use UTF-8
  to approximate the LaTeX typsetting for 5e5.

  When \code{type} is \code{"latex"} \code{sanitize.final} has no
  effect. When \code{type} is \code{"html"}, multiple spaces are
  replaced by a single space and occurrences of \code{' align="left"'}
  are eliminated.

  \code{as.is} and \code{as.math} are trivial helper functions to
  disable sanitizing and to insert a some mathematics in a string
  respectively.
}
\value{
  Returns the sanitized character object.
}

\author{
  Code was extracted from \code{print.xtable()}, in version 1.8.0 of
  \pkg{xtable}. Various authors contributed the original code: Jonathan
  Swinton <jonathan@swintons.net>, Uwe Ligges
  <ligges@statistik.uni-dortmund.de>, and probably David B. Dahl
  <dahl@stat.byu.edu>.
  \code{as.is} and \code{as.math} suggested and provided by Stefan
  Edwards <sme@iysik.com>.
}

\examples{
insane <- c("&",">", ">","_","\%","$","\\\\","#","^","~","{","}")
names(insane) <- c("Ampersand","Greater than","Less than",
                   "Underscore","Percent","Dollar",
                   "Backslash","Hash","Caret","Tilde",
                   "Left brace","Right brace")
sanitize(insane, type = "latex")
insane <- c("&",">","<")
names(insane) <- c("Ampersand","Greater than","Less than")
sanitize(insane, type = "html")
x <- rnorm(10)
sanitize.numbers(x, "latex", TRUE)
sanitize.numbers(x*10^(10), "latex", TRUE, TRUE)
sanitize.numbers(x, "html", TRUE, TRUE)
as.is(insane)
as.math("x10^10", ": mathematical expression")
}

\keyword{print }

\name{string}
\alias{string}
\alias{print.string}
\alias{is.string}
\alias{as.string}
\alias{+.string}
\title{String handling functions}
\description{Private functions for conveniently working with strings.}
\usage{
  string(text,file="",append=FALSE)
  \method{print}{string}(x,...)
  \method{+}{string}(x,y)
  as.string(x,file="",append=FALSE)
  is.string(x)
}
\arguments{
  \item{text}{A character object.}
  \item{file}{Name of the file that should receive the printed string.}
  \item{append}{Should the printed string be appended to the file?}
  \item{x}{A \code{string} object.}
  \item{y}{A \code{string} object.}
  \item{...}{Additional arguments.  (Currently ignored.)}
}
\details{
  These functions are private functions used by \code{print.xtable}.  They are
  not intended to be used elsewhere.
}
\author{
  David Dahl \email{dahl@stat.byu.edu} with contributions and
  suggestions from many others (see source code).
}
\seealso{\code{\link{print.xtable}}}
\keyword{print}

\name{xtable}
\alias{xtable}
\alias{xtable.anova}
\alias{xtable.aov}
\alias{xtable.aovlist}
\alias{xtable.data.frame}
\alias{xtable.glm}
\alias{xtable.lm}
\alias{xtable.matrix}
\alias{xtable.xtableMatharray}
\alias{xtable.prcomp}
\alias{xtable.coxph}
\alias{xtable.summary.aov}
\alias{xtable.summary.aovlist}
\alias{xtable.summary.glm}
\alias{xtable.summary.lm}
\alias{xtable.summary.prcomp}
\alias{xtable.ts}
\alias{xtable.table}
\alias{xtable.zoo}
\alias{xtable.sarlm}
\alias{xtable.summary.sarlm}
\alias{xtable.gmsar}
\alias{xtable.summary.gmsar}
\alias{xtable.stsls}
\alias{xtable.summary.stsls}
\alias{xtable.sarlm.pred}
\alias{xtable.lagImpact}
\alias{xtable.splm}
\alias{xtable.summary.splm}
\alias{xtable.sphet}
\alias{xtable.summary.sphet}
\alias{xtable.spautolm}
\alias{xtable.summary.spautolm}


\title{Create Export Tables}
\description{
  Convert an \R object to an \code{xtable} object, which can
  then be printed as a LaTeX or HTML table.
}
\usage{
xtable(x, caption = NULL, label = NULL, align = NULL, digits = NULL,
       display = NULL, auto = FALSE, ...)
}
\arguments{
  \item{x}{An \R object of class found among \code{methods(xtable)}.  See
    below on how to write additional method functions for \code{xtable}.}
  \item{caption}{Character vector of length 1 or 2 containing the
    table's caption or title.  If length is 2, the second item is the
    "short caption" used when LaTeX generates a "List of Tables". Set to
    \code{NULL} to suppress the caption.  Default value is \code{NULL}. }
  \item{label}{Character vector of length 1 containing the LaTeX label
    or HTML anchor. Set to \code{NULL} to suppress the label.  Default
    value is \code{NULL}. }
  \item{align}{Character vector of length equal to the number of columns
    of the resulting table, indicating the alignment of the corresponding
    columns.  Also, \code{"|"} may be used to produce vertical lines
    between columns in LaTeX tables, but these are effectively ignored
    when considering the required length of the supplied vector.  If a
    character vector of length one is supplied, it is split as
    \code{strsplit(align, "")[[1]]} before processing. Since the row
    names are printed in the first column, the length of \code{align} is
    one greater than \code{ncol(x)} if \code{x} is a
    \code{data.frame}. Use \code{"l"}, \code{"r"}, and \code{"c"} to
    denote left, right, and center alignment, respectively.  Use
    \code{"p{3cm}"} etc. for a LaTeX column of the specified width. For
    HTML output the \code{"p"} alignment is interpreted as \code{"l"},
    ignoring the width request. Default depends on the class of
    \code{x}. }
  \item{digits}{
    Numeric vector of length equal to one (in which case it will be
    replicated as necessary) or to the number of columns of the
    resulting table \bold{or} matrix of the same size as the resulting
    table, indicating the number of digits to display in the
    corresponding columns. Since the row names are printed in the first
    column, the length of the vector \code{digits} or the number of
    columns of the matrix \code{digits} is one greater than
    \code{ncol(x)} if \code{x} is a \code{data.frame}. Default depends
    on the class of \code{x}. If values of \code{digits} are negative, the
    corresponding values of \code{x} are displayed in scientific format
    with \code{abs(digits)} digits.}
  \item{display}{
    Character vector of length equal to the number of columns of the
    resulting table, indicating the format for the corresponding columns.
    Since the row names are printed in the first column, the length of
    \code{display} is one greater than \code{ncol(x)} if \code{x} is a
    \code{data.frame}.  These values are passed to the \code{formatC}
    function.  Use \code{"d"} (for integers), \code{"f"}, \code{"e"},
    \code{"E"}, \code{"g"}, \code{"G"}, \code{"fg"} (for reals), or
    \code{"s"} (for strings).  \code{"f"} gives numbers in the usual
    \code{xxx.xxx} format; \code{"e"} and \code{"E"} give
    \code{n.ddde+nn} or \code{n.dddE+nn} (scientific format); \code{"g"}
    and \code{"G"} put \code{x[i]} into scientific format only if it
    saves space to do so.  \code{"fg"} uses fixed format as \code{"f"},
    but \code{digits} as number of \emph{significant} digits.  Note that
    this can lead to quite long result strings.  Default depends on the
    class of \code{x}.}
  \item{auto}{
    Logical, indicating whether to apply automatic format when no value
    is passed to \code{align}, \code{digits}, or \code{display}. This
    \sQuote{autoformat} (based on \code{xalign}, \code{xdigits}, and
    \code{xdisplay}) can be useful to quickly format a typical
    \code{matrix} or \code{data.frame}. Default value is \code{FALSE}.}
  \item{...}{Additional arguments.  (Currently ignored.)}
}
\details{

  This function extracts tabular information from \code{x} and returns
  an object of class \code{"xtable"}.  The nature of the table generated
  depends on the class of \code{x}.  For example, \code{aov} objects
  produce ANOVA tables while \code{data.frame} objects produce a table
  of the entire data frame.  One can optionally provide a caption
  or label (called an anchor in HTML), as well
  as formatting specifications.  Default values for \code{align},
  \code{digits}, and \code{display} are class dependent.

  The available method functions for \code{xtable} are given by
  \code{methods(xtable)}.  Users can extend the list of available
  classes by writing methods for the generic function \code{xtable}.
  These methods functions should have \code{x} as their first argument,
  with additional arguments to specify \code{caption}, \code{label},
  \code{align}, \code{digits}, and \code{display}.  Optionally, other
  arguments may be passed to specify how the object \code{x} should be
  manipulated.  All method functions should return an object whose class
  is \code{c("xtable","data.frame")}.  The resulting object can
  have attributes \code{caption} and \code{label}, but must have
  attributes \code{align}, \code{digits}, and \code{display}.
}
\value{
  For most \code{xtable} methods, an object of class \code{"xtable"}
  which inherits the \code{data.frame} class and contains several
  additional attributes specifying the table formatting options.

}
\author{David Dahl \email{dahl@stat.byu.edu} with contributions and
  suggestions from many others (see source code).
}
\seealso{
  \code{\link{print.xtable}}, \code{\link{caption}},
  \code{\link{label}}, \code{\link{align}}, \code{\link{digits}},
  \code{\link{display}}

  \code{\link{autoformat}}, \code{\link{xalign}}, \code{\link{xdigits}},
  \code{\link{xdisplay}}

  \code{\link{xtableList}}, \code{\link{xtableMatharray}}
}
\examples{

## Load example dataset
data(tli)

## Demonstrate data.frame
tli.table <- xtable(tli[1:20, ])
print(tli.table)
print(tli.table, type = "html")
xtable(mtcars)
xtable(mtcars, auto = TRUE)

## Demonstrate data.frame with different digits in cells
tli.table <- xtable(tli[1:20, ])
display(tli.table)[c(2,6)] <- "f"
digits(tli.table) <- matrix(0:4, nrow = 20, ncol = ncol(tli)+1)
print(tli.table)
print(tli.table, type = "html")

## Demonstrate matrix
design.matrix <- model.matrix(~ sex*grade, data = tli[1:20, ])
design.table <- xtable(design.matrix, auto = TRUE)
print(design.table)
print(design.table, type = "html")

## Demonstrate aov
fm1 <- aov(tlimth ~ sex + ethnicty + grade + disadvg, data = tli)
fm1.table <- xtable(fm1)
print(fm1.table)
print(fm1.table, type = "html")

## Demonstrate lm
fm2 <- lm(tlimth ~ sex*ethnicty, data = tli)
fm2.table <- xtable(fm2)
print(fm2.table)
print(fm2.table, type = "html")
print(xtable(anova(fm2)))
print(xtable(anova(fm2)), type = "html")
fm2b <- lm(tlimth ~ ethnicty, data = tli)
print(xtable(anova(fm2b, fm2)))
print(xtable(anova(fm2b, fm2)), type = "html")

## Demonstrate glm
fm3 <- glm(disadvg ~ ethnicty*grade, data = tli, family = binomial())
fm3.table <- xtable(fm3)
print(fm3.table)
print(fm3.table, type = "html")
print(xtable(anova(fm3)))
print(xtable(anova(fm3)), type = "html")

## Demonstrate aov
## Taken from help(aov) in R 1.1.1
## From Venables and Ripley (1997) p.210.
N <- c(0,1,0,1,1,1,0,0,0,1,1,0,1,1,0,0,1,0,1,0,1,1,0,0)
P <- c(1,1,0,0,0,1,0,1,1,1,0,0,0,1,0,1,1,0,0,1,0,1,1,0)
K <- c(1,0,0,1,0,1,1,0,0,1,0,1,0,1,1,0,0,0,1,1,1,0,1,0)
yield <- c(49.5,62.8,46.8,57.0,59.8,58.5,55.5,56.0,62.8,55.8,69.5,55.0,
           62.0,48.8,45.5,44.2,52.0,51.5,49.8,48.8,57.2,59.0,53.2,56.0)
npk <- data.frame(block = gl(6,4), N = factor(N), P = factor(P),
                  K = factor(K), yield = yield)
npk.aov <- aov(yield ~ block + N*P*K, npk)
op <- options(contrasts = c("contr.helmert", "contr.treatment"))
npk.aovE <- aov(yield ~  N*P*K + Error(block), npk)
options(op)

summary(npk.aov)
print(xtable(npk.aov))
print(xtable(anova(npk.aov)))
print(xtable(summary(npk.aov)))

summary(npk.aovE)
print(xtable(npk.aovE), type = "html")
print(xtable(summary(npk.aovE)), type = "html")

## Demonstrate lm
## Taken from help(lm) in R 1.1.1
## Annette Dobson (1990) "An Introduction to Generalized Linear Models".
## Page 9: Plant Weight Data.
ctl <- c(4.17,5.58,5.18,6.11,4.50,4.61,5.17,4.53,5.33,5.14)
trt <- c(4.81,4.17,4.41,3.59,5.87,3.83,6.03,4.89,4.32,4.69)
group <- gl(2,10,20, labels = c("Ctl","Trt"))
weight <- c(ctl, trt)
lm.D9 <- lm(weight ~ group)
print(xtable(lm.D9))
print(xtable(anova(lm.D9)))

## Demonstrate glm
## Taken from help(glm) in R 1.1.1
## Annette Dobson (1990) "An Introduction to Generalized Linear Models".
## Page 93: Randomized Controlled Trial :
counts <- c(18,17,15,20,10,20,25,13,12)
outcome <- gl(3,1,9)
treatment <- gl(3,3)
d.AD <- data.frame(treatment, outcome, counts)
glm.D93 <- glm(counts ~ outcome + treatment, family = poisson())
print(xtable(glm.D93, align = "r|llrc"))
print(xtable(anova(glm.D93)), hline.after = c(1), size = "small")

## Demonstration of additional formatC() arguments.
print(fm1.table, format.args = list(big.mark = "'", decimal.mark = ","))

## Demonstration of "short caption" support.
fm1sc <- aov(tlimth ~ sex + ethnicty + grade, data = tli)
fm1sc.table <- xtable(fm1sc,
  caption = c("ANOVA Model with Predictors Sex, Ethnicity, and Grade",
    "ANOVA: Sex, Ethnicity, Grade"))
print(fm1sc.table)

## Demonstration of longtable support.
## Remember to insert \usepackage{longtable} on your LaTeX preamble
x <- matrix(rnorm(1000), ncol = 10)
x.big <- xtable(x, label = 'tabbig',
                caption = 'Example of longtable spanning several pages')
print(x.big, tabular.environment = 'longtable', floating = FALSE)
x <- x[1:30, ]
x.small <- xtable(x, label = 'tabsmall', caption = 'regular table env')
print(x.small)  # default, no longtable

## Demonstration of sidewaystable support.
## Remember to insert \usepackage{rotating} on your LaTeX preamble
print(x.small, floating.environment = 'sidewaystable')

if(require(stats, quietly = TRUE)) {
  ## Demonstrate prcomp
  ## Taken from help(prcomp) in mva package of R 1.1.1
  data(USArrests)
  pr1 <- prcomp(USArrests)
  print(xtable(pr1))
  print(xtable(summary(pr1)))

#  ## Demonstrate princomp
#  ## Taken from help(princomp) in mva package of R 1.1.1
#  pr2 <- princomp(USArrests)
#  print(xtable(pr2))
}

## Demonstrate include.rownames, include.colnames,
## only.contents and add.to.row arguments
set.seed(2345)
res <- matrix(sample(0:9, size = 6*9, replace = TRUE), ncol = 6, nrow = 9)
xres <- xtable(res)
digits(xres) <- rep(0, 7)
addtorow <- list()
addtorow$pos <- list()
addtorow$pos[[1]] <- c(0, 2)
addtorow$pos[[2]] <- 4
addtorow$command <- c('\\vspace{2mm} \n', '\\vspace{10mm} \n')
print(xres, add.to.row = addtorow, include.rownames = FALSE,
      include.colnames = TRUE, only.contents = TRUE,
      hline.after = c(0, 0, 9, 9))

## Demonstrate include.rownames, include.colnames,
## only.contents and add.to.row arguments in Rweave files

\dontrun{
 \begin{small}
 \setlongtables % For longtable version 3.x or less
 \begin{longtable}{
 <<results = tex, fig = FALSE>>=
 cat(paste(c('c', rep('cc', 34/2-1), 'c'), collapse = '@{\\hspace{2pt}}'))
 @
 }
 \hline
 \endhead
 \hline
 \endfoot
 <<results = tex, fig = FALSE>>=
 library(xtable)
 set.seed(2345)
 res <- matrix(sample(0:9, size = 34*90, replace = TRUE), ncol = 34, nrow = 90)
 xres <- xtable(res)
 digits(xres) <- rep(0, 35)
 addtorow <- list()
 addtorow$pos <- list()
 addtorow$pos[[1]] <- c(seq(4, 40, 5), seq(49, 85, 5))
 addtorow$pos[[2]] <- 45
 addtorow$command <- c('\\vspace{2mm} \n', '\\newpage \n')
 print(xres, add.to.row = addtorow, include.rownames = FALSE,
       include.colnames = FALSE, only.contents = TRUE, hline.after = NULL)
 @
 \end{longtable}
 \end{small}
}

## Demonstrate sanitization
mat <- round(matrix(c(0.9, 0.89, 200, 0.045, 2.0), c(1, 5)), 4)
rownames(mat) <- "$y_{t-1}$"
colnames(mat) <- c("$R^2$", "$\\\bar{R}^2$", "F-stat", "S.E.E", "DW")
print(xtable(mat), type = "latex", sanitize.text.function = function(x){x})

## Demonstrate booktabs
print(tli.table)
print(tli.table, hline.after = c(-1,0))
print(tli.table, hline.after = NULL)
print(tli.table,
      add.to.row = list(pos = list(2), command = c("\\vspace{2mm} \n")))

print(tli.table, booktabs = TRUE)
print(tli.table, booktabs = TRUE, hline.after = c(-1,0))
print(tli.table, booktabs = TRUE, hline.after = NULL)
print(tli.table, booktabs = TRUE,
  add.to.row = list(pos = list(2), command = c("\\vspace{2mm} \n")))
print(tli.table, booktabs = TRUE, add.to.row = list(pos = list(2),
  command = c("youhou\n")), tabular.environment = "longtable")

\testonly{
  for(i in c("latex","html")) {
    outFileName <- paste("xtable.", ifelse(i=="latex", "tex", i), sep = "")
    print(tli.table, type = i, file = outFileName, append = FALSE)
    print(design.table, type = i, file = outFileName, append = TRUE)
    print(fm1.table, type = i, file = outFileName, append = TRUE)
    print(fm2.table, type = i, file = outFileName, append = TRUE)
    print(fm2.table, type = i, file = outFileName, append = TRUE,
          math.style.negative = TRUE)
    print(xtable(anova(fm2)), type = i, file = outFileName, append = TRUE)
    print(xtable(anova(fm2b, fm2)), type = i, file = outFileName, append = TRUE)
    print(fm3.table, type = i, file = outFileName, append = TRUE)
    print(xtable(anova(fm3)), type = i, file = outFileName, append = TRUE)
    print(xtable(npk.aov), type = i, file = outFileName, append = TRUE)
    print(xtable(anova(npk.aov)), type = i, file = outFileName, append = TRUE)
    print(xtable(summary(npk.aov)), type = i, file = outFileName, append = TRUE)
    print(xtable(npk.aovE), type = i, file = outFileName, append = TRUE)
    print(xtable(summary(npk.aovE)),
          type = i, file = outFileName, append = TRUE)
    if(i=="latex") cat("\\\clearpage\n", file = outFileName, append = TRUE)
    print(xtable(lm.D9),
          type = i, file = outFileName, append = TRUE, latex.environment = NULL)
    print(xtable(lm.D9),
          type = i, file = outFileName, append = TRUE, latex.environment = "")
    print(xtable(lm.D9),
          type = i, file = outFileName, append = TRUE,
          latex.environment = "center")
    print(xtable(anova(lm.D9)), type = i, file = outFileName, append = TRUE)
    print(xtable(glm.D93), type = i, file = outFileName, append = TRUE)
    print(xtable(anova(glm.D93, test = "Chisq")),
          type = i, file = outFileName, append = TRUE)
    print(xtable(glm.D93, align = "r|llrc"),
          include.rownames = FALSE, include.colnames = TRUE,
          type = i, file = outFileName, append = TRUE)
    print(xtable(glm.D93, align = "r||llrc"),
          include.rownames = TRUE, include.colnames = FALSE,
          type = i, file = outFileName, append = TRUE)
    print(xtable(glm.D93, align = "|r||llrc"),
          include.rownames = FALSE, include.colnames = FALSE,
          type = i, file = outFileName, append = TRUE)
    print(xtable(glm.D93, align = "|r||llrc|"),
          type = i, file = outFileName, append = TRUE)
    print(xtable(anova(glm.D93)),
          hline.after = c(1), size = "small",
          type = i, file = outFileName, append = TRUE)
    if(require(stats, quietly = TRUE)) {
      print(xtable(pr1), type = i, file = outFileName, append = TRUE)
      print(xtable(summary(pr1)), type = i, file = outFileName, append = TRUE)
      # print(xtable(pr2), type = i, file = outFileName, append = TRUE)
    }
    temp.table <- xtable(ts(cumsum(1+round(rnorm(100), 2)),
                            start = c(1954, 7), frequency = 12))
    caption(temp.table) <- "Time series example"
    print(temp.table, type = i, file = outFileName,
          append = TRUE, caption.placement = "top", table.placement = "h")
    print(temp.table, type = i, file = outFileName,
          append = TRUE, caption.placement = "bottom", table.placement = "htb")
  }
}

}
\keyword{file}

\name{tli}
\alias{tli}
\title{Math scores from Texas Assessment of Academic Skills (TAAS)}
\description{
This data set contains math scores and demographic data of
100 randomly selected students participating in the Texas
Assessment of Academic Skills (TAAS).
}
\usage{data(tli)}
\format{A \code{data.frame} containing 100 observations with the
  following columns:
        \describe{
          \item{\code{grade}}{Year in school of student}
          \item{\code{sex}}{Gender of student}
          \item{\code{disadvg}}{Is the student economically disadvantaged?}
          \item{\code{ethnicty}}{Race of student}
          \item{\code{tlimth}}{Math score of student}
        }
}
\source{Texas Education Agency, \url{http://www.tea.state.tx.us}}
\keyword{datasets}

\name{print.xtableMatharray}
\alias{print.xtableMatharray}
\title{Print Math Array}
\description{
  For an object of class \code{"xtableMatharray"}, returns the LaTeX
  commands to produce an array.
}
\usage{
\method{print}{xtableMatharray}(x,
  print.results = TRUE,
  format.args = getOption("xtable.format.args", NULL),
  scalebox = getOption("xtable.scalebox", NULL),
  comment = FALSE,
  timestamp = NULL,
  ...)
}
\arguments{
  \item{x}{An object of class \code{"xtableMatharray"}.}
  \item{print.results}{If \code{TRUE}, the generated table is printed to
    standard output.  Set this to \code{FALSE} if you will just be using
    the character vector that is returned invisibly.
    Default value is \code{TRUE}.}
  \item{format.args}{List of arguments for the \code{formatC} function.
    For example, standard German number separators can be specified as
    \code{format.args=list(big.mark = "'", decimal.mark = ",")}. The
    arguments \code{digits} and \code{format} should not be included in
    this list. See details for function \code{\link{print.xtable}}.
    Default value is \code{NULL}.}
  \item{scalebox}{If not \code{NULL}, a \code{scalebox} clause will be
    added around the tabular environment with the specified value used
    as the scaling factor.
    Default value is \code{NULL}.}
  \item{comment}{If \code{TRUE}, the version and timestamp comment is
    included.  Default value is \code{FALSE}. }
  \item{timestamp}{Timestamp to include in LaTeX comment.  Set this
    to \code{NULL} to exclude the timestamp. Default value is \code{NULL}. }
  \item{...}{Additional arguments.  (Currently ignored.) }
}
\details{
 This command prints an array of numbers which may be included in a
 mathematical expression in a LaTeX document created using \pkg{Sweave}
 or \pkg{knitr}. Internally it calls \code{print.data.frame} but with
 special values for the arguments, namely that the tabular environment
 is \code{array}, row names and column names are not included, and there
 are no horizontal lines. Note that the default values for the arguments
 \code{comment} and \code{timestamp} are different to the default values
 for \code{print.xtable}, the justification being that comments would
 make the resulting LaTeX harder to read.
}
\value{
  A character vector containing the LaTeX code for incorporating an
  array in a mathematical expression.
}

\author{
  David Scott \email{d.scott@auckland.ac.nz}.
}
\seealso{
  \code{\link{xtableMatharray}}, \code{\link{print.xtable}}
}

\examples{
V <- matrix(c(1.140380e-03,  3.010497e-05,  7.334879e-05,
              3.010497e-05,  3.320683e-04, -5.284854e-05,
              7.334879e-05, -5.284854e-05,  3.520928e-04), nrow = 3)
### Simple test of print.xtableMatharray
print.xtableMatharray(xtable(V, display = rep("E", 4)))

class(V) <- c("xtableMatharray")
class(V)

### Test without any additional arguments
mth <- xtableMatharray(V)
str(mth)
print(mth)

### Test with arguments to xtable
mth <- xtableMatharray(V, display = rep("E", 4))
str(mth)
print(mth)

mth <- xtableMatharray(V, digits = 6)
str(mth)
print(mth)

### Test with additional print.xtableMatharray arguments
mth <- xtableMatharray(V, digits = 6)
str(mth)
print(mth, format.args = list(decimal.mark = ","))
print(mth, scalebox = 0.5)
print(mth, comment = TRUE)
print(mth, timestamp = "2000-01-01")
print(mth, comment = TRUE, timestamp = "2000-01-01")
}

\keyword{print}

\name{toLatex.xtable}
\alias{toLatex.xtable}
\title{Convert Table to Latex}
\description{
  Function creating a LaTeX representation of an object of class
  \code{xtable}.
}
\usage{
\method{toLatex}{xtable}(object, ...)
}
\arguments{
  \item{object}{An object of class \code{"xtable"}.}
  \item{...}{Other arguments to \code{print.xtable}.}  
}
\details{
  This function creates a LaTeX representation of an object of class
  \code{"xtable"}.  This is a method for the generic \code{"toLatex"} in
  the core R package \code{"utils"}.
}
\author{
  Charles Roosen \email{roosen@gmail.com} with contributions and
  suggestions from many others (see source code).
}
\seealso{\code{\link{print.xtable}}}

\keyword{toLatex}

\name{table.attributes}
\alias{caption}
\alias{caption.xtable}
\alias{caption<-}
\alias{caption<-.xtable}
\alias{label}
\alias{label.xtable}
\alias{label<-}
\alias{label<-.xtable}
\alias{align}
\alias{align.xtable}
\alias{align<-}
\alias{align<-.xtable}
\alias{digits}
\alias{digits.xtable}
\alias{digits<-}
\alias{digits<-.xtable}
\alias{display}
\alias{display.xtable}
\alias{display<-}
\alias{display<-.xtable}
\title{Retrieve and Set Options for Export Tables}
\description{Functions retrieving or setting table attributes for the supplied object of class \code{"xtable"}.}
\usage{
  caption(x,...)
  caption(x) <- value
  label(x,...)
  label(x) <- value
  align(x,...)
  align(x) <- value
  digits(x,...)
  digits(x) <- value
  display(x,...)
  display(x) <- value
}
\arguments{
  \item{x}{An \code{"xtable"} object.}
  \item{value}{The value of the corresponding attribute.}
  \item{...}{Additional arguments.  (Currently ignored.)}
}
\details{
  These functions retrieve or set table attributes of the object \code{x} of class \code{"xtable"}.  See
  \code{xtable} for a description of the options.
}
\author{David Dahl \email{dahl@stat.byu.edu} with contributions and suggestions from many others (see source code).}
\seealso{
  \code{\link{xtable}}, \code{\link{print.xtable}}

  \code{\link{autoformat}}, \code{\link{xalign}}, \code{\link{xdigits}},
  \code{\link{xdisplay}}
}
\keyword{print}

\name{print.xtable}
\alias{print.xtable}
\title{Print Export Tables}
\description{
  Function returning and displaying or writing to disk the LaTeX or HTML
  code associated with the supplied object of class \code{xtable}.
}
\usage{
\method{print}{xtable}(x,
  type = getOption("xtable.type", "latex"),
  file = getOption("xtable.file", ""),
  append = getOption("xtable.append", FALSE),
  floating = getOption("xtable.floating", TRUE),
  floating.environment = getOption("xtable.floating.environment", "table"),
  table.placement = getOption("xtable.table.placement", "ht"),
  caption.placement = getOption("xtable.caption.placement", "bottom"),
  caption.width = getOption("xtable.caption.width", NULL),
  latex.environments = getOption("xtable.latex.environments", c("center")),
  tabular.environment = getOption("xtable.tabular.environment", "tabular"),
  size = getOption("xtable.size", NULL),
  hline.after = getOption("xtable.hline.after", c(-1,0,nrow(x))),
  NA.string = getOption("xtable.NA.string", ""),
  include.rownames = getOption("xtable.include.rownames", TRUE),
  include.colnames = getOption("xtable.include.colnames", TRUE),
  only.contents = getOption("xtable.only.contents", FALSE),
  add.to.row = getOption("xtable.add.to.row", NULL),
  sanitize.text.function = getOption("xtable.sanitize.text.function", NULL),
  sanitize.rownames.function = getOption("xtable.sanitize.rownames.function",
                                         sanitize.text.function),
  sanitize.colnames.function = getOption("xtable.sanitize.colnames.function",
                                         sanitize.text.function),
  math.style.negative = getOption("xtable.math.style.negative", FALSE),
  math.style.exponents = getOption("xtable.math.style.exponents", FALSE),
  html.table.attributes = getOption("xtable.html.table.attributes",
                                    "border=1"),
  print.results = getOption("xtable.print.results", TRUE),
  format.args = getOption("xtable.format.args", NULL),
  rotate.rownames = getOption("xtable.rotate.rownames", FALSE),
  rotate.colnames = getOption("xtable.rotate.colnames", FALSE),
  booktabs = getOption("xtable.booktabs", FALSE),
  scalebox = getOption("xtable.scalebox", NULL),
  width = getOption("xtable.width", NULL),
  comment = getOption("xtable.comment", TRUE),
  timestamp = getOption("xtable.timestamp", date()),
  ...)}
\arguments{
  \item{x}{An object of class \code{"xtable"}.}
  \item{type}{Type of table to produce. Possible values for \code{type}
    are \code{"latex"} or \code{"html"}.
    Default value is \code{"latex"}.}
  \item{file}{Name of file where the resulting code should be saved.  If
    \code{file=""}, output is displayed on screen.  Note that the
    function also (invisibly) returns a character vector of the results
    (which can be helpful for post-processing).
    Default value is \code{""}.}
  \item{append}{If \code{TRUE} and \code{file!=""}, code will be
    appended to \code{file} instead of overwriting \code{file}.
    Default value is \code{FALSE}.}
  \item{floating}{If \code{TRUE} and \code{type="latex"}, the resulting
    table will be a floating table (using, for example,
    \code{\\begin\{table\}} and \code{\\end\{table\}}).  See
    \code{floating.environment} below.
    Default value is \code{TRUE}. }
  \item{floating.environment}{If \code{floating=TRUE} and
    \code{type="latex"}, the resulting table uses the specified floating
    environment. Possible values include \code{"table"}, \code{"table*"},
    and other floating environments defined in LaTeX packages.
    Default value is \code{"table"}.}
  \item{table.placement}{If \code{floating=TRUE} and
    \code{type="latex"}, the floating table will have placement given by
    \code{table.placement} where \code{table.placement} must be
    \code{NULL} or contain only elements of
    \{"h","t","b","p","!","H"\}.
    Default value is \code{"ht"}.}
  \item{caption.placement}{The caption will be placed at the bottom
    of the table if \code{caption.placement} is \code{"bottom"} and at
    the top of the table if it equals \code{"top"}.
    Default value is \code{"bottom"}.}
  \item{caption.width}{The caption will be placed in a \code{"parbox"}
    of the specified width if \code{caption.width} is not \code{NULL} and
	\code{type="latex"}. Default value is \code{NULL}.}
  \item{latex.environments}{If \code{floating=TRUE} and
    \code{type="latex"}, the specified LaTeX environments (provided as
    a character vector) will enclose the tabular environment.
    Default value is \code{"center"}. }
  \item{tabular.environment}{When \code{type="latex"}, the tabular
    environment that will be used.
    When working with tables that extend more than one page, using
    \code{tabular.environment="longtable"} with the corresponding
    LaTeX package (see Fairbairns, 2005) allows one to typeset them
    uniformly. Note that \code{floating} should be set to
    \code{FALSE} when using the \code{longtable} environment.
    Default value is \code{"tabular"}.}
  \item{size}{A character vector that is inserted just before the
    tabular environment starts. This can be used to set the font size
    and a variety of other table settings. Initial backslashes are
    automatically prefixed, if not supplied by user.
    Default value is \code{NULL}. }
  \item{hline.after}{When \code{type="latex"}, a vector of numbers
    between -1 and \code{nrow(x)}, inclusive, indicating the rows
    after which a horizontal line should appear.  If \code{NULL} is used
    no lines are produced. Repeated values are allowed.
    Default value is \code{c(-1,0,nrow(x))} which means draw a line
    before and after the columns names and at the end of the
    table.}
  \item{NA.string}{String to be used for missing values in table
    entries.
    Default value is \code{""}.}
  \item{include.rownames}{If \code{TRUE} the rows names are
    printed.
    Default value is \code{TRUE}.}
  \item{include.colnames}{If \code{TRUE} the columns names are
    printed.
    Default value is \code{TRUE}.}
  \item{only.contents}{If \code{TRUE} only the rows of the
    table are printed.
    Default value is \code{FALSE}. }
  \item{add.to.row}{A list of two components. The first component (which
    should be called 'pos') is a list that contains the position of rows on
    which extra commands should be added at the end. The second
    component (which should be called 'command') is a character vector
    of the same length as the first component, which contains the command
    that should be added at the end of the specified rows.
    Default value is \code{NULL}, i.e. do not add commands.}
  \item{sanitize.text.function}{All non-numeric entries (except row and
    column names) are sanitized in an attempt to remove characters which
    have special meaning for the output format. If
    \code{sanitize.text.function} is not \code{NULL}, it should
    be a function taking a character vector and returning one, and will
    be used for the sanitization instead of the default internal
    function.
    Default value is \code{NULL}.}
  \item{sanitize.rownames.function}{Like the
    \code{sanitize.text.function}, but applicable to row names.
    The default uses the \code{sanitize.text.function}. }
  \item{sanitize.colnames.function}{Like the
    \code{sanitize.text.function}, but applicable to column names.
    The default uses the \code{sanitize.text.function}. }
  \item{math.style.negative}{In a LaTeX table, if \code{TRUE}, then use
    $-$ for the negative sign (as was the behavior prior to version 1.5-3).
    Default value is \code{FALSE}.}
  \item{math.style.exponents}{In a LaTeX table, if \code{TRUE} or
    \code{"$$"}, then use \verb{$5 \times 10^{5}$} for 5e5. If
    \code{"ensuremath"}, then use \verb{\\ensuremath{5 \times 10^{5}}}
    for 5e5. If \code{"UTF-8"} or \code{"UTF-8"}, then use UTF-8 to
    approximate the LaTeX typsetting for 5e5.
    Default value is \code{FALSE}.}
  \item{html.table.attributes}{In an HTML table, attributes associated
    with the \code{<TABLE>} tag.
    Default value is \code{"border=1"}.}
  \item{print.results}{If \code{TRUE}, the generated table is printed to
    standard output.  Set this to \code{FALSE} if you will just be using
    the character vector that is returned invisibly.
  Default value is \code{TRUE}.}
  \item{format.args}{List of arguments for the \code{formatC} function.
    For example, standard German number separators can be specified as
    \code{format.args=list(big.mark = "'", decimal.mark =
      ","))}. The arguments \code{digits} and \code{format} should not be
    included in this list. See details.
    Default value is \code{NULL}.}
  \item{rotate.rownames}{If \code{TRUE}, the row names are displayed
    vertically in LaTeX.
    Default value is \code{FALSE}.}
  \item{rotate.colnames}{If \code{TRUE}, the column names are displayed
    vertically in LaTeX.
    Default value is \code{FALSE}.}
  \item{booktabs}{If \code{TRUE}, the \code{toprule}, \code{midrule} and
    \code{bottomrule} commands from the LaTeX "booktabs" package are used
    rather than \code{hline} for the horizontal line tags. }
  \item{scalebox}{If not \code{NULL}, a \code{scalebox} clause will be
    added around the tabular environment with the specified value used
    as the scaling factor.
    Default value is \code{NULL}.}
  \item{width}{If not \code{NULL}, the specified value is included in
    parentheses between the tabular environment \code{begin} tag and the
    alignment specification.  This allows specification of the table
    width when using tabular environments such as \code{tabular*} and
    \code{tabularx}.  Note that table width specification is not
    supported with the \code{tabular} or \code{longtable} environments.
    Default value is \code{NULL}.}
  \item{comment}{If \code{TRUE}, the version and timestamp comment is
    included.  Default value is \code{TRUE}. }
  \item{timestamp}{Timestamp to include in LaTeX comment.  Set this
    to \code{NULL} to exclude the timestamp. Default value is
    \code{date()}. }
  \item{...}{Additional arguments.  (Currently ignored.) }
}
\details{
  This function displays or writes to disk the code to produce a table
  associated with an object \code{x} of class \code{"xtable"}.
  The resulting code is either a LaTeX or HTML table, depending on the
  value of \code{type}.  The function also (invisibly) returns a
  character vector of the results (which can be helpful for
  post-processing).

  Since version 1.4 the non default behavior of \code{hline.after} is
  changed. To obtain the same results as the previous versions add to
  the \code{hline.after} vector the vector \code{c(-1, 0, nrow(x))}
  where \code{nrow(x)} is the numbers of rows of the object.

  From version 1.4-3, all non-numeric columns are sanitized, and all
  LaTeX special characters are sanitized for LaTeX output.  See Section
  3 of the \code{xtableGallery} vignette for an example of customizing
  the sanitization. From version 1.4-4, the sanitization also applies to
  column names.  To remove any text sanitization, specify
  \code{sanitize.text.function=function(x){x}}.

  From version 1.6-1 the default values for the arguments other than
  \code{x} are obtained using \code{getOption()}.  Hence the user can
  set the values once with \code{options()} rather than setting them in
  every call to \code{print.xtable()}.

  The argument \code{format.args} is used to supply arguments to the
  \code{formatC} function, but will throw an error if values for
  \code{digits} or \code{format} are included in the list of
  arguments. The recommended approach to specify \code{digits} is to supply
  the argument \code{digits} to \code{xtable}, and to specify
  \code{format} supply the argument \code{display} to \code{xtable}. See
  the examples.
}
\author{
  David Dahl \email{dahl@stat.byu.edu} with contributions and
  suggestions from many others (see source code).
}
\references{
  Fairbairns, Robin (2005) \emph{Tables longer than a single page.} The
  UK List of TeX Frequently Asked Questions on the
  Web. \url{http://www.tex.ac.uk/cgi-bin/texfaq2html?label=longtab}
}
\seealso{
  \code{\link{xtable}}, \code{\link{caption}}, \code{\link{label}},
  \code{\link{align}}, \code{\link{digits}}, \code{\link{display}},
  \code{\link{formatC}}
}

\examples{
df <- data.frame(A = c(1.00123, 33.1, 6),
                 B = c(111111, 3333333, 3123.233))
## The following code gives the error
## formal argument "digits" matched by multiple actual arguments
## print(xtable(df, display = c("s","e","e")),
##       format.args = list(digits = 3, big.mark = " ", decimal.mark = ","))
## specify digits as argument to xtable instead
print(xtable(df, display = c("s","f","f"), digits = 4),
      format.args = list(big.mark = " ", decimal.mark = ","))
## The following code gives the error
## formal argument "format" matched by multiple actual arguments
## print(xtable(df, digits = 4),
##       format.args = list(format = c("s","e","e"),
##                          big.mark = " ", decimal.mark = ","))
## specify format using display argument in xtable
print(xtable(df, display = c("s","e","e"), digits = 4),
      format.args = list(big.mark = " ", decimal.mark = ","))

}

\keyword{print}

\name{xtableList}
\alias{xtableList}
\alias{print.xtableList}

\title{
  Create and Export Lists of Tables
}
\description{
  \code{xtableList} creates an object from a list of tables, which can
  be used by \code{print.xtableList} to produce a composite table
  containing the information from the individual tables.
}
\usage{
xtableList(x, caption = NULL, label = NULL,
           align = NULL, digits = NULL, display = NULL, ...)

\method{print}{xtableList}(x,
  type = getOption("xtable.type", "latex"),
  file = getOption("xtable.file", ""),
  append = getOption("xtable.append", FALSE),
  floating = getOption("xtable.floating", TRUE),
  floating.environment = getOption("xtable.floating.environment", "table"),
  table.placement = getOption("xtable.table.placement", "ht"),
  caption.placement = getOption("xtable.caption.placement", "bottom"),
  caption.width = getOption("xtable.caption.width", NULL),
  latex.environments = getOption("xtable.latex.environments", c("center")),
  tabular.environment = getOption("xtable.tabular.environment", "tabular"),
  size = getOption("xtable.size", NULL),
  hline.after = NULL,
  NA.string = getOption("xtable.NA.string", ""),
  include.rownames = getOption("xtable.include.rownames", TRUE),
  colnames.format = "single",
  only.contents = getOption("xtable.only.contents", FALSE),
  add.to.row = NULL,
  sanitize.text.function = getOption("xtable.sanitize.text.function", NULL),
  sanitize.rownames.function = getOption("xtable.sanitize.rownames.function",
                                         sanitize.text.function),
  sanitize.colnames.function = getOption("xtable.sanitize.colnames.function",
                                         sanitize.text.function),
  sanitize.subheadings.function =
    getOption("xtable.sanitize.subheadings.function",
              sanitize.text.function),
  sanitize.message.function =
    getOption("xtable.sanitize.message.function",
              sanitize.text.function),
  math.style.negative = getOption("xtable.math.style.negative", FALSE),
  math.style.exponents = getOption("xtable.math.style.exponents", FALSE),
  html.table.attributes = getOption("xtable.html.table.attributes", "border=1"),
  print.results = getOption("xtable.print.results", TRUE),
  format.args = getOption("xtable.format.args", NULL),
  rotate.rownames = getOption("xtable.rotate.rownames", FALSE),
  rotate.colnames = getOption("xtable.rotate.colnames", FALSE),
  booktabs = getOption("xtable.booktabs", FALSE),
  scalebox = getOption("xtable.scalebox", NULL),
  width = getOption("xtable.width", NULL),
  comment = getOption("xtable.comment", TRUE),
  timestamp = getOption("xtable.timestamp", date()),
  ...)
}
\arguments{
  \item{x}{
    For \code{xtableList}, a list of \R objects all of the same class,
    being a class found among \code{methods(xtable)}. The list may also
    have attributes \code{"subheadings"} and \code{"message"}. The
    attribute \code{"subheadings"} should be a character vector of the
    same length as the list \code{x}. The attribute \code{"message"}
    should be a character vector of any length.
    For \code{print.xtableList}, an object of class \code{xtableList}
    produced by a call to \code{xtableList}.}
  \item{caption}{Character vector of length 1 or 2 containing the
    table's caption or title.  If length is 2, the second item is the
    "short caption" used when LaTeX generates a "List of Tables". Set to
    \code{NULL} to suppress the caption.  Default value is \code{NULL}. }
  \item{label}{Character vector of length 1 containing the LaTeX label
    or HTML anchor. Set to \code{NULL} to suppress the label.  Default
    value is \code{NULL}. }
  \item{align}{Character vector of length equal to the number of columns
    of the resulting table, indicating the alignment of the corresponding
    columns.  Also, \code{"|"} may be used to produce vertical lines
    between columns in LaTeX tables, but these are effectively ignored
    when considering the required length of the supplied vector.  If a
    character vector of length one is supplied, it is split as
    \code{strsplit(align, "")[[1]]} before processing. Since the row
    names are printed in the first column, the length of \code{align} is
    one greater than \code{ncol(x)} if \code{x} is a
    \code{data.frame}. Use \code{"l"}, \code{"r"}, and \code{"c"} to
    denote left, right, and center alignment, respectively.  Use
    \code{"p{3cm}"} etc. for a LaTeX column of the specified width. For
    HTML output the \code{"p"} alignment is interpreted as \code{"l"},
    ignoring the width request. Default depends on the class of
    \code{x}. }
  \item{digits}{
    Either \code{NULL}, or a vector of length one, or a vector of length
    equal to the number of columns in the resulting table, indicating
    the number of digits to display in the corresponding columns, or a
    list if length equal to the number of R objects making up \code{x},
    all members being vectors of the same length, either length one or
    of length equal to the number of columns in the resulting table. See
    details for further information.}
  \item{display}{
    Either \code{NULL}, or a vector of length one, or a vector of length
    equal to the number of columns in the resulting table, indicating
    the format of the corresponding columns, or a
    list if length equal to the number of R objects making up \code{x},
    all members being vectors of the same length, either length one or
    of length equal to the number of columns in the resulting table. See
    details for further information.}
  \item{type}{Type of table to produce. Possible values for \code{type}
    are \code{"latex"} or \code{"html"}.
    Default value is \code{"latex"}.}
  \item{file}{Name of file where the resulting code should be saved.  If
    \code{file=""}, output is displayed on screen.  Note that the
    function also (invisibly) returns a character vector of the results
    (which can be helpful for post-processing).
    Default value is \code{""}.}
  \item{append}{If \code{TRUE} and \code{file!=""}, code will be
    appended to \code{file} instead of overwriting \code{file}.
    Default value is \code{FALSE}.}
  \item{floating}{If \code{TRUE} and \code{type="latex"}, the resulting
    table will be a floating table (using, for example,
    \code{\\begin\{table\}} and \code{\\end\{table\}}).  See
    \code{floating.environment} below.
    Default value is \code{TRUE}. }
  \item{floating.environment}{If \code{floating=TRUE} and
    \code{type="latex"}, the resulting table uses the specified floating
    environment. Possible values include \code{"table"}, \code{"table*"},
    and other floating environments defined in LaTeX packages.
    Default value is \code{"table"}.}
  \item{table.placement}{If \code{floating=TRUE} and
    \code{type="latex"}, the floating table will have placement given by
    \code{table.placement} where \code{table.placement} must be
    \code{NULL} or contain only elements of
    \{"h","t","b","p","!","H"\}.
    Default value is \code{"ht"}.}
  \item{caption.placement}{The caption will be placed at the bottom
    of the table if \code{caption.placement} is \code{"bottom"} and at
    the top of the table if it equals \code{"top"}.
    Default value is \code{"bottom"}.}
  \item{caption.width}{The caption will be placed in a \code{"parbox"}
    of the specified width if \code{caption.width} is not \code{NULL} and
	\code{type="latex"}. Default value is \code{NULL}.}
  \item{latex.environments}{If \code{floating=TRUE} and
    \code{type="latex"}, the specified LaTeX environments (provided as
    a character vector) will enclose the tabular environment.
    Default value is \code{"center"}. }
  \item{tabular.environment}{When \code{type="latex"}, the tabular
    environment that will be used.
    When working with tables that extend more than one page, using
    \code{tabular.environment="longtable"} with the corresponding
    LaTeX package (see Fairbairns, 2005) allows one to typeset them
    uniformly. Note that \code{floating} should be set to
    \code{FALSE} when using the \code{longtable} environment.
    Default value is \code{"tabular"}.}
  \item{size}{A character vector that is inserted just before the
    tabular environment starts. This can be used to set the font size
    and a variety of other table settings. Initial backslashes are
    automatically prefixed, if not supplied by user.
    Default value is \code{NULL}. }
  \item{hline.after}{When \code{type="latex"}, a vector of numbers
    between -1 and the number of rows in the resulting table, inclusive,
    indicating the rows after which a horizontal line should
    appear. Determining row numbers is not straightforward since some
    lines in the resulting table don't enter into the count.  The
    default depends on the value of \code{col.names.format}.}
  \item{NA.string}{String to be used for missing values in table
    entries.
    Default value is \code{""}.}
  \item{include.rownames}{If \code{TRUE} the rows names are
    printed.
    Default value is \code{TRUE}.}
  \item{colnames.format}{Either \code{"single"} or \code{"multiple"}.
    Default is \code{"single"}.}
  \item{only.contents}{If \code{TRUE} only the rows of the
    table are printed.
    Default value is \code{FALSE}. }
  \item{add.to.row}{A list of two components. The first component (which
    should be called 'pos') is a list that contains the position of rows on
    which extra commands should be added at the end. The second
    component (which should be called 'command') is a character vector
    of the same length as the first component, which contains the command
    that should be added at the end of the specified rows.
    Default value is \code{NULL}, i.e. do not add commands.}
  \item{sanitize.text.function}{All non-numeric entries (except row and
    column names) are sanitized in an attempt to remove characters which
    have special meaning for the output format. If
    \code{sanitize.text.function} is not \code{NULL}, it should
    be a function taking a character vector and returning one, and will
    be used for the sanitization instead of the default internal
    function.
    Default value is \code{NULL}.}
  \item{sanitize.rownames.function}{Like the
    \code{sanitize.text.function}, but applicable to row names.
    The default uses the \code{sanitize.text.function}. }
  \item{sanitize.colnames.function}{Like the
    \code{sanitize.text.function}, but applicable to column names.
    The default uses the \code{sanitize.text.function}. }
  \item{sanitize.subheadings.function}{Like the
    \code{sanitize.text.function}, but applicable to subheadings.
    The default uses the \code{sanitize.text.function}. }
  \item{sanitize.message.function}{Like the
    \code{sanitize.text.function}, but applicable to the message.
    The default uses the \code{sanitize.text.function}. }
  \item{math.style.negative}{In a LaTeX table, if \code{TRUE}, then use
    $-$ for the negative sign (as was the behavior prior to version 1.5-3).
    Default value is \code{FALSE}.}
  \item{math.style.exponents}{In a LaTeX table, if \code{TRUE} or
    \code{"$$"}, then use \verb{$5 \times 10^{5}$} for 5e5. If
    \code{"ensuremath"}, then use \verb{\\ensuremath{5 \times 10^{5}}}
    for 5e5. If \code{"UTF-8"} or \code{"UTF-8"}, then use UTF-8 to
    approximate the LaTeX typsetting for 5e5.
    Default value is \code{FALSE}.}
  \item{html.table.attributes}{In an HTML table, attributes associated
    with the \code{<TABLE>} tag.
    Default value is \code{"border=1"}.}
  \item{print.results}{If \code{TRUE}, the generated table is printed to
    standard output.  Set this to \code{FALSE} if you will just be using
    the character vector that is returned invisibly.
  Default value is \code{TRUE}.}
  \item{format.args}{List of arguments for the \code{formatC} function.
    For example, standard German number separators can be specified as
    \code{format.args=list(big.mark = "'", decimal.mark =
      ","))}. The arguments \code{digits} and \code{format} should not be
    included in this list. See details.
    Default value is \code{NULL}.}
  \item{rotate.rownames}{If \code{TRUE}, the row names are displayed
    vertically in LaTeX.
    Default value is \code{FALSE}.}
  \item{rotate.colnames}{If \code{TRUE}, the column names are displayed
    vertically in LaTeX.
    Default value is \code{FALSE}.}
  \item{booktabs}{If \code{TRUE}, the \code{toprule}, \code{midrule} and
    \code{bottomrule} commands from the LaTeX "booktabs" package are used
    rather than \code{hline} for the horizontal line tags. }
  \item{scalebox}{If not \code{NULL}, a \code{scalebox} clause will be
    added around the tabular environment with the specified value used
    as the scaling factor.
    Default value is \code{NULL}.}
  \item{width}{If not \code{NULL}, the specified value is included in
    parentheses between the tabular environment \code{begin} tag and the
    alignment specification.  This allows specification of the table
    width when using tabular environments such as \code{tabular*} and
    \code{tabularx}.  Note that table width specification is not
    supported with the \code{tabular} or \code{longtable} environments.
    Default value is \code{NULL}.}
  \item{comment}{If \code{TRUE}, the version and timestamp comment is
    included.  Default value is \code{TRUE}. }
  \item{timestamp}{Timestamp to include in LaTeX comment.  Set this
    to \code{NULL} to exclude the timestamp. Default value is
    \code{date()}. }
  \item{\dots}{Additional arguments.  (Currently ignored.)}
}
\details{
  \code{xtableList} produces an object suitable for printing using
  \code{print.xtableList}.

  The elements of the list \code{x} supplied to \code{xtableList} must
  all have the same structure. When these list items are submitted to
  \code{xtable} the resulting table must have the same number of columns
  with the same column names and type of data.

  The values supplied to arguments \code{digits} and \code{display},
  must be composed of elements as specified in those same arguments for
  the function \code{\link{xtable}}. See the help for
  \code{\link{xtable}} for details.

  \code{print.xtableList} produces tables in two different formats
  depending on the value of \code{col.names.format}. If
  \code{col.names.format = "single"}, the resulting table has only a
  single heading row. If \code{col.names.format = "multiple"} there is a
  heading row for each of the subtables making up the complete table.

  By default if \code{col.names.format = "single"}, there are horizontal
  lines above and below the heading row, and at the end of each
  subtable. If \code{col.names.format = "multiple"}, there are
  horizontal lines above and below each appearance of the heading row,
  and at the end of each subtable.

  If \code{"subheadings"} is not \code{NULL}, the individual elements of
  this vector (which can include newlines \verb{\\n}) produce a heading
  line or lines for the subtables. When \code{col.names.format =
    "multiple"} these subheadings appear \emph{above} the heading rows.

  If \code{"message"} is not \code{NULL} the vector produces a line or
  lines at the end of the table.

  Consult the vignette \sQuote{The \code{xtableList} Gallery} to see
  the behaviour of these functions.

  Note that at present there is no code for \code{type = "html"}.
}
\value{
  \code{xtableList} produces an object of class
  \code{"xtableList"}. An object of this class is a list of
  \code{"xtable"} objects with some additional attributes. Each element
  of the list can have a \code{"subheading"} attribute. The list can
  also have a \code{"message"} attribute.

  \code{print.xtableList} produces a character string containing LaTeX
  markup which produces a composite table in a LaTeX document.
}
\author{
  David Scott \email{d.scott@auckland.ac.nz}.
}
\seealso{
  \code{\link{xtable}}, \code{\link{print.xtable}},
  \code{\link{formatC}}

  \code{\link{caption}}, \code{\link{label}}, \code{\link{align}},
  \code{\link{digits}}, \code{\link{display}}
}
\examples{
data(mtcars)
mtcars <- mtcars[, 1:6]
mtcarsList <- split(mtcars, f = mtcars$cyl)
attr(mtcarsList, "subheadings") <- paste0("Number of cylinders = ",
                                          names(mtcarsList))
attr(mtcarsList, "message") <- c("Line 1 of Message",
                                 "Line 2 of Message")
xList <- xtableList(mtcarsList)
print.xtableList(xList)
print.xtableList(xList, colnames.format = "multiple")
}
\keyword{ print }

\name{xtable-internal}
\alias{xtableLSMeans}


\title{Internal xtable Functions}
\description{
  Internal functions for the package xtable
}
\details{
  Functions which are either not intended to be called by the user or
  are waiting to be documented.
}
\keyword{ internal }

\name{xtableFtable}
\alias{xtableFtable}
\alias{print.xtableFtable}

\title{
  Create and Export Flat Tables
}
\description{
  \code{xtableFtable} creates an object which contains information about
  a flat table which can be used by \code{print.xtableFtable} to produce
  a character string which when included in a document produces a nicely
  formatted flat table.
}
\usage{
xtableFtable(x, caption = NULL, label = NULL,
             align = NULL, digits = 0, display = NULL,
             quote = FALSE,
             method = c("non.compact", "row.compact",
                         "col.compact", "compact"),
             lsep = " $\\\\vert$ ", ...)

\method{print}{xtableFtable}(x,
  type = getOption("xtable.type", "latex"),
  file = getOption("xtable.file", ""),
  append = getOption("xtable.append", FALSE),
  floating = getOption("xtable.floating", TRUE),
  floating.environment = getOption("xtable.floating.environment", "table"),
  table.placement = getOption("xtable.table.placement", "ht"),
  caption.placement = getOption("xtable.caption.placement", "bottom"),
  caption.width = getOption("xtable.caption.width", NULL),
  latex.environments = getOption("xtable.latex.environments", c("center")),
  tabular.environment = getOption("xtable.tabular.environment", "tabular"),
  size = getOption("xtable.size", NULL),
  hline.after = getOption("xtable.hline.after", NULL),
  NA.string = getOption("xtable.NA.string", ""),
  only.contents = getOption("xtable.only.contents", FALSE),
  add.to.row = getOption("xtable.add.to.row", NULL),
  sanitize.text.function = getOption("xtable.sanitize.text.function", as.is),
  sanitize.rownames.function = getOption("xtable.sanitize.rownames.function",
                                         sanitize.text.function),
  sanitize.colnames.function = getOption("xtable.sanitize.colnames.function",
                                         sanitize.text.function),
  math.style.negative = getOption("xtable.math.style.negative", FALSE),
  math.style.exponents = getOption("xtable.math.style.exponents", FALSE),
  html.table.attributes = getOption("xtable.html.table.attributes",
                                    "border=1"),
  print.results = getOption("xtable.print.results", TRUE),
  format.args = getOption("xtable.format.args", NULL),
  rotate.rownames = getOption("xtable.rotate.rownames", FALSE),
  rotate.colnames = getOption("xtable.rotate.colnames", FALSE),
  booktabs = getOption("xtable.booktabs", FALSE),
  scalebox = getOption("xtable.scalebox", NULL),
  width = getOption("xtable.width", NULL),
  comment = getOption("xtable.comment", TRUE),
  timestamp = getOption("xtable.timestamp", date()),
  ...)
}

\arguments{
  \item{x}{For \code{xtableFtable}, an object of class
  \code{"ftable"}. For \code{print.xtableFtable}, an object of class
  \code{c("xtableFtable", "ftable")}.}
  \item{caption}{Character vector of length 1 or 2 containing the
    table's caption or title.  If length is 2, the second item is the
    "short caption" used when LaTeX generates a "List of Tables". Set to
    \code{NULL} to suppress the caption.  Default value is \code{NULL}. }
  \item{label}{Character vector of length 1 containing the LaTeX label
    or HTML anchor. Set to \code{NULL} to suppress the label.  Default
    value is \code{NULL}. }
  \item{align}{Character vector of length equal to the number of columns
    of the resulting table, indicating the alignment of the corresponding
    columns.  Also, \code{"|"} may be used to produce vertical lines
    between columns in LaTeX tables, but these are effectively ignored
    when considering the required length of the supplied vector.  If a
    character vector of length one is supplied, it is split as
    \code{strsplit(align, "")[[1]]} before processing. For a flat table,
    the number of columns is the number of columns of data, plus the
    number of row variables in the table, plus one for the row names,
    even though row names are not printed.
    Use \code{"l"}, \code{"r"}, and \code{"c"} to
    denote left, right, and center alignment, respectively.  Use
    \code{"p{3cm}"} etc. for a LaTeX column of the specified width. For
    HTML output the \code{"p"} alignment is interpreted as \code{"l"},
    ignoring the width request.
    If \code{NULL} all row variable labels will be left aligned,
    separated from the data columns by a vertical line, and all data
    columns will be right aligned. The actual length of \code{align}
    depends on the value of \code{method}.}
  \item{digits}{
    Numeric vector of length equal to one (in which case it will be
    replicated as necessary) or to the number of columns in the
    resulting table. Since data in the table consists of
    counts, the default is 0. If the value of \code{digits} is negative, the
    corresponding columns are displayed in scientific format
    with \code{abs(digits)} digits.}
  \item{display}{
    Character vector of length equal to the number of columns of the
    resulting table, indicating the format for the corresponding columns.
    These values are passed to the \code{formatC}
    function.  Use \code{"d"} (for integers), \code{"f"}, \code{"e"},
    \code{"E"}, \code{"g"}, \code{"G"}, \code{"fg"} (for reals), or
    \code{"s"} (for strings).  \code{"f"} gives numbers in the usual
    \code{xxx.xxx} format; \code{"e"} and \code{"E"} give
    \code{n.ddde+nn} or \code{n.dddE+nn} (scientific format); \code{"g"}
    and \code{"G"} put \code{x[i]} into scientific format only if it
    saves space to do so.  \code{"fg"} uses fixed format as \code{"f"},
    but \code{digits} as number of \emph{significant} digits.  Note that
    this can lead to quite long result strings.
    If \code{NULL} all row variable names and labels will have format
    \code{"s"}, and all data columns will have format \code{"d"}. The
    actual length of \code{display} depends on the value of
    \code{method}.}
  \item{quote}{A character string giving the set of quoting characters
    for \code{format.ftable} used in \code{print.xtableFtable}. To
    disable quoting altogether, use \code{quote=""}.}
  \item{method}{String specifying how the \code{"xtableFtable"} object is
    printed in the \code{print} method.  Can be abbreviated.  Available
    methods are (see the examples in \code{\link{print.ftable}}):
    \describe{
      \item{"non.compact"}{the default representation of an
        \code{"ftable"} object.}
      \item{"row.compact"}{a row-compact version without empty cells
        below the column labels.}
      \item{"col.compact"}{a column-compact version without empty cells
        to the right of the row labels.}
      \item{"compact"}{a row- and column-compact version.  This may imply
        a row and a column label sharing the same cell.  They are then
        separated by the string \code{lsep}.}
    }
  }
  \item{lsep}{Only for \code{method = "compact"}, the separation string
    for row and column labels.}
  \item{type}{Type of table to produce. Possible values for \code{type}
    are \code{"latex"} or \code{"html"}.
    Default value is \code{"latex"} and is the only type implemented so far.}
  \item{file}{Name of file where the resulting code should be saved.  If
    \code{file=""}, output is displayed on screen.  Note that the
    function also (invisibly) returns a character vector of the results
    (which can be helpful for post-processing).
    Default value is \code{""}.}
  \item{append}{If \code{TRUE} and \code{file!=""}, code will be
    appended to \code{file} instead of overwriting \code{file}.
    Default value is \code{FALSE}.}
  \item{floating}{If \code{TRUE} and \code{type="latex"}, the resulting
    table will be a floating table (using, for example,
    \code{\\begin\{table\}} and \code{\\end\{table\}}).  See
    \code{floating.environment} below.
    Default value is \code{TRUE}. }
  \item{floating.environment}{If \code{floating=TRUE} and
    \code{type="latex"}, the resulting table uses the specified floating
    environment. Possible values include \code{"table"}, \code{"table*"},
    and other floating environments defined in LaTeX packages.
    Default value is \code{"table"}.}
  \item{table.placement}{If \code{floating=TRUE} and
    \code{type="latex"}, the floating table will have placement given by
    \code{table.placement} where \code{table.placement} must be
    \code{NULL} or contain only elements of
    \{"h","t","b","p","!","H"\}.
    Default value is \code{"ht"}.}
  \item{caption.placement}{The caption will be placed at the bottom
    of the table if \code{caption.placement} is \code{"bottom"} and at
    the top of the table if it equals \code{"top"}.
    Default value is \code{"bottom"}.}
  \item{caption.width}{The caption will be placed in a \code{"parbox"}
    of the specified width if \code{caption.width} is not \code{NULL} and
	\code{type="latex"}. Default value is \code{NULL}.}
  \item{latex.environments}{If \code{floating=TRUE} and
    \code{type="latex"}, the specified LaTeX environments (provided as
    a character vector) will enclose the tabular environment.
    Default value is \code{"center"}. }
  \item{tabular.environment}{When \code{type="latex"}, the tabular
    environment that will be used.
    When working with tables that extend more than one page, using
    \code{tabular.environment="longtable"} with the corresponding
    LaTeX package (see Fairbairns, 2005) allows one to typeset them
    uniformly. Note that \code{floating} should be set to
    \code{FALSE} when using the \code{longtable} environment.
    Default value is \code{"tabular"}.}
  \item{size}{A character vector that is inserted just before the
    tabular environment starts. This can be used to set the font size
    and a variety of other table settings. Initial backslashes are
    automatically prefixed, if not supplied by user.
    Default value is \code{NULL}. }
  \item{hline.after}{When \code{type="latex"}, a vector of numbers
    between -1 and \code{nrow(x)}, inclusive, indicating the rows after
    which a horizontal line should appear.  Repeated values are
    allowed. If \code{NULL} the default is to draw a line before before
    starting the table, after the column variable names and labels, and
    at the end of the table.}
  \item{NA.string}{String to be used for missing values in table
    entries.
    Default value is \code{""}.}
 \item{only.contents}{If \code{TRUE} only the rows of the
    table are printed.
    Default value is \code{FALSE}. }
  \item{add.to.row}{A list of two components. The first component (which
    should be called 'pos') is a list that contains the position of rows on
    which extra commands should be added at the end. The second
    component (which should be called 'command') is a character vector
    of the same length as the first component, which contains the command
    that should be added at the end of the specified rows.
    Default value is \code{NULL}, i.e. do not add commands.}
  \item{sanitize.text.function}{Since the table entries are counts no
    sanitization is necessary. The default is \code{as.is}, which is the
    function which makes no changes. This also applies to the labels for
    the row and column variables since these are also part of the table
    which is printed using a call to \code{print.xtable}.}
  \item{sanitize.rownames.function}{Like the
    \code{sanitize.text.function}, but applicable to row names.
    The default uses the \code{sanitize.text.function}. }
  \item{sanitize.colnames.function}{Like the
    \code{sanitize.text.function}, but applicable to column names.
    The default uses the \code{sanitize.text.function}. }
  \item{math.style.negative}{In a LaTeX table, if \code{TRUE}, then use
    $-$ for the negative sign (as was the behavior prior to version 1.5-3).
    Default value is \code{FALSE}.}
  \item{math.style.exponents}{In a LaTeX table, if \code{TRUE} or
    \code{"$$"}, then use \verb{$5 \times 10^{5}$} for 5e5. If
    \code{"ensuremath"}, then use \verb{\\ensuremath{5 \times 10^{5}}}
    for 5e5. If \code{"UTF-8"} or \code{"UTF-8"}, then use UTF-8 to
    approximate the LaTeX typesetting for 5e5.
    Default value is \code{FALSE}.}
  \item{html.table.attributes}{In an HTML table, attributes associated
    with the \code{<TABLE>} tag.
    Default value is \code{"border=1"}.}
  \item{print.results}{If \code{TRUE}, the generated table is printed to
    standard output.  Set this to \code{FALSE} if you will just be using
    the character vector that is returned invisibly.
  Default value is \code{TRUE}.}
  \item{format.args}{List of arguments for the \code{formatC} function.
    For example, standard German number separators can be specified as
    \code{format.args=list(big.mark = "'", decimal.mark =
      ","))}. The arguments \code{digits} and \code{format} should not be
    included in this list.
    Default value is \code{NULL}.}
  \item{rotate.rownames}{If \code{TRUE}, the row names and labels, and
    column variable names are displayed vertically in LaTeX.
    Default value is \code{FALSE}.}
  \item{rotate.colnames}{If \code{TRUE}, the column names and labels,
    and row variable names are displayed vertically in LaTeX.
    Default value is \code{FALSE}.}
  \item{booktabs}{If \code{TRUE}, the \code{toprule}, \code{midrule} and
    \code{bottomrule} commands from the LaTeX "booktabs" package are used
    rather than \code{hline} for the horizontal line tags. }
  \item{scalebox}{If not \code{NULL}, a \code{scalebox} clause will be
    added around the tabular environment with the specified value used
    as the scaling factor.
    Default value is \code{NULL}.}
  \item{width}{If not \code{NULL}, the specified value is included in
    parentheses between the tabular environment \code{begin} tag and the
    alignment specification.  This allows specification of the table
    width when using tabular environments such as \code{tabular*} and
    \code{tabularx}.  Note that table width specification is not
    supported with the \code{tabular} or \code{longtable} environments.
    Default value is \code{NULL}.}
  \item{comment}{If \code{TRUE}, the version and timestamp comment is
    included.  Default value is \code{TRUE}. }
  \item{timestamp}{Timestamp to include in LaTeX comment.  Set this
    to \code{NULL} to exclude the timestamp. Default value is
    \code{date()}. }
  \item{...}{Additional arguments.  (Currently ignored.) }
}
\details{
  \code{xtableFtable} carries out some calculations to determine the
    number of rows and columns of names and labels which will be in the
    table when formatted as a flat table, which depends on the value of
    \code{method}. It uses the results of those calculations to set
    sensible values for \code{align} and \code{display} if these have
    not been supplied. It attaches attributes to the resulting object
    which specify details of the function call which are needed when
    printing the resulting object which is of class
    \code{c("xtableFtable", "ftable")}.

    \code{print.xtableFtable} uses the attributes attached to an object
    of class \code{c("xtableFtable", "ftable")} to create a suitable
    character matrix object for subsequent printing. Formatting is
    carried out by changing the class of the \code{c("xtableFtable",
    "ftable")} to \code{"ftable"} then using the generic \code{format}
    to invoke \code{format.ftable}, from the \pkg{stats} package. The
    matrix object produced is then printed via a call to
    \code{print.xtable}.

    Note that at present there is no code for \code{type = "html"}.
}
\value{
  For \code{xtableFtable} an object of class \code{c("xtableFtable",
    "ftable")}, with attributes
  \item{ftableCaption}{the value of the \code{caption} argument}
  \item{ftableLabel}{the value of the \code{label} argument}
  \item{ftableAlign}{the value of the \code{label} argument}
  \item{ftableDigits}{the value of the \code{digits} argument or the
    default value if \code{digits = NULL}}
  \item{quote}{the value of the \code{quote} argument}
  \item{ftableDisplay}{the value of the \code{display} argument or the
    default value if \code{align = NULL}}
  \item{method}{the value of the \code{method} argument}
  \item{lsep}{the value of the \code{lsep} argument}
  \item{nChars}{a vector of length 2 giving the number of character rows
    and the number of character columns}

  For \code{print.xtableFtable} a character string which will produce a
  formatted table when included in a LaTeX document.

}
\references{
  Fairbairns, Robin (2005) \emph{Tables longer than a single page.} The
  UK List of TeX Frequently Asked Questions on the
  Web. \url{http://www.tex.ac.uk/cgi-bin/texfaq2html?label=longtab}
}
\author{
  David Scott \email{d.scott@auckland.ac.nz}.
}
\note{
  The functions \code{xtableFtable} and \code{print.xtableFtable} are
  new and their behaviour may change in the future based on user
  experience and recommendations.

  It is not recommended that users change the values of \code{align},
  \code{digits} or \code{align}. First of all, alternative values have
  not been tested. Secondly, it is most likely that to determine
  appropriate values for these arguments, users will have to investigate
  the code for \code{xtableFtable} and/or \code{print.xtableFtable}.
}

\seealso{
  \code{\link{ftable}}, \code{\link{print.ftable}}

  \code{\link{xtable}}, \code{\link{caption}}, \code{\link{label}},
  \code{\link{align}}, \code{\link{digits}}, \code{\link{display}},
  \code{\link{formatC}}
}
\examples{
data(mtcars)
mtcars$cyl <- factor(mtcars$cyl, levels = c("4","6","8"),
                     labels = c("four","six","eight"))
tbl <- ftable(mtcars$cyl, mtcars$vs, mtcars$am, mtcars$gear,
              row.vars = c(2, 4),
              dnn = c("Cylinders", "V/S", "Transmission", "Gears"))
xftbl <- xtableFtable(tbl, method = "compact")
print.xtableFtable(xftbl, booktabs = TRUE)
xftbl <- xtableFtable(tbl, method = "row.compact")
print.xtableFtable(xftbl, rotate.colnames = TRUE,
                   rotate.rownames = TRUE)
}

\keyword{ category }
\keyword{ print }

\name{xtableMatharray}
\alias{xtableMatharray}

\title{
 Create LaTeX Mathematical Array
}
\description{
  Convert an array of numbers or mathematical expressions into an
 \code{xtableMatharray} object so it can be printed. A convenience
 function to enable the printing of arrays in mathematical expressions
 in LaTeX.
}
\usage{
xtableMatharray(x, caption = NULL, label = NULL, align = NULL,
                digits = NULL, display = NULL, auto = FALSE, ...)
}
\arguments{
  \item{x}{A numeric or character matrix.}
  \item{caption}{Character vector of length 1 or 2 containing the
    table's caption or title.  If length is 2, the second item is the
    "short caption" used when LaTeX generates a "List of Tables". Set to
    \code{NULL} to suppress the caption.  Default value is \code{NULL}.
    Included here only for consistency with \code{xtable} methods. Not
    expected to be of use.}
  \item{label}{Character vector of length 1 containing the LaTeX
    label. Set to \code{NULL} to suppress the label.
    Default value is \code{NULL}. }
  \item{align}{Character vector of length equal to the number of columns
    of the resulting table, indicating the alignment of the corresponding
    columns.  Also, \code{"|"} may be used to produce vertical lines
    between columns in LaTeX tables, but these are effectively ignored
    when considering the required length of the supplied vector.  If a
    character vector of length one is supplied, it is split as
    \code{strsplit(align, "")[[1]]} before processing. Since the row
    names are printed in the first column, the length of \code{align} is
    one greater than \code{ncol(x)} if \code{x} is a
    \code{data.frame}. Use \code{"l"}, \code{"r"}, and \code{"c"} to
    denote left, right, and center alignment, respectively.  Use
    \code{"p{3cm}"} etc. for a LaTeX column of the specified width. For
    HTML output the \code{"p"} alignment is interpreted as \code{"l"},
    ignoring the width request. Default depends on the class of
    \code{x}. }
  \item{digits}{Numeric vector of length equal to one (in which case it
    will be replicated as necessary) or to the number of columns of the
    resulting table \bold{or} matrix of the same size as the resulting
    table, indicating the number of digits to display in the
    corresponding columns. Since the row names are printed in the first
    column, the length of the vector \code{digits} or the number of
    columns of the matrix \code{digits} is one greater than
    \code{ncol(x)} if \code{x} is a \code{data.frame}. Default depends
    on the class of \code{x}. If values of \code{digits} are negative,
    the corresponding values of \code{x} are displayed in scientific
    format with \code{abs(digits)} digits.}
  \item{display}{
    Character vector of length equal to the number of columns of the
    resulting table, indicating the format for the corresponding columns.
    Since the row names are printed in the first column, the length of
    \code{display} is one greater than \code{ncol(x)} if \code{x} is a
    \code{data.frame}.  These values are passed to the \code{formatC}
    function.  Use \code{"d"} (for integers), \code{"f"}, \code{"e"},
    \code{"E"}, \code{"g"}, \code{"G"}, \code{"fg"} (for reals), or
    \code{"s"} (for strings).  \code{"f"} gives numbers in the usual
    \code{xxx.xxx} format; \code{"e"} and \code{"E"} give
    \code{n.ddde+nn} or \code{n.dddE+nn} (scientific format); \code{"g"}
    and \code{"G"} put \code{x[i]} into scientific format only if it
    saves space to do so.  \code{"fg"} uses fixed format as \code{"f"},
    but \code{digits} as number of \emph{significant} digits.  Note that
    this can lead to quite long result strings.  Default depends on the
    class of \code{x}.}
  \item{auto}{
    Logical, indicating whether to apply automatic format when no value
    is passed to \code{align}, \code{digits}, or \code{display}. This
    \sQuote{autoformat} (based on \code{xalign}, \code{xdigits}, and
    \code{xdisplay}) can be useful to quickly format a typical
    \code{matrix} or \code{data.frame}. Default value is \code{FALSE}.}
  \item{...}{Additional arguments.  (Currently ignored.)}
}
\details{
  This function is only usable for production of LaTeX documents, not
  HTML.

  Creates an object of class
 \code{c("xtableMatharray","xtable","data.frame")}, to ensure that it is
 printed by the print method \code{print.xtableMatharray}.
}
\value{
 An object of class \code{c("xtableMatharray","xtable","data.frame")}.
}

\author{
  David Scott <d.scott@auckland.ac.nz>
}
\seealso{
\code{\link{print.xtableMatharray}}
}
\examples{
V <- matrix(c(1.140380e-03,  3.010497e-05,  7.334879e-05,
              3.010497e-05,  3.320683e-04, -5.284854e-05,
              7.334879e-05, -5.284854e-05,  3.520928e-04), nrow = 3)
mth <- xtableMatharray(V)
class(mth)
str(mth)
unclass(mth)
}
\keyword{ print }
